Package: cts
Title: Continuous Time Autoregressive Models
Version: 1.0-26
Date: 2026-05-14
Authors@R: c(
    person("Granville", "Tunnicliffe-Wilson", role = "aut"),
    person("Zhu", "Wang", email = "zwang145@uthsc.edu", role = c("aut", "cre")),
    person("Cleve", "Moler", role = c("ctb", "cph")),
    person("Jack", "Dongarra", role = c("ctb", "cph")),
    person("Jim", "Bunch", role = c("ctb", "cph")),
    person("G. W.", "Stewart", role = c("ctb", "cph")),
    person("John", "Nash", role = "ctb"))
Description: Provides tools for fitting continuous-time autoregressive (CAR) and
    complex CAR (CZAR) models for irregularly sampled time series using an exact
    Gaussian state-space formulation and Kalman filtering/smoothing. Implements
    maximum-likelihood estimation with stable parameterizations of characteristic
    roots, model selection via AIC, residual and spectral diagnostics, forecasting
    and simulation, and extraction of fitted state estimates. Methods are
    described in Wang (2013) <doi:10.18637/jss.v053.i05>.
License: GPL (>= 2)
NeedsCompilation: yes
Packaged: 2026-05-14 15:43:20 UTC; zwang145
Author: Granville Tunnicliffe-Wilson [aut],
  Zhu Wang [aut, cre],
  Cleve Moler [ctb, cph],
  Jack Dongarra [ctb, cph],
  Jim Bunch [ctb, cph],
  G. W. Stewart [ctb, cph],
  John Nash [ctb]
Maintainer: Zhu Wang <zwang145@uthsc.edu>
Repository: CRAN
Date/Publication: 2026-05-17 18:50:02 UTC
Built: R 4.5.2; aarch64-apple-darwin20; 2026-05-17 20:07:26 UTC; unix
Archs: cts.so.dSYM

Package: cts
Title: Continuous Time Autoregressive Models
Version: 1.0-26
Date: 2026-05-14
Authors@R: c(
    person("Granville", "Tunnicliffe-Wilson", role = "aut"),
    person("Zhu", "Wang", email = "zwang145@uthsc.edu", role = c("aut", "cre")),
    person("Cleve", "Moler", role = c("ctb", "cph")),
    person("Jack", "Dongarra", role = c("ctb", "cph")),
    person("Jim", "Bunch", role = c("ctb", "cph")),
    person("G. W.", "Stewart", role = c("ctb", "cph")),
    person("John", "Nash", role = "ctb"))
Description: Provides tools for fitting continuous-time autoregressive (CAR) and
    complex CAR (CZAR) models for irregularly sampled time series using an exact
    Gaussian state-space formulation and Kalman filtering/smoothing. Implements
    maximum-likelihood estimation with stable parameterizations of characteristic
    roots, model selection via AIC, residual and spectral diagnostics, forecasting
    and simulation, and extraction of fitted state estimates. Methods are
    described in Wang (2013) <doi:10.18637/jss.v053.i05>.
License: GPL (>= 2)
NeedsCompilation: yes
Packaged: 2026-05-14 15:43:20 UTC; zwang145
Author: Granville Tunnicliffe-Wilson [aut],
  Zhu Wang [aut, cre],
  Cleve Moler [ctb, cph],
  Jack Dongarra [ctb, cph],
  Jim Bunch [ctb, cph],
  G. W. Stewart [ctb, cph],
  John Nash [ctb]
Maintainer: Zhu Wang <zwang145@uthsc.edu>
Repository: CRAN
Date/Publication: 2026-05-17 18:50:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-17 20:21:53 UTC; unix
Archs: cts.so.dSYM

useDynLib(cts)
export(car, tsdiag, factab, kalsmo, kalsmoComp, spectrum, plotSpecCar, plotSpecLs, spec.ls, car_control, spec.ci)

S3method(plot, car)
S3method(plot, predict.car)
S3method(print, car)
S3method(summary, car)
S3method(predict, car)
S3method(tsdiag, car)
S3method(spectrum, car)
S3method(AIC, car)
S3method(kalsmo, car)
S3method(print, factab)
importFrom("graphics", "abline", "lines", "matplot", "par", "plot",
     "points", "title")     
importFrom("utils", "capture.output")
importFrom("stats", "Box.test", "acf", "as.ts", "bandwidth.kernel",
     "cpgram", "df.kernel", "frequency", "is.tskernel",
     "kernapply", "na.fail", "na.pass", "nextn", "pchisq",
     "qchisq", "spec.taper")

<!-- README.md is generated from README.Rmd. Please edit that file -->
# cts

<!-- badges: start -->
<!-- badges: end -->
The goal of cts is to fit continuous time autoregressive models with the Kalman filter. See Wang (2013) <https://www.jstatsoft.org/article/view/v053i05>.

## Installation

You can install the development version of cts from [GitHub](https://github.com/) with:

``` r
# install.packages("devtools")
devtools::install_github("zhuwang46/cts")
```

## Example

This is a basic example which shows you how to solve a common problem:

``` r
library(cts)
#> 
#> Attaching package: 'cts'
#> The following objects are masked from 'package:stats':
#> 
#>     spectrum, tsdiag
## basic example code
data(V22174)
fit <- car(V22174,scale=0.2,order=7, ctrl=car_control(trace=FALSE))
summary(fit)
#> 
#> Call:
#> car(x = V22174, scale = 0.2, order = 7, ctrl = car_control(trace = FALSE))
#> 
#> Order of model = 7, sigma^2 = 1.37e-09 
#> 
#> Estimated coefficients (standard errors):
#>       phi_1 phi_2 phi_3  phi_4 phi_5  phi_6 phi_7
#> coef -0.501 0.355 0.085 -0.022 0.605 -0.371 0.483
#> S.E.  0.108 0.111 0.060  0.071 0.084  0.124 0.112
#> 
#> Estimated mean (standard error):
#> [1] 0.173
#> [1] 0.022
AIC(fit)
#> 
#> Call:
#> car(x = V22174, scale = 0.2, order = 7, ctrl = car_control(trace = FALSE))
#> 
#> Model selection statistics 
#> 
#>  order t.statistic     AIC
#>      1       -4.77  -20.78
#>      2       -4.45  -38.57
#>      3        3.25  -47.15
#>      4        2.37  -50.76
#>      5        6.11  -86.05
#>      6       -0.76  -84.63
#>      7        4.32 -101.27
factab(fit)
#> 
#> Call:
#> factab(object = fit)
#> 
#> Characteristic root of original parameterization in alpha 
#> 
#>              1               2               3               4               5  
#>  -0.006+0.058i   -0.006-0.058i   -0.029+0.300i   -0.029-0.300i   -0.030+0.135i
#>              6               7  
#>  -0.006+0.058i   -0.006-0.058i
#> 
#> Frequency 
#> 
#>     1      2      3      4      5      6      7  
#> 0.009  0.009  0.048  0.048  0.022  0.022  0.000
```

#.First.lib <- function(lib, pkg)
#        library.dynam("cts", pkg, lib)
spectrum <- function(object, ...)
    UseMethod("spectrum")

"spectrum.car" <-
function (object, frmult=1, n.freq, plot.it = TRUE, na.action = na.fail, ...) 
{
  x <- object
  ## must be a result of an AR fit
#  cn <- match(c("frequency", "spectrum"), names(x))
  cn <- match(c("b","order","scale"),names(x))
  if (any(is.na(cn))) 
    stop("object must be a car() fit")
  b <- x$b
  order <- x$order
  scale <- x$scale
  if(missing(n.freq)) n.freq <- 500
  Z <- .Fortran("cspec",
           as.double(b),
           as.integer(order),
           as.double(scale),
           as.double(frmult),
           as.integer(n.freq),
           freq=double(n.freq),
           spec=double(n.freq),
           PACKAGE="cts")
  spg.out <- list(freq = Z$freq, spec = Z$spec, method = paste("CAR (",order, ") spectrum ", sep = ""))
  class(spg.out) <- "spec.car"
  if (plot.it) {
    plotSpecCar(spg.out, ci = 0, ...)
    return(invisible(spg.out))
  }
  else return(spg.out)
}

c      SUBROUTINE RESG0
c     compare to resg1d.f file. The relationship is below;
c     In this file (resg0new.f), CPR (real part of CP), CPI (imaginary part c     of CP), they are corresponding to CPS (resg1d.f)
c     MR and MI corresponding to MS (resg1d.f)
c     NR and NI corresponding to NS (resg1d.f)
c     WPR and WPI corresponding to WPS (resg1d.f)
c     CSR and CSI corresponding to CS (resg1d.f)
c     CR and CI corresponding to CC (resg1d.f)

      subroutine resg0new
      INTEGER I,J,T
      DOUBLE PRECISION U,V,W,X,Y,Z,PRED,PEV
      DOUBLE PRECISION FSESR(5000,20),FSCVR(5000,20,20)
      DOUBLE PRECISION FSESI(5000,20),FSCVI(5000,20,20)
      DOUBLE PRECISION FSER0(5000,20),TRANR(5000,20),TRANI(5000,20)
      DOUBLE PRECISION PSESR(5000,20),PSESI(5000,20)
      DOUBLE PRECISION PSCVR(5000,20,20),PSCVI(5000,20,20)
      SAVE FSESR,FSCVR,FSESI,FSCVI,FSER0,TRANR
      SAVE TRANI,PSESR,PSESI,PSCVR,PSCVI
c      INCLUDE 'model.txt'
c      INCLUDE 'series.txt'
c      INCLUDE 'setcon.txt'
c      INCLUDE 'veccom.txt'
c      INCLUDE 'repcom.txt'
c      INCLUDE 'repar3.txt'
c      INCLUDE 'resgn1.txt'
c      INCLUDE 'resgn2.txt'
c      include 'kfasav.txt'
C*****model.txt
      INTEGER PFI,ARP,VRI,CCV,LYAP,SCC,fct
      DOUBLE PRECISION SCALE,VR,CONST,PHI(20),PRDG
      COMMON/MODEL/SCALE,VR,CONST,PHI,PRDG,PFI,ARP,VRI,CCV,LYAP,SCC,fct
      integer for, fty
      common/model/for, fty
      integer ARI, tra
      common/model/ari,tra

C*****series.txt
      CHARACTER(LEN=40) NAME
      INTEGER LEN
      DOUBLE PRECISION TIM(5000),SER(5000),TDIF(5000)
c      COMMON/SERIES/NAME,LEN,TIM,SER,TDIF
      COMMON/SERIES/TIM,SER,TDIF,NAME,LEN
C*****setcon.txt
      LOGICAL CONV,FAIL
      INTEGER NP,ITCT,PPIND
      DOUBLE PRECISION CSO,CSZ,LAM,SSOLD,GMOLD,GMNEW,SIGSQ,OLDB(22)
c      COMMON/SETCON/CONV,FAIL,NP,ITCT,PPIND,CSO,CSZ,LAM,SSOLD,
c     *GMOLD,GMNEW,SIGSQ,OLDB
      COMMON/SETCON/CSO,CSZ,LAM,SSOLD,
     *GMOLD,GMNEW,SIGSQ,OLDB,CONV,FAIL,NP,ITCT,PPIND
C*****veccom.txt
      DOUBLE PRECISION B(22),NEWB(22),PERB(22),DELB(22),
     * ERR(5000),RES(5000),PRES(22,5000),sres(5000)
      COMMON/VECCOM/B,NEWB,PERB,DELB,ERR,RES,PRES,sres
C*****repcom.txt
      LOGICAL PRPI
      INTEGER REQSW,KFSW
      DOUBLE PRECISION VOB
c      COMMON/REPCOM/PRPI,REQSW,VOB,KFSW
      COMMON/REPCOM/VOB,REQSW,KFSW,PRPI 
C*****repar3.txt
      DOUBLE PRECISION ALPHA(21),ROOTR(20),ROOTI(20),XVECR(20),
     *XVECI(20)
      COMMON/REPAR3/ALPHA,ROOTR,ROOTI,XVECR,XVECI
C*****resgn1.txt
      DOUBLE PRECISION DR(20),DI(20),MR(20),MI(20),NR(20),NI(20),
     * UR(20),UI(20),WR(20),WI(20),WPR(20),WPI(20),CR(20,20),CI(20,20),
     * CPR(20,20),CPI(20,20),CSR(20,20),CSI(20,20)
      COMMON/RESGN1/DR,DI,MR,MI,NR,NI,UR,UI,WR,WI,WPR,WPI,CR,CI,CPR,CPI,
     * CSR,CSI
C*****resgn2.txt
      DOUBLE PRECISION WK(20),VT(5000),BI(2,20,20),R(2,20,20),
     *RI(2,20,20)
c       DOUBLE PRECISION WK(20),VT(5000),BI(2,20,20),R(20,20),RI(2,20,20)
      INTEGER ERRNO1
      COMMON/RESGN2/WK,VT,BI,R,RI,ERRNO1
C*****kfasav.txt
      DOUBLE PRECISION PRE(5000),PRV(5000),FSER(5000),FVAR(5000)
     1,SSER(5000)
     1,SVAR(5000)
      DOUBLE PRECISION FSES(5000,20),FSCV(5000,20,20),SSES(5000,20)
     1,SSCV(5000,20,20),PSES(5000,20),PSCV(5000,20,20),TRAN(5000,20,20)
c      COMMON/KFASAV/PRE,PRV,FSER,FVAR,SSER,SVAR,FSES,FSCV,SSES,SSCV
c     1,PSES,PSCV,TRAN
      COMMON/KFASAV/FSES,FSCV,SSES,SSCV
     1,PSES,PSCV,TRAN,PRE,PRV,FSER,FVAR,SSER,SVAR


      DO J=1,ARP
        DO I=1,ARP
          IF(I.EQ.1)THEN
            U=CSO
            V=CSZ
          ELSE IF(I.EQ.2)THEN
            U=ROOTR(J)
            V=ROOTI(J)
            IF(ARP.GT.2)THEN
              W=U
              X=V
            END IF
          ELSE IF(I.GT.2)THEN
            CALL MULC(U,V,W,X,Y,Z)
            U=Y
            V=Z
          END IF
          R(1,I,J)=U
          R(2,I,J)=V
        END DO
      END DO
      CALL CINVERT
      DO 102 I=1,ARP
      UR(I)=BI(1,I,ARP)
      UI(I)=BI(2,I,ARP)
c     in correspondence to the change of F04ADF to zgesv in LAPACK, the resuc     lts are in BI, not RI
c      UR(I)=RI(1,I,ARP)
c      UI(I)=RI(2,I,ARP)
c     end of change by Z.W.
  102 CONTINUE
      DO 104 I=1,ARP
      WR(I)=CSZ
      WI(I)=CSZ
      DO 103 J=1,ARP
      CALL ADDC(ROOTR(I),ROOTI(I),ROOTR(J),-ROOTI(J),U,V)
      CALL MULC(-UR(I),-UI(I),UR(J),-UI(J),Y,Z)
      CALL DIVC(Y,Z,U,V,W,X)
      CSR(I,J)=W
      CSI(I,J)=X
      CR(I,J)=W
      CI(I,J)=X
  103 CONTINUE
  104 CONTINUE
C      DO 150 T=1,LEN
      do 150 t=1,2
      W=TDIF(T)
      DO 105 I=1,ARP
      CALL MULR(ROOTR(I),ROOTI(I),W,U,V)
      U=DEXP(U)
      DR(I)=U*DCOS(V)
      DI(I)=U*DSIN(V)
  105 CONTINUE
      DO 107 I=1,ARP
      CALL MULC(DR(I),DI(I),WR(I),WI(I),WPR(I),WPI(I))
c      print *,'T=',t
c      print *,'CALL MULC(DR(I),DI(I),WR(I),WI(I),WPR(I),WPI(I))'
c      print *,'WPR(I)=',WPR(I)
c      print *,'WPI(I)=',WPI(I)
      DO 106 J=1,ARP
      CALL SUBC(CR(I,J),CI(I,J),CSR(I,J),CSI(I,J),U,V)
c      print *,'SUBC,U=',U
c      print *,'SUBC,V=',V
      CALL MULC(U,V,DR(J),-DI(J),Y,Z)
c      print *,'MULC(U,V,DR(J),-DI(J),Y,Z)' 
c      print *,'MULC,Y',Y
c      print *,'MULC,Z=',Z
      CALL MULC(DR(I),DI(I),Y,Z,U,V)
c      print *,' MULC(DR(I),DI(I),Y,Z,U,V)'
c      print *,'U=',U
c      print *,'V=',V
      CALL ADDC(U,V,CSR(I,J),CSI(I,J),CPR(I,J),CPI(I,J))
c      print *,'CPR=',CPR(I,J)
c      print *,'CPI=',CPI(I,J)
  106 CONTINUE
  107 CONTINUE
      DO 109 I=1,ARP
      Y=CSZ
      Z=CSZ
      DO 108 J=1,ARP
      CALL MULC(CPR(I,J),CPI(I,J),XVECR(J),-XVECI(J),W,X)
c      print *,'MULC(CPR(I,J),CPI(I,J),XVECR(J),-XVECI(J),W,X)'
c      print *,'XVECR(J)=',XVECR(J)
c      print *,'XVECI(J)=',XVECI(J)
c      print *,'W=',W
c      print *,'X=',X
      Y=Y+W
      Z=Z+X
  108 CONTINUE
      MR(I)=Y
      MI(I)=Z
c      print *,'MR(I)=',Y
c      print *,'MI(I)=',Z
  109 CONTINUE
      PEV=CSZ
      PRED=CSZ
      DO 110 I=1,ARP
      U=XVECR(I)
      V=XVECI(I)
      CALL MULC(U,V,MR(I),MI(I),W,X)
      PEV=PEV+W
c      print *,'PEV=',PEV
      CALL MULC(U,V,WPR(I),WPI(I),W,X)
      PRED=PRED+W
c      print *,'PRED=',PRED
  110 CONTINUE
      PEV=PEV+VOB
      VT(T)=PEV
      U=SER(T)-CONST-PRED
      ERR(T)=U
c      print *,'ERR=',U
      DO 111 I=1,ARP
      NR(I)=MR(I)/PEV
      NI(I)=MI(I)/PEV
      CALL MULR(NR(I),NI(I),U,W,X)
      CALL ADDC(WPR(I),WPI(I),W,X,WR(I),WI(I))
  111 CONTINUE
      DO I=1,ARP
        DO J=1,ARP
          CALL MULC(MR(I),MI(I),NR(J),-NI(J),W,X)
c      print *,'CALL MULC(MR(I),MI(I),NR(J),-NI(J),W,X)'
c      print *,'W=',W
c      print *,'X=',X
          CALL SUBC(CPR(I,J),CPI(I,J),W,X,CR(I,J),CI(I,J))
c      print *,'CALL SUBC(CPR(I,J),CPI(I,J),W,X,CR(I,J),CI(I,J))'
c      print *,'CR(I,J)=',CR(I,J)
c      print *,'CI(I,J)=',CI(I,J)
        END DO
      END DO
c     add the following lines to store results in order to calculate smoothic     ng states and covariance matrix, smoothing series and variances
      if (KFSW.EQ.0) then
         pre(T)=const+pred
         prv(T)=pev
         u=csz
         DO I=1,ARP
            FSESR(T,I)=WR(I)
            FSESI(T,I)=WI(I)
            PSESR(T,I)=WPR(I)
            PSESI(T,I)=WPI(I)
            U=U+XVECR(I)*WR(I)-XVECI(I)*WI(I)
         END DO
            FSER0(T,I)=U+CONST
            U=CSZ
            DO I=1,ARP
               TRANR(T,I)=DR(I)
               TRANI(T,I)=DI(I)
               DO J=1,ARP
                  FSCVR(T,I,J)=CR(I,J)
                  FSCVI(T,I,J)=CI(I,J)
                  PSCVR(T,I,J)=CPR(I,J)
                  PSCVI(T,I,J)=CPI(I,J)
               END DO
            END DO
c 135              U=U+
               ENDIF
 150        CONTINUE
            RETURN
            END
      

*UPTODATE SDATATEXT
      SUBROUTINE SDATA
C
C        MARK 2.2 RELEASE.  NAG COPYRIGHT 1987.
C
C PURPOSE
C
C        This subroutine initialises the values of machine dependent
C        constants, for use by other sub-programs.
C
C METHOD
C
C        Pre-determined values assigned to common blocks.
C
C COMMON BLOCK
C
C        MDEPS        length 4 units ( 4*REAL )
C
C                BASE    REAL
C                        -The base of the machine representation of
C                        numerical values.
C
C                BIG     REAL
C                        -A large number that can be represented
C                        in the machine without causing overflow.
C
C                SMALL   REAL
C                        -A small number for which the machine
C                        can evaluate the expression  1.0 / SMALL
C                        in without causing overflow.
C
C                EPS     REAL
C                        -The relative accuracy of machine arithmetic.
C                        Namely, approximately the smallest positive
C                        value for which the logical expression
C
C                                1.0 + EPS  .GT.  1.0
C
C                        is true. Note that this is sometimes referred
C                        to as  MACHEP , MACHEPS , EPS or MPREC.
C
C        SMPREC       length  1  unit  ( 1*REAL )
C
C                EPSS    REAL
C                        -Equivalent to  EPS  above.
C
C*** NOTE that the common block ADDEPS is not used within SLICE
C    but is used to output other useful machine dependent values.
C
C        ADDEPS       length  3  units ( 2*INTEGER, 1*REAL )
C
C                T       INTEGER
C                        -The number of digits in the mantissa.
C
C                EMIN    INTEGER
C                        -The minimum exponent before (gradual)
C                         underflow occurs.
C
C                RMIN    REAL
C                        -The smallest normalized number for the machine
C
C
C VERSION        SDATA 2.2  A Fortran 77 version.
C
C LIBRARY INDEX
C
C                SLICE 2.2
C                Section  1.  Utility Routines
C                Subsection  1.0  General Routines
C
C COMMENTS
C
C                None
C
C******************************************************************
C
C     .. Scalars in Common ..
      DOUBLE PRECISION BASE,BIG,EPS,EPSS,RMIN,SMALL
      INTEGER EMIN,T
C     .. Local Scalars ..
      DOUBLE PRECISION FLMIN,MEPS
      INTEGER BETA
      LOGICAL FIRST
C     .. External Subroutines ..
      EXTERNAL ENVIRN
C     .. Common blocks ..
      COMMON /ADDEPS/RMIN,EMIN,T
      COMMON /MDEPS/BASE,BIG,SMALL,EPS
      COMMON /SMPREC/EPSS
C     .. Save statement ..
      SAVE FIRST
C     .. Data statements ..
C
      DATA FIRST/.TRUE./
C     .. Executable Statements ..
C
      IF (FIRST) THEN
          FIRST = .FALSE.
C
          CALL ENVIRN(BETA,T,MEPS,EMIN,RMIN)
C
          FLMIN = RMIN*BETA**4
C
          BASE = BETA
          SMALL = FLMIN
          BIG = 1/FLMIN
          EPS = MEPS
          EPSS = EPS
      END IF
      RETURN
      END

      SUBROUTINE D9RPLY(SSS, NZ, IFLAG, P, QP, K, QK)
C VARIABLE-SHIFT H POLYNOMIAL ITERATION FOR A REAL
C ZERO.
C SSS   - STARTING ITERATE
C NZ    - NUMBER OF ZERO FOUND
C IFLAG - FLAG TO INDICATE A PAIR OF ZEROS NEAR REAL
C         AXIS.
C
      COMMON /P77PLY/ SR, SI, U,
     1 V, A, B, C, D, A1, A2, A3, A6, A7, E, F, G,
     2 H, SZR, SZI, LZR, LZI, ETA, ARE, MRE, N, NN
C
      INTEGER N, NN
      INTEGER NZ, IFLAG, I, J, NM1
C
      DOUBLE PRECISION ETA, ARE, MRE
      DOUBLE PRECISION MS, MP, OMP, EE
C
      DOUBLE PRECISION P(*), QP(*), K(*),
     1   QK(*), SR, SI, U, V, A, B, C, D,
     2   A1, A2, A3, A6, A7, E, F, G, H, SZR, SZI,
     3   LZR, LZI
      DOUBLE PRECISION PV, KV, T, S, SSS
      NM1 = N - 1
      NZ = 0
      S = SSS
      IFLAG = 0
      J = 0
      OMP = 0.D0
      T = 0.D0
C MAIN LOOP
   10 PV = P(1)
C EVALUATE P AT S
      QP(1) = PV
      DO 20 I=2,NN
        PV = PV*S + P(I)
        QP(I) = PV
   20 CONTINUE
      MP = DABS(PV)
C COMPUTE A RIGOROUS BOUND ON THE ERROR IN EVALUATING
C P
      MS = DABS(S)
      EE = (MRE/(ARE+MRE))*ABS(QP(1))
      DO 30 I=2,NN
        EE = EE*MS + ABS(QP(I))
   30 CONTINUE
C ITERATION HAS CONVERGED SUFFICIENTLY IF THE
C POLYNOMIAL VALUE IS LESS THAN 20 TIMES THIS BOUND
      IF (MP.GT.20.*((ARE+MRE)*EE-MRE*MP)) GO TO 40
      NZ = 1
      SZR = S
      SZI = 0.D0
      RETURN
   40 J = J + 1
C STOP ITERATION AFTER 10 STEPS
      IF (J.GT.10) RETURN
      IF (J.LT.2) GO TO 50
      IF (DABS(T).GT..001*DABS(S-T) .OR. MP.LE.OMP)
     1   GO TO 50
C A CLUSTER OF ZEROS NEAR THE REAL AXIS HAS BEEN
C ENCOUNTERED RETURN WITH IFLAG SET TO INITIATE A
C QUADRATIC ITERATION
      IFLAG = 1
      SSS = S
      RETURN
C RETURN IF THE POLYNOMIAL VALUE HAS INCREASED
C SIGNIFICANTLY
   50 OMP = MP
C COMPUTE T, THE NEXT POLYNOMIAL, AND THE NEW ITERATE
      KV = K(1)
      QK(1) = KV
      DO 60 I=2,N
        KV = KV*S + K(I)
        QK(I) = KV
   60 CONTINUE
      IF (DABS(KV).LE.DABS(K(N))*10.*ETA) GO TO 80
C USE THE SCALED FORM OF THE RECURRENCE IF THE VALUE
C OF K AT S IS NONZERO
      T = -PV/KV
      K(1) = QP(1)
      DO 70 I=2,N
        K(I) = T*QK(I-1) + QP(I)
   70 CONTINUE
      GO TO 100
C USE UNSCALED FORM
   80 K(1) = 0.0D0
      DO 90 I=2,N
        K(I) = QK(I-1)
   90 CONTINUE
  100 KV = K(1)
      DO 110 I=2,N
        KV = KV*S + K(I)
  110 CONTINUE
      T = 0.D0
      IF (DABS(KV).GT.DABS(K(N))*10.*ETA) T = -PV/KV
      S = S + T
      GO TO 10
      END

      SUBROUTINE SETUP (PFI1,ARP1,VRI1,CCV1,SCALE1,ARI1,VR1,
     *PHI1,LYAP1,PRDG1,SCC1,FCT1,FTY1,
     *LEN1,FOR1,TIM1,SER1,
     *NIT1,OPM1,RGM1,REQ1,CONCRIT1,RPERT1,IVLAM1,
     *FAC1,STLAM1,SMLAM1,GTLAM1,KST1,NP1, tra1)
      IMPLICIT NONE
c      CHARACTER(LEN=1) ST9
c      CHARACTER(LEN=4) ST0,ST1
      INTEGER I,J,K,L,M
      DOUBLE PRECISION S
c      INCLUDE 'model.txt'
c      INCLUDE 'series.txt'
c      INCLUDE 'conpar.txt'
c      INCLUDE 'setcon.txt'
c      INCLUDE 'repar2.txt'
c      INCLUDE 'model2.txt'
c      INCLUDE 'veccom.txt'
c      INCLUDE 'repcom.txt'
c      INCLUDE 'repar3.txt'
c      INCLUDE 'resgn1.txt'
c      INCLUDE 'resgn2.txt'
c      INCLUDE 'resgd3.txt'
c      INCLUDE 'redcom.txt'
c      INCLUDE 'seacom.txt'
c      INCLUDE 'kfasav.txt'
C*****model.txt
      INTEGER PFI,ARP,VRI,CCV,LYAP,SCC,fct
      DOUBLE PRECISION SCALE,VR,CONST,PHI(20),PRDG
      COMMON/MODEL/SCALE,VR,CONST,PHI,PRDG,PFI,ARP,VRI,CCV,LYAP,SCC,fct
      integer for, fty
      common/model/for, fty
      integer ARI, tra
      common/model/ari,tra      
C*****series.txt
      CHARACTER(LEN=40) NAME
      INTEGER LEN
      DOUBLE PRECISION TIM(5000),SER(5000),TDIF(5000)
c      COMMON/SERIES/NAME,LEN,TIM,SER,TDIF
      COMMON/SERIES/TIM,SER,TDIF,NAME,LEN

C*****conpar.txt    
      INTEGER NIT,OPM,RGM,KUP,KSP,KST
      DOUBLE PRECISION REQ,CONCRIT,RPERT,IVLAM,FAC,STLAM,SMLAM,GTLAM
c      COMMON/CONPAR/NIT,OPM,RGM,REQ,CONCRIT,RPERT,IVLAM,FAC,STLAM,
c     * SMLAM,GTLAM,KUP,KSP,KST
      COMMON/CONPAR/REQ,CONCRIT,RPERT,IVLAM,FAC,STLAM,
     * SMLAM,GTLAM,NIT,OPM,RGM,KUP,KSP,KST

C*****setcon.txt
      LOGICAL CONV,FAIL
      INTEGER NP,ITCT,PPIND
      DOUBLE PRECISION CSO,CSZ,LAM,SSOLD,GMOLD,GMNEW,SIGSQ,OLDB(22)
c      COMMON/SETCON/CONV,FAIL,NP,ITCT,PPIND,CSO,CSZ,LAM,SSOLD,
c     *GMOLD,GMNEW,SIGSQ,OLDB
      COMMON/SETCON/CSO,CSZ,LAM,SSOLD,
     *GMOLD,GMNEW,SIGSQ,OLDB,CONV,FAIL,NP,ITCT,PPIND
C*****repar2.txt
      DOUBLE PRECISION BCT(21),CHT(21,21)
      COMMON/REPAR2/BCT,CHT
C*****model2.txt
      INTEGER PFI1,ARP1,ARI1,VRI1,CCV1,LYAP1,SCC1,fct1,fty1
      DOUBLE PRECISION SCALE1,VR1,PHI1(20),PRDG1
      integer for1

C*****veccom.txt
      DOUBLE PRECISION B(22),NEWB(22),PERB(22),DELB(22),
     * ERR(5000),RES(5000),PRES(22,5000),sres(5000)
      COMMON/VECCOM/B,NEWB,PERB,DELB,ERR,RES,PRES,sres
C*****repcom.txt
      LOGICAL PRPI
      INTEGER REQSW,KFSW
      DOUBLE PRECISION VOB
c      COMMON/REPCOM/PRPI,REQSW,VOB,KFSW
      COMMON/REPCOM/VOB,REQSW,KFSW,PRPI 

C*****repar3.txt
      DOUBLE PRECISION ALPHA(21),ROOTR(20),ROOTI(20),XVECR(20),
     *XVECI(20)
      COMMON/REPAR3/ALPHA,ROOTR,ROOTI,XVECR,XVECI
C*****resgn1.txt
      DOUBLE PRECISION DR(20),DI(20),MR(20),MI(20),NR(20),NI(20),
     * UR(20),UI(20),WR(20),WI(20),WPR(20),WPI(20),CR(20,20),CI(20,20),
     * CPR(20,20),CPI(20,20),CSR(20,20),CSI(20,20)
      COMMON/RESGN1/DR,DI,MR,MI,NR,NI,UR,UI,WR,WI,WPR,WPI,CR,CI,CPR,CPI,
     * CSR,CSI
C*****resgn2.txt
      DOUBLE PRECISION WK(20),VT(5000),BI(2,20,20),R(2,20,20),
     *RI(2,20,20)
c       DOUBLE PRECISION WK(20),VT(5000),BI(2,20,20),R(20,20),RI(2,20,20)
      INTEGER ERRNO1
      COMMON/RESGN2/WK,VT,BI,R,RI,ERRNO1
C*****resgd3.txt
      INTEGER IW1(20)
      DOUBLE PRECISION W1(20),W2(20),AL(20,20),BL(20,20),
     * CL(20,20),DL(20,20)
      DOUBLE PRECISION MS(20),NS(20),WS(20),WPS(20),AS(20,20),
     * CC(20,20),CS(20,20),CPS(20,20),AW0(20,20),AW1(20,20)
c      COMMON/RESGN3/IW1,W1,W2,AL,BL,CL,DL,MS,NS,WS,WPS,AS,CC,CS,
c     * CPS,AW0,AW1
      COMMON/RESGN3/W1,W2,AL,BL,CL,DL,MS,NS,WS,WPS,AS,CC,CS,
     * CPS,AW0,AW1,IW1
C*****redcom.txt
      DOUBLE PRECISION GRAD(22),SSP(22,22)
      COMMON/REDCOM/GRAD,SSP
C*****seacom.txt
      LOGICAL SOLV
      DOUBLE PRECISION ESSP(22,22),ECOV(22,22)
c      COMMON/SEACOM/SOLV,ESSP,ECOV
      COMMON/SEACOM/ESSP,ECOV,SOLV

C*****kfasav.txt
      DOUBLE PRECISION PRE(5000),PRV(5000),FSER(5000),FVAR(5000)
     1,SSER(5000)
     1,SVAR(5000)
      DOUBLE PRECISION FSES(5000,20),FSCV(5000,20,20),SSES(5000,20)
     1,SSCV(5000,20,20),PSES(5000,20),PSCV(5000,20,20),TRAN(5000,20,20)
c      COMMON/KFASAV/PRE,PRV,FSER,FVAR,SSER,SVAR,FSES,FSCV,SSES,SSCV
c     1,PSES,PSCV,TRAN
      COMMON/KFASAV/FSES,FSCV,SSES,SSCV
     1,PSES,PSCV,TRAN,PRE,PRV,FSER,FVAR,SSER,SVAR
      integer tra1

      INTEGER LEN1, NP1
      DOUBLE PRECISION TIM1(5000),SER1(5000)
C*****conpar1.txt    
      INTEGER NIT1,OPM1,RGM1,KST1
      DOUBLE PRECISION REQ1,CONCRIT1,RPERT1,IVLAM1,FAC1,STLAM1,SMLAM1,
     *GTLAM1
      Do 100 I=1, LEN1
         TIM(I)=TIM1(I)
         SER(I)=SER1(I)
 100     continue

 
      tra=tra1   
      PFI=PFI1
      ARP=ARP1
      ARI=ARI1
      VRI=VRI1
      CCV=CCV1
      SCALE=SCALE1
      VR=VR1
      Do 105 I=1, ARP1
         PHI(I)=PHI1(I)
 105  continue
      LYAP=LYAP1
      PRDG=PRDG1
      SCC=SCC1
      FCT=FCT1
      FTY=FTY1
      LEN=LEN1
      FOR=FOR1
      NIT=NIT1
      OPM=OPM1
      RGM=RGM1
      REQ=REQ1
      CONCRIT=CONCRIT1
      RPERT=RPERT1
      IVLAM=IVLAM1
      FAC=FAC1
      STLAM=STLAM1
      SMLAM=SMLAM1
      GTLAM=GTLAM1
      KST=KST1 

C  INITIALISE CONST
      CONST=CSZ
      IF(CCV.GT.0)THEN
        DO 202 I=1,LEN
        CONST=CONST+SER(I)
  202   CONTINUE
        CONST=CONST/LEN
      END IF
C  INITIALISE TDIF ARRAY
      TDIF(1)=CSZ
      DO 203 I=2,LEN
      TDIF(I)=TIM(I)-TIM(I-1)
  203 CONTINUE
c initialise the length of data, to use part of data if fct=0
      if(fct.EQ.0)then
         len=len-for
      endif
C  INITIALISE NP
      I=0
      IF(VRI.EQ.1)I=I+1
      IF(CCV.EQ.2)I=I+1
      NP=ARP+I
      NP1=NP
C  INITIALISE OLDB ARRAY
      DO 204 I=1,ARP
      OLDB(I)=PHI(I)
  204 CONTINUE
      M=ARP+1
      IF(VRI.EQ.1)OLDB(M)=DSQRT(VR)
      IF(CCV.EQ.2)THEN
        IF(VRI.EQ.0)THEN
          OLDB(M)=CONST
        ELSE
          OLDB(M+1)=CONST
        END IF
      END IF
C  SET INITIAL VALUES
      CONV=.FALSE.
      FAIL=.FALSE.
      ITCT=0
      PPIND=0
      CSO=1.0D0
      LAM=IVLAM
      SSOLD=CSZ
      KFSW=0
      DO 304 J=1,M
      BCT(1)=CSO
      DO 301 I=2,M
      BCT(I)=CSZ
  301 CONTINUE
      DO K=2,M
      IF(K.GT.J)THEN
        S=CSO
      ELSE
        S=-CSO
      END IF
      DO L=2,K
      I=K-L+2
      BCT(I)=BCT(I)+S*BCT(I-1)/SCALE
      END DO
      END DO
      DO 303 I=1,M
      CHT(I,J)=BCT(M+1-I)
  303 CONTINUE
  304 CONTINUE
      BCT(1)=CSO
      DO 400 I=2,M
      BCT(I)=CSZ
  400 CONTINUE
      DO K=2,ARP
      DO I=2,K
      J=K-I+2
      BCT(J)=BCT(J)+BCT(J-1)/SCALE
      END DO
      END DO
      RETURN
      END

      SUBROUTINE ROOTS
      IMPLICIT NONE
      INTEGER I,J
      DOUBLE PRECISION T
C     DOUBLE PRECISION ROOTC(40),WORK(42)    CHANGED ZW
C     LOGICAL RSCAL  
c      double precision alpha1(3)                        
c      INCLUDE 'model.txt'
c      INCLUDE 'setcon.txt'
c      INCLUDE 'veccom.txt'
c      INCLUDE 'repar3.txt'
C*****model.txt
      INTEGER PFI,ARP,VRI,CCV,LYAP,SCC,fct
      DOUBLE PRECISION SCALE,VR,CONST,PHI(20),PRDG
      COMMON/MODEL/SCALE,VR,CONST,PHI,PRDG,PFI,ARP,VRI,CCV,LYAP,SCC,fct
      integer for, fty
      common/model/for, fty
      integer ARI, tra
      common/model/ari,tra      
C*****setcon.txt
      LOGICAL CONV,FAIL
      INTEGER NP,ITCT,PPIND
      DOUBLE PRECISION CSO,CSZ,LAM,SSOLD,GMOLD,GMNEW,SIGSQ,OLDB(22)
c      COMMON/SETCON/CONV,FAIL,NP,ITCT,PPIND,CSO,CSZ,LAM,SSOLD,
c     *GMOLD,GMNEW,SIGSQ,OLDB
      COMMON/SETCON/CSO,CSZ,LAM,SSOLD,
     *GMOLD,GMNEW,SIGSQ,OLDB,CONV,FAIL,NP,ITCT,PPIND
C*****veccom.txt
      DOUBLE PRECISION B(22),NEWB(22),PERB(22),DELB(22),
     * ERR(5000),RES(5000),PRES(22,5000),sres(5000)
      COMMON/VECCOM/B,NEWB,PERB,DELB,ERR,RES,PRES,sres
C*****repar3.txt
      DOUBLE PRECISION ALPHA(21),ROOTR(20),ROOTI(20),XVECR(20),
     *XVECI(20)
      COMMON/REPAR3/ALPHA,ROOTR,ROOTI,XVECR,XVECI

      J=ARP+1
      ALPHA(1)=CSO
      DO 100 I=2,J
         ALPHA(I)=B(I-1)
 100  CONTINUE
      I=0
      T=1.110D-16
C     CALL C02AEF(ALPHA,J,ROOTR,ROOTI,T,I)
C     RSCAL=.TRUE.                            CHANGED ZW
Cc     CALL C02AGF(ALPHA,J,RSCAL,ROOTC,WORK,I) CHANGED ZW
C     PRINT *,  ROOTC
c     the original call for c02agf where J is the degree of polynomial, should J be J-1?
c      print *,'alpha= ',alpha
c      print *,'alpha(1)=',alpha(1)
c      print *,'alpha(2)=',alpha(2)
c      print *,'alpha(3)=',alpha(3)
c      print *,'order J= ',J-1
c      call rpoly(alpha,J-1,rootr,rooti,fail)  
c     call rpoly using netlib subroutines      ZW 3/26/05
 
c      alpha1(1)=alpha(1)
c      alpha1(2)=alpha(2)
c      alpha1(3)=alpha(3)
c      print *,'alpha1= ',alpha1
c      print *,'alpha1(1)=',alpha1(1)
c      print *,'alpha1(2)=',alpha1(2)
c      print *,'alpha1(3)=',alpha1(3)
      call drpoly(J-1,ALPHA,rootr,rooti)
c      call drpoly(2,alpha1,rootr,rooti)  3/26/05
      if (fail.EQV..TRUE.) then
         call rexit('program fails in rpoly')
      endif
c         print *,'program fails in rpoly'  3/26/05
c         print *,'program fails in rpoly'  3/26/05
c         stop                              3/26/05
c      else                                 3/26/05
c      print *,'rootr= ',rootr
c      print *,'rooti= ',rooti
C     added ZW
c     IF(I.NE.0)THEN
c     PRINT *,'PROGRAM FAILS IN C02AGF WITH ERROR: ',FAIL
c     STOP
c     ELSE
C     K=0
C     DO 105 I=1,ARP
C     K=K+1
C     ROOTR(I)=ROOTC(K)
C     K=K+1
C     ROOTI(I)=ROOTC(K)
C     105   CONTINUE 
            DO 101 I=1,ARP
               ALPHA(I)=B(I)
 101           CONTINUE
            ALPHA(ARP+1)=CSZ
c         END IF                3/26/05
         RETURN
         END

"plotSpecCar" <-
function (x, add = FALSE, ci = 0.95, log = "dB", xlab = "frequency", 
    ylab = NULL, type = "l", main = NULL, sub = NULL,...) 
{
    if (is.null(ylab)) 
        ylab <- if (log == "dB") 
            "spectrum (dB)"
        else "spectrum"
    if (is.logical(log)) 
        log <- if (log) 
            "yes"
        else "no"
    if (missing(log) && getOption("ts.S.compat")) 
        log <- "dB"
    log <- match.arg(log)
    ylog <- ""
    if (log == "dB") 
        x$spec <- 10 * log10(x$spec)
    if (log == "yes") 
        ylog <- "y"
    if (add) {
        matplot(x$freq, x$spec, type = type, add = TRUE, ...)
    }
    else {
        matplot(x$freq, x$spec, xlab = xlab, ylab = ylab, type = type, log = ylog)
#        if (ci <= 0 || !is.numeric(x$df) || log == "no") {
#            ci.text <- ""
#        }
#        else {
#            conf.lim <- spec.ci(x, coverage = ci)
#            if (log == "dB") {
#                conf.lim <- 10 * log10(conf.lim)
#                conf.y <- max(x$spec) - conf.lim[2]
#                conf.x <- max(x$freq) - x$bandwidth
#                lines(rep(conf.x, 2), conf.y + conf.lim)
#                lines(conf.x + c(-0.5, 0.5) * x$bandwidth, rep(conf.y, 
#                  2))
#                ci.text <- paste(", ", round(100 * ci, 2), "% C.I. is (", 
#                  paste(format(conf.lim, digits = 3), collapse = ","), 
#                  ")dB", sep = "")
#            }
#            else {
#                ci.text <- ""
#                conf.y <- max(x$spec)/conf.lim[2]
#                conf.x <- max(x$freq) - x$bandwidth
#                lines(rep(conf.x, 2), conf.y * conf.lim)
#                lines(conf.x + c(-0.5, 0.5) * x$bandwidth, rep(conf.y, 
#                  2))
#            }
#        }
          if (is.null(main)) 
            main <- paste(x$method, sep = "\n")
        if (is.null(sub) && is.numeric(x$bandwidth)) 
            sub <- paste("bandwidth = ", format(x$bandwidth,                 digits = 3), sep = "")
          title(main = main,sub=sub)
      }
    invisible(x)
}

c     SUBROUTINE KFILSM
      SUBROUTINE FORECAST
      IMPLICIT NONE
      INTEGER I, T
c      INCLUDE 'model.txt'
c      INCLUDE 'repcom.txt'
c      INCLUDE 'kfasav.txt'
c      INCLUDE 'setcon.txt'
c      INCLUDE 'veccom.txt'
c      INCLUDE 'series.txt'
c      INCLUDE 'repar3.txt'
C*****model.txt
      INTEGER PFI,ARP,VRI,CCV,LYAP,SCC,fct
      DOUBLE PRECISION SCALE,VR,CONST,PHI(20),PRDG
      COMMON/MODEL/SCALE,VR,CONST,PHI,PRDG,PFI,ARP,VRI,CCV,LYAP,SCC,fct
      integer for, fty
      common/model/for, fty
      integer ARI, tra
      common/model/ari,tra

C*****repcom.txt
      LOGICAL PRPI
      INTEGER REQSW,KFSW
      DOUBLE PRECISION VOB
c      COMMON/REPCOM/PRPI,REQSW,VOB,KFSW
      COMMON/REPCOM/VOB,REQSW,KFSW,PRPI
C*****kfasav.txt
      DOUBLE PRECISION PRE(5000),PRV(5000),FSER(5000),FVAR(5000)
     1,SSER(5000)
     1,SVAR(5000)
      DOUBLE PRECISION FSES(5000,20),FSCV(5000,20,20),SSES(5000,20)
     1,SSCV(5000,20,20),PSES(5000,20),PSCV(5000,20,20),TRAN(5000,20,20)
c      COMMON/KFASAV/PRE,PRV,FSER,FVAR,SSER,SVAR,FSES,FSCV,SSES,SSCV
c     1,PSES,PSCV,TRAN
      COMMON/KFASAV/FSES,FSCV,SSES,SSCV
     1,PSES,PSCV,TRAN,PRE,PRV,FSER,FVAR,SSER,SVAR
C*****setcon.txt
      LOGICAL CONV,FAIL
      INTEGER NP,ITCT,PPIND
      DOUBLE PRECISION CSO,CSZ,LAM,SSOLD,GMOLD,GMNEW,SIGSQ,OLDB(22)
c      COMMON/SETCON/CONV,FAIL,NP,ITCT,PPIND,CSO,CSZ,LAM,SSOLD,
c     *GMOLD,GMNEW,SIGSQ,OLDB
      COMMON/SETCON/CSO,CSZ,LAM,SSOLD,
     *GMOLD,GMNEW,SIGSQ,OLDB,CONV,FAIL,NP,ITCT,PPIND

C*****veccom.txt
      DOUBLE PRECISION B(22),NEWB(22),PERB(22),DELB(22),
     * ERR(5000),RES(5000),PRES(22,5000),sres(5000)
      COMMON/VECCOM/B,NEWB,PERB,DELB,ERR,RES,PRES,sres

C*****series.txt
      CHARACTER(LEN=40) NAME
      INTEGER LEN
      DOUBLE PRECISION TIM(5000),SER(5000),TDIF(5000)
c      COMMON/SERIES/NAME,LEN,TIM,SER,TDIF
      COMMON/SERIES/TIM,SER,TDIF,NAME,LEN
C*****repar3.txt
      DOUBLE PRECISION ALPHA(21),ROOTR(20),ROOTI(20),XVECR(20),
     *XVECI(20)
      COMMON/REPAR3/ALPHA,ROOTR,ROOTI,XVECR,XVECI

      if(tra.EQ.1)then
C      PRINT *,'FORECASTING IN PROGRESS'
      endif
      KFSW=1
      DO I=1,NP
        B(I)=OLDB(I)
      END DO

      CALL REPAR
c     resg1pre and resg1pre1 together are predicting
c fct=0 means to use part (len-for) of data to fit a model, then forecast. So the data length will be changed.
      if(fct.EQ.0)then
        if(tra.EQ.1)then
C         print *,'  THE LENGTH OF THE DATA USED TO MODEL IS'
C     *,LEN 
C         print *,'  FCT=0, FORECAST LAST L-STEP'  
        endif
c note: for fct=0, LEN has been reduced to LEN-FOR in setup subroutine      
         call resg1pre1
c fct=1 means to use all data to fit a model, then forecast.
c subroutine resg1pre uses len-for of data to estimate optimal state and covariance matrices.
      else
        if(tra.EQ.1)then
C         print *,'  THE LENGTH OF THE DATA USED TO MODEL IS'
C     *,LEN         
        endif
         if(fty.EQ.1)then
           if(tra.EQ.1)then
C            print *,'  FCT=1 AND FTY=1, FORECAST PAST THE END'
           endif
c     construct equally spaced time points to forecast
c     this may be changed, for example, from input file
            do 15 T=LEN+1,LEN+FOR
               TIM(T)=TIM(T-1)+1
c changed 6/6/06
               TDIF(T)=TIM(T)-TIM(T-1)  
 15         CONTINUE
            call resg1pre1
         else
            if(fty.EQ.2)then
              if(tra.EQ.1)then
C               print *,'  FCT=1 AND FTY=2, FORECAST LAST L-STEP'
              endif
c               estimate state and convariance matrix up to time LEN-FOR
               call resg1pre
               LEN=LEN-FOR
               call resg1pre1
             else
              if(tra.EQ.1)then
C                print *,'  FCT=1 AND FTY=3, FORECAST LAST L-STEP UPDATE
C     *D(FILTERING)' 
              endif
             endif
         endif
      endif

ccccccccccc
c      open(unit=4,file='forecast.dat',status='UNKNOWN')
c      WRITE(4,195)
c  195 FORMAT(7H LENGTH)
c      WRITE(4,196) for
c  196 FORMAT(I10)
c      WRITE(4,201) 
c  201 format(27H  TIME SERIES PRESER PREVAR) 
      if(fct.EQ.0)then
c         DO 20 T=LEN+1,LEN+FOR
c 20         write(4,202) TIM(T),SER(T),PRE(T),PRV(T)
c 202        FORMAT(4E13.5)
         else
            if(fty.EQ.3)then
               LEN=LEN-FOR
            endif
c            do 30 T=LEN+1,LEN+for
c 30            write(4,203) TIM(T),SER(T),PRE(T),PRV(T)
c 203           FORMAT(4E13.5)
            endif
c            CLOSE(UNIT=4)
C      DO 5 I=1, 10
C      call dblepr("in forecast subroutine, PRE(I)", -1, PRE(I), 1)
C   5   CONTINUE
            RETURN
            END

      SUBROUTINE KFILSM
      IMPLICIT NONE
      INTEGER I, T
c      INCLUDE 'model.txt'
c      INCLUDE 'repcom.txt'
c      INCLUDE 'kfasav.txt'
c      INCLUDE 'setcon.txt'
c      INCLUDE 'veccom.txt'
c      INCLUDE 'series.txt'
c      INCLUDE 'repar3.txt'
C*****model.txt
      INTEGER PFI,ARP,VRI,CCV,LYAP,SCC,fct
      DOUBLE PRECISION SCALE,VR,CONST,PHI(20),PRDG
      COMMON/MODEL/SCALE,VR,CONST,PHI,PRDG,PFI,ARP,VRI,CCV,LYAP,SCC,fct
      integer for, fty
      common/model/for, fty
      integer ARI, tra
      common/model/ari,tra

C*****repcom.txt
      LOGICAL PRPI
      INTEGER REQSW,KFSW
      DOUBLE PRECISION VOB
c      COMMON/REPCOM/PRPI,REQSW,VOB,KFSW
      COMMON/REPCOM/VOB,REQSW,KFSW,PRPI 
C*****kfasav.txt
      DOUBLE PRECISION PRE(5000),PRV(5000),FSER(5000),FVAR(5000)
     1,SSER(5000)
     1,SVAR(5000)
      DOUBLE PRECISION FSES(5000,20),FSCV(5000,20,20),SSES(5000,20)
     1,SSCV(5000,20,20),PSES(5000,20),PSCV(5000,20,20),TRAN(5000,20,20)
c      COMMON/KFASAV/PRE,PRV,FSER,FVAR,SSER,SVAR,FSES,FSCV,SSES,SSCV
c     1,PSES,PSCV,TRAN
      COMMON/KFASAV/FSES,FSCV,SSES,SSCV
     1,PSES,PSCV,TRAN,PRE,PRV,FSER,FVAR,SSER,SVAR
C*****setcon.txt
      LOGICAL CONV,FAIL
      INTEGER NP,ITCT,PPIND
      DOUBLE PRECISION CSO,CSZ,LAM,SSOLD,GMOLD,GMNEW,SIGSQ,OLDB(22)
c      COMMON/SETCON/CONV,FAIL,NP,ITCT,PPIND,CSO,CSZ,LAM,SSOLD,
c     *GMOLD,GMNEW,SIGSQ,OLDB
      COMMON/SETCON/CSO,CSZ,LAM,SSOLD,
     *GMOLD,GMNEW,SIGSQ,OLDB,CONV,FAIL,NP,ITCT,PPIND
C*****veccom.txt
      DOUBLE PRECISION B(22),NEWB(22),PERB(22),DELB(22),
     * ERR(5000),RES(5000),PRES(22,5000),sres(5000)
      COMMON/VECCOM/B,NEWB,PERB,DELB,ERR,RES,PRES,sres
C*****series.txt
      CHARACTER(LEN=40) NAME
      INTEGER LEN
      DOUBLE PRECISION TIM(5000),SER(5000),TDIF(5000)
c      COMMON/SERIES/NAME,LEN,TIM,SER,TDIF
      COMMON/SERIES/TIM,SER,TDIF,NAME,LEN
C*****repar3.txt
      DOUBLE PRECISION ALPHA(21),ROOTR(20),ROOTI(20),XVECR(20),
     *XVECI(20)
      COMMON/REPAR3/ALPHA,ROOTR,ROOTI,XVECR,XVECI

      if(tra.EQ.1)then
C      PRINT *,'SAVING OF FILTERED AND SMOOTHED STATES IN PROGRESS' 
      end if
      KFSW=1
      DO I=1,NP
        B(I)=OLDB(I)
      END DO
c      print *,'B',(B(I),I=1,NP)
      CALL REPAR
      CALL RESG1    
C      RETURN
      CALL REVG1
c     calculate standardised residuals (innovations)
c     SIGSQ is the innovation variance
      do 11 T=1,LEN
         sres(T)=err(T)/sqrt(prv(T))/sqrt(SIGSQ)
 11   continue
C     STORE FILTERED AND SMOOTHED VALUES, TOGETHER WITH ORIGINAL DATA
c      OPEN(UNIT=4,FILE='filser.dat',STATUS='unknown')
c      WRITE(4,195)
c  195 FORMAT(7H LENGTH)
c      WRITE(4,196) LEN
c  196 FORMAT(I10)
c      WRITE(4,201) 
c  201 FORMAT(54H  TIME SERIES FILSER FILVAR ERROR ERRVAR SMOSER SMOVAR)
c  201 FORMAT(68H  TIME SERIES FILSER FILVAR ERROR ERRVAR SMOSER SMOVAR I
c     +NNSER STDINN)
c      DO 20 T=1,LEN
c      WRITE(4,202) TIM(T),SER(T),FSER(T),FVAR(T)
c      WRITE(4,202) PRE(T),PRV(T),SSER(T),SVAR(T)
c   20 write(4,202) err(T),sres(T)
c  202 FORMAT(4E13.5)
c      CLOSE(UNIT=4)
c      OPEN(UNIT=4,FILE='fsest.dat',STATUS='unknown')
c      WRITE(4,295)
c  295 FORMAT(18H  DIMENSION LENGTH )
c      WRITE(4,296) ARP,LEN
c  296 FORMAT(2I10)
c      WRITE(4,300)
c  300 FORMAT(20H  OBSERVATION VECTOR  )
c      WRITE(4,302) (XVECR(I),I=1,ARP)
c      WRITE(4,301)
c  301 FORMAT(35H  FILSTATE SMOOSTATE FILVAR SMOOVAR  )
c      DO 30 T=1,LEN
c      WRITE(4,302) (FSES(T,I),I=1,ARP)
c      WRITE(4,302) (SSES(T,I),I=1,ARP)
c      WRITE(4,302) (FSCV(T,I,I),I=1,ARP)
c   30 WRITE(4,302) (SSCV(T,I,I),I=1,ARP)
c  302 FORMAT(4E13.5)
c      CLOSE(UNIT=4)
      RETURN
      END

*UPTODATE GETMINTEXT
      SUBROUTINE GETMIN(EMIN,START,BASE)
C
C        MARK 2.2 RELEASE.  NAG COPYRIGHT 1987.
C
C     Service routine for ENVIRN.
C
C
C  Nag Fortran 77 O( 1 ) basic linear algebra routine.
C
C  -- Written on 2-June-1987.
C     Mick Pont, Nag Central Office.
C
C     .. Scalar Arguments ..
      DOUBLE PRECISION START
      INTEGER BASE,EMIN
C     .. Local Scalars ..
      DOUBLE PRECISION A,B1,B2,C1,C2,D1,D2,ONE,RBASE,ZERO
      INTEGER I
C     .. External Functions ..
      DOUBLE PRECISION STORE
      EXTERNAL STORE
C     .. Executable Statements ..
      A = START
      ONE = 1
      RBASE = ONE/BASE
      ZERO = 0
      EMIN = 1
      B1 = STORE(A*RBASE,ZERO)
      C1 = A
      C2 = A
      D1 = A
      D2 = A
   20 IF ((C1.EQ.A) .AND. (C2.EQ.A) .AND. (D1.EQ.A) .AND.
     +    (D2.EQ.A)) THEN
          EMIN = EMIN - 1
          A = B1
          B1 = STORE(A/BASE,ZERO)
          C1 = STORE(B1*BASE,ZERO)
          D1 = ZERO
          DO I = 1,BASE
              D1 = D1 + B1
          END DO
          B2 = STORE(A*RBASE,ZERO)
          C2 = STORE(B2/RBASE,ZERO)
          D2 = ZERO
          DO I = 1,BASE
              D2 = D2 + B2
          END DO
          GO TO 20
      END IF
      RETURN
      END

*UPTODATE BALINVTEXT
      SUBROUTINE BALINV(NM,N,LOW,IGH,SCALE,A)
C
C        MARK 2.2 RELEASE.  NAG COPYRIGHT 1987.
C
C     .. Scalar Arguments ..
      INTEGER IGH,LOW,N,NM
C     .. Array Arguments ..
      DOUBLE PRECISION A(NM,N),SCALE(N)
C     .. Local Scalars ..
      DOUBLE PRECISION S
      INTEGER I,II,J,K
C     .. Executable Statements ..
      IF (IGH.EQ.LOW) GO TO 100
      DO 40 I = LOW,IGH
          S = SCALE(I)
          DO 20 J = 1,N
              A(I,J) = S*A(I,J)
   20     CONTINUE
   40 CONTINUE
      DO 80 J = LOW,IGH
          S = 1.0D0/SCALE(J)
          DO 60 I = 1,N
              A(I,J) = A(I,J)*S
   60     CONTINUE
   80 CONTINUE
  100 DO 160 II = 1,N
          I = II
          IF (I.GE.LOW .AND. I.LE.IGH) GO TO 160
          IF (I.LT.LOW) I = LOW - II
          K = INT(SCALE(I))
          IF (K.EQ.I) GO TO 160
          DO 120 J = 1,N
              S = A(I,J)
              A(I,J) = A(K,J)
              A(K,J) = S
  120     CONTINUE
          DO 140 J = 1,N
              S = A(J,I)
              A(J,I) = A(J,K)
              A(J,K) = S
  140     CONTINUE
  160 CONTINUE
      RETURN
      END

      SUBROUTINE CSPEC(B1,ARP1,SCALE1,FRMULT,NFREQ,F1,S1)
      IMPLICIT NONE
      INTEGER I,J,M,NF,NF1,NFREQ,ARP1
      DOUBLE PRECISION B1(ARP1+1),F1(NFREQ),S1(NFREQ)
      DOUBLE PRECISION FAC,P2,S2,F,S,U,V,W,X,Y,Z,SCALE1
      DOUBLE PRECISION M_SCALE
c      INCLUDE 'model.txt'
c      INCLUDE 'setcon.txt'
c      INCLUDE 'repar3.txt'
c      INCLUDE 'veccom.txt'
      INTEGER PFI,ARP,VRI,CCV,LYAP,SCC,fct
      DOUBLE PRECISION SCALE,VR,CONST,PHI(20),PRDG
      COMMON/MODEL/SCALE,VR,CONST,PHI,PRDG,PFI,ARP,VRI,CCV,LYAP,SCC,fct
      integer for, fty
      common/model/for, fty
      integer ARI, tra
      common/model/ari,tra

C*****setcon.txt
      LOGICAL CONV,FAIL
      INTEGER NP,ITCT,PPIND
      DOUBLE PRECISION CSO,CSZ,LAM,SSOLD,GMOLD,GMNEW,SIGSQ,OLDB(22)
c      COMMON/SETCON/CONV,FAIL,NP,ITCT,PPIND,CSO,CSZ,LAM,SSOLD,
c     *GMOLD,GMNEW,SIGSQ,OLDB
      COMMON/SETCON/CSO,CSZ,LAM,SSOLD,
     *GMOLD,GMNEW,SIGSQ,OLDB,CONV,FAIL,NP,ITCT,PPIND
C*****repar3.txt
      DOUBLE PRECISION ALPHA(21),ROOTR(20),ROOTI(20),XVECR(20),
     *XVECI(20)
      COMMON/REPAR3/ALPHA,ROOTR,ROOTI,XVECR,XVECI
C*****veccom.txt
      DOUBLE PRECISION B(22),NEWB(22),PERB(22),DELB(22),
     * ERR(5000),RES(5000),PRES(22,5000),sres(5000)
      COMMON/VECCOM/B,NEWB,PERB,DELB,ERR,RES,PRES,sres
      DOUBLE PRECISION FRMULT

c      PRINT *, 'SAVING OF MODEL SPECTRUM IN PROGRESS'
      P2=6.2831853071795864D0
c     added by ZW
      ARP=ARP1
      DO 10 J=1,ARP+1
         B(J)=B1(J)
 10   CONTINUE
      SCALE=SCALE1
      S2=SCALE*SCALE
C      FRMULT=1.0
      M=ARP-1
c      NF=200
      NF=NFREQ-1
      NF1=NF+1
c      OPEN(UNIT=4,FILE='frqspc.dat',STATUS='NEW')
c      IF(PFI.EQ.1)THEN
c        PRINT *,'NOT YET IMPLEMENTED'
c      ELSE IF(PFI.EQ.2)THEN
c        PRINT *,'NOT YET IMPLEMENTED'
c      ELSE IF(PFI.GE.3)THEN
        IF(PFI.GE.3)THEN
        CALL ROOTS
        IF(PFI.GE.5)THEN
          DO 50 J=1,ARP
          U=ROOTR(J)
          Z=ROOTI(J)
          W=CSO-U
          X=-Z
          Y=CSO+U
          CALL DIVC(W,X,Y,Z,U,V)
          M_SCALE = -SCALE
          CALL MULR(U,V,M_SCALE,ROOTR(J),ROOTI(J))
   50     CONTINUE
        END IF
c        PRINT *,'FINAL ROOTS: '
c        DO 100 I=1,ARP
c        PRINT *,'   ',I,' ( ',ROOTR(I),' , ',ROOTI(I),' I)'
c  100   CONTINUE
        DO 102 I=1,NF1
        F=FRMULT*DBLE(FLOAT(I-1)/NF)*SCALE
        Y=P2*F
        FAC=CSO
        DO 101 J=1,ARP
        W=ROOTR(J)
        V=ROOTI(J)+Y
        FAC=FAC*(W*W+V*V)
  101   CONTINUE
        S=CSO/FAC
        IF(PFI.EQ.4.OR.PFI.EQ.6)THEN
C          FAC=SCALE*SCALE+Y*Y
C  July 20, 2012. This is not consistent with the factor in the numerator of your equation (6) taken from Belcher et al equation (13). To be consistent the statement should be
          FAC=1+Y*Y/(SCALE*SCALE)
          S=S*(FAC**M)
        END IF
        F1(I)=F
C        S1(I)=S, changed July 20, 2012, ZW. Spectrum should be
C        multiplied by 2\pi \sigma^2
        S1(I)=P2*SIGSQ*S
  102   CONTINUE
      END IF
      RETURN
      END

      SUBROUTINE SETUPDATE (PHI1)
      IMPLICIT NONE
      INTEGER I
c      INCLUDE 'setcon.txt'
C*****setcon.txt
      LOGICAL CONV,FAIL
      INTEGER NP,ITCT,PPIND
      DOUBLE PRECISION CSO,CSZ,LAM,SSOLD,GMOLD,GMNEW,SIGSQ,OLDB(22)
c      COMMON/SETCON/CONV,FAIL,NP,ITCT,PPIND,CSO,CSZ,LAM,SSOLD,
c     *GMOLD,GMNEW,SIGSQ,OLDB
      COMMON/SETCON/CSO,CSZ,LAM,SSOLD,
     *GMOLD,GMNEW,SIGSQ,OLDB,CONV,FAIL,NP,ITCT,PPIND

      DOUBLE PRECISION PHI1(20)
      INTEGER PFI,ARP,VRI,CCV,LYAP,SCC,fct
      DOUBLE PRECISION SCALE,VR,CONST,PHI(20),PRDG
      COMMON/MODEL/SCALE,VR,CONST,PHI,PRDG,PFI,ARP,VRI,CCV,LYAP,SCC,fct
      integer for, fty
      common/model/for, fty
      integer ARI, tra
      common/model/ari,tra

      DO 100 I=1, ARP+1
         PHI1(I)=PHI(I)
 100  CONTINUE
      RETURN
      END

      SUBROUTINE D7RPLY(UU, VV, NZ, P, QP, K, QK, SVK)
C VARIABLE-SHIFT K-POLYNOMIAL ITERATION FOR A
C QUADRATIC FACTOR CONVERGES ONLY IF THE ZEROS ARE
C EQUIMODULAR OR NEARLY SO.
C UU,VV - COEFFICIENTS OF STARTING QUADRATIC
C NZ - NUMBER OF ZERO FOUND
C
      COMMON /P77PLY/ SR, SI, U,
     1 V, A, B, C, D, A1, A2, A3, A6, A7, E, F, G,
     2 H, SZR, SZI, LZR, LZI, ETA, ARE, MRE, N, NN
C
      INTEGER N, NN
      INTEGER NZ, TYPE, I, J
C
      DOUBLE PRECISION ETA, ARE, MRE
      DOUBLE PRECISION MP, OMP, EE, RELSTP, T, ZM
C
      DOUBLE PRECISION P(*), QP(*), K(*),
     1   QK(*), SVK(*), SR, SI, U, V, A, B, C, D,
     2   A1, A2, A3, A6, A7, E, F, G, H, SZR, SZI,
     3   LZR, LZI
C
      DOUBLE PRECISION UI, VI, UU, VV
      LOGICAL TRIED
      SVK(1) = SVK(1)
C
      NZ = 0
      TRIED = .FALSE.
      U = UU
      V = VV
      J = 0
      OMP = 0.D0
      RELSTP = 0.D0
C MAIN LOOP
   10 CALL D6RPLY(1.D0, U, V, SZR, SZI, LZR, LZI)
C RETURN IF ROOTS OF THE QUADRATIC ARE REAL AND NOT
C CLOSE TO MULTIPLE OR NEARLY EQUAL AND  OF OPPOSITE
C SIGN
      IF (DABS(DABS(SZR)-DABS(LZR)).GT..01D0*
     1   DABS(LZR)) RETURN
C EVALUATE POLYNOMIAL BY QUADRATIC SYNTHETIC DIVISION
      CALL D8RPLY(NN, U, V, P, QP, A, B)
      MP = DABS(A-SZR*B) + DABS(SZI*B)
C COMPUTE A RIGOROUS  BOUND ON THE ROUNDING ERROR IN
C EVALUTING P
      ZM = SQRT(ABS(V))
      EE = 2.*ABS(QP(1))
      T = -SZR*B
      DO 20 I=2,N
        EE = EE*ZM + ABS(QP(I))
   20 CONTINUE
      EE = EE*ZM + ABS(A)+T
      EE = (5.*MRE+4.*ARE)*EE - (5.*MRE+2.*ARE)*
     2   (ABS(A+T)+ABS(B)*ZM) +
     3   2.*ARE*ABS(T)
C ITERATION HAS CONVERGED SUFFICIENTLY IF THE
C POLYNOMIAL VALUE IS LESS THAN 20 TIMES THIS BOUND
      IF (MP.GT.20.*EE) GO TO 30
      NZ = 2
      RETURN
   30 J = J + 1
C STOP ITERATION AFTER 20 STEPS
      IF (J.GT.20) RETURN
      IF (J.LT.2) GO TO 50
      IF (RELSTP.GT..01 .OR. MP.LT.OMP .OR. TRIED)
     1   GO TO 50
C A CLUSTER APPEARS TO BE STALLING THE CONVERGENCE.
C FIVE FIXED SHIFT STEPS ARE TAKEN WITH A U,V CLOSE
C TO THE CLUSTER
      IF (RELSTP.LT.ETA) RELSTP = ETA
      RELSTP = SQRT(RELSTP)
      U = U - U*RELSTP
      V = V + V*RELSTP
      CALL D8RPLY(NN, U, V, P, QP, A, B)
      DO 40 I=1,5
        CALL D2RPLY(TYPE, K, QK)
        CALL D5RPLY(TYPE, QP, K, QK)
   40 CONTINUE
      TRIED = .TRUE.
      J = 0
   50 OMP = MP
C CALCULATE NEXT K POLYNOMIAL AND NEW U AND V
      CALL D2RPLY(TYPE, K, QK)
      CALL D5RPLY(TYPE, QP, K, QK)
      CALL D2RPLY(TYPE, K, QK)
      CALL D4RPLY(TYPE, UI, VI, P, K)
C IF VI IS ZERO THE ITERATION IS NOT CONVERGING
      IF (VI.EQ.0.D0) RETURN
      RELSTP = DABS((VI-V)/VI)
      U = UI
      V = VI
      GO TO 10
      END

*UPTODATE ORTRANTEXT
      SUBROUTINE ORTRAN(NM,N,LOW,IGH,A,ORT,Z)
C
C     .. Scalar Arguments ..
      INTEGER IGH,LOW,N,NM
C     .. Array Arguments ..
      DOUBLE PRECISION A(NM,IGH),ORT(IGH),Z(NM,N)
C     .. Local Scalars ..
      DOUBLE PRECISION G
      INTEGER I,J,KL,MM,MP,MP1
C     .. Executable Statements ..
C
C     THIS SUBROUTINE IS A TRANSLATION OF THE ALGOL PROCEDURE ORTRANS,
C     NUM. MATH. 16, 181-204(1970) BY PETERS AND WILKINSON.
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 372-395(1971).
C
C     THIS SUBROUTINE ACCUMULATES THE ORTHOGONAL SIMILARITY
C     TRANSFORMATIONS USED IN THE REDUCTION OF A REAL GENERAL
C     MATRIX TO UPPER HESSENBERG FORM BY  ORTHES.
C
C     ON INPUT-
C
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
C          DIMENSION STATEMENT,
C
C        N IS THE ORDER OF THE MATRIX,
C
C        LOW AND IGH ARE INTEGERS DETERMINED BY THE BALANCING
C          SUBROUTINE  BALANC.  IF  BALANC  HAS NOT BEEN USED,
C          SET LOW=1, IGH=N,
C
C        A CONTAINS INFORMATION ABOUT THE ORTHOGONAL TRANS-
C          FORMATIONS USED IN THE REDUCTION BY  ORTHES
C          IN ITS STRICT LOWER TRIANGLE,
C
C        ORT CONTAINS FURTHER INFORMATION ABOUT THE TRANS-
C          FORMATIONS USED IN THE REDUCTION BY  ORTHES.
C          ONLY ELEMENTS LOW THROUGH IGH ARE USED.
C
C     ON OUTPUT-
C
C        Z CONTAINS THE TRANSFORMATION MATRIX PRODUCED IN THE
C          REDUCTION BY  ORTHES,
C
C        ORT HAS BEEN ALTERED.
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO B. S. GARBOW,
C     APPLIED MATHEMATICS DIVISION, ARGONNE NATIONAL LABORATORY
C
C     ------------------------------------------------------------------
C
C     ********** INITIALIZE Z TO IDENTITY MATRIX **********
      DO 40 I = 1,N
C
          DO 20 J = 1,N
              Z(I,J) = 0.0D0
   20     CONTINUE
C
          Z(I,I) = 1.0D0
   40 CONTINUE
C
      KL = IGH - LOW - 1
      IF (KL.LT.1) GO TO 160
C     ********** FOR MP=IGH-1 STEP -1 UNTIL LOW+1 DO -- **********
      DO 140 MM = 1,KL
          MP = IGH - MM
          IF (A(MP,MP-1).EQ.0.0D0) GO TO 140
          MP1 = MP + 1
C
          DO 60 I = MP1,IGH
              ORT(I) = A(I,MP-1)
   60     CONTINUE
C
          DO 120 J = MP,IGH
              G = 0.0D0
C
              DO 80 I = MP,IGH
                  G = G + ORT(I)*Z(I,J)
   80         CONTINUE
C     ********** DIVISOR BELOW IS NEGATIVE OF H FORMED IN ORTHES.
C                DOUBLE DIVISION AVOIDS POSSIBLE UNDERFLOW **********
              G = (G/ORT(MP))/A(MP,MP-1)
C
              DO 100 I = MP,IGH
                  Z(I,J) = Z(I,J) + G*ORT(I)
  100         CONTINUE
C
  120     CONTINUE
C
  140 CONTINUE
C
  160 RETURN
C     ********** LAST CARD OF ORTRAN **********
      END

      SUBROUTINE COMPLETE(SS, BIT)
      IMPLICIT NONE
      INTEGER I,J,K
      EXTERNAL DBLEPR, INTPR
      DOUBLE PRECISION U
c      INCLUDE 'model.txt'
c      INCLUDE 'conpar.txt'
c      INCLUDE 'setcon.txt'
c      INCLUDE 'series.txt'
c      INCLUDE 'veccom.txt'
c      INCLUDE 'redcom.txt'
c      INCLUDE 'seacom.txt'
c      INCLUDE 'repar3.txt'
C*****model.txt
      INTEGER PFI,ARP,VRI,CCV,LYAP,SCC,fct
      DOUBLE PRECISION SCALE,VR,CONST,PHI(20),PRDG
      COMMON/MODEL/SCALE,VR,CONST,PHI,PRDG,PFI,ARP,VRI,CCV,LYAP,SCC,fct
      integer for, fty
      common/model/for, fty
      integer ARI, tra
      common/model/ari,tra

C*****conpar.txt    
      INTEGER NIT,OPM,RGM,KUP,KSP,KST
      DOUBLE PRECISION REQ,CONCRIT,RPERT,IVLAM,FAC,STLAM,SMLAM,GTLAM
c      COMMON/CONPAR/NIT,OPM,RGM,REQ,CONCRIT,RPERT,IVLAM,FAC,STLAM,
c     * SMLAM,GTLAM,KUP,KSP,KST
      COMMON/CONPAR/REQ,CONCRIT,RPERT,IVLAM,FAC,STLAM,
     * SMLAM,GTLAM,NIT,OPM,RGM,KUP,KSP,KST

C*****setcon.txt
      LOGICAL CONV,FAIL
      INTEGER NP,ITCT,PPIND
      DOUBLE PRECISION CSO,CSZ,LAM,SSOLD,GMOLD,GMNEW,SIGSQ,OLDB(22)
c      COMMON/SETCON/CONV,FAIL,NP,ITCT,PPIND,CSO,CSZ,LAM,SSOLD,
c     *GMOLD,GMNEW,SIGSQ,OLDB
      COMMON/SETCON/CSO,CSZ,LAM,SSOLD,
     *GMOLD,GMNEW,SIGSQ,OLDB,CONV,FAIL,NP,ITCT,PPIND
C*****series.txt
      CHARACTER(LEN=40) NAME
      INTEGER LEN
      DOUBLE PRECISION TIM(5000),SER(5000),TDIF(5000)
c      COMMON/SERIES/NAME,LEN,TIM,SER,TDIF
      COMMON/SERIES/TIM,SER,TDIF,NAME,LEN
C*****veccom.txt
      DOUBLE PRECISION B(22),NEWB(22),PERB(22),DELB(22),
     * ERR(5000),RES(5000),PRES(22,5000),sres(5000)
      COMMON/VECCOM/B,NEWB,PERB,DELB,ERR,RES,PRES,sres
C*****redcom.txt
      DOUBLE PRECISION GRAD(22),SSP(22,22)
      COMMON/REDCOM/GRAD,SSP
C*****seacom.txt
      LOGICAL SOLV
      DOUBLE PRECISION ESSP(22,22),ECOV(22,22)
c      COMMON/SEACOM/SOLV,ESSP,ECOV
      COMMON/SEACOM/ESSP,ECOV,SOLV

C*****repar3.txt
      DOUBLE PRECISION ALPHA(21),ROOTR(20),ROOTI(20),XVECR(20),
     *XVECI(20)
      COMMON/REPAR3/ALPHA,ROOTR,ROOTI,XVECR,XVECI
      DOUBLE PRECISION SS, BIT(22) 
      INTEGER IDUM(1)
      DOUBLE PRECISION DDUM(1)

C      CALL NEWLINE
      if(tra.EQ.1)then
        call NEWLINE
        call intpr('PROCESSING COMPLETED', 20, IDUM, 0) 
      end if
      IF(ITCT.GE.NIT)THEN
        I=NIT
      ELSE
        I=ITCT
      END IF
      if(tra.EQ.1)then
       IDUM(1) = I
       call intpr('ITERATIONS COMPLETED:', 21, IDUM, 1)
C      end if
      IF(CONV)THEN
        call intpr('CONVERGENCE ACHIEVED', 20, IDUM, 0)
      ELSE
        call intpr('CONVERGENCE NOT ACHIEVED', 24, IDUM, 0)
      END IF
      IF(FAIL)THEN
        call intpr('SEARCH FAILED TO PROGRESS', 25, IDUM, 0)
      ELSE
C       if(tra.EQ.1)then
        call intpr('SEARCH PROGRESSED', 17, IDUM, 0) 
C       end if
      END IF
      END IF
      DO I=1,NP
        DELB(I)=0.0D0
        DO J=1,NP
          ESSP(I,J)=SSP(I,J)
        END DO
      END DO
      CALL SIMI
      K=LEN-ARP
      IF(VRI.EQ.1)K=K-1
      IF(CCV.EQ.2)K=K-1
      U=SSOLD/K
      SIGSQ=U/GMOLD
      DO I=1,NP
        DO J=1,NP
          ESSP(I,J)=ESSP(I,J)*U  
        END DO
      END DO
c added by Z.W.
c      OPEN(UNIT=4,FILE='cov.dat',STATUS='NEW')
c      OPEN(UNIT=4,FILE='cov.dat',STATUS='unknown')
c      write(4,95) ((ESSP(I,J),J=1,NP-1),I=1,NP-1)
c   95 FORMAT(5E16.8)
c Note: NP-th parameter is the constant estimator in the last line
c      close(unit=4)
C     for model selection purpose, find covariance matrix ECOV
      DO I=1,ARP
        DO J=1,ARP
          ECOV(I,J)=ESSP(I,J)
        END DO
      END DO                                  
c end by Z.W.   
      DO I=1,NP
        DELB(I)=DSQRT(ESSP(I,I))
      END DO
      DO I=1,NP
        U=DELB(I)
        DO J=1,NP
          ESSP(I,J)=ESSP(I,J)/(U*DELB(J))
        END DO
      END DO
      if(tra.EQ.1)then
      CALL NEWLINE
       DDUM(1) = SSOLD
       call dblepr('FINAL SUM OF SQUARES: ', 22, DDUM, 1)
       DDUM(1) = SIGSQ*GMOLD
       call dblepr('MEAN SUM OF SQUARES : ', 22, DDUM, 1)
       DDUM(1) = SIGSQ
       call dblepr('INNOVATION PROCESS VARIANCE ESTIMATE: ',38, DDUM,
     *1) 
       DDUM(1) = GMOLD
       call dblepr('GEOMETRIC MEAN VARIANCE MULTIPLIER:   ',38, DDUM,
     *1) 
       DDUM(1) = OLDB(1)
       call dblepr('FINAL PARAMETER VALUES:',23, DDUM, 1)
      end if
      DO 104 I=1,NP
      B(I)=OLDB(I)
C      if(tra.EQ.1)then
C      PRINT *,'   ',I,'  ',B(I),'   +/- ',DELB(I)
C      end if
  104 CONTINUE
      if(tra.EQ.1)then
      CALL NEWLINE
C      PRINT *,'CORRELATION MATRIX:'
c      DO 106 I=1,NP
c      WRITE(6,105)(ESSP(I,J),J=1,I)
c  105 FORMAT(1H ,7(D10.3,2X))
c  106 CONTINUE
      end if
      SS = SSOLD
      DO 107 I=1,NP
      BIT(I)=OLDB(I)
  107 CONTINUE
c      OPEN(UNIT=4,FILE='parest.dat',STATUS='NEW')
c      open(unit=4,file='parest.dat',status='unknown')
c      WRITE(4,195)
c  195 FORMAT('MOD.PAR.IND., ORDER, NO.PAR., VAR.RAT.IND., CON.IND.')
c      WRITE(4,196) PFI,ARP,NP,VRI,CCV
c  196 FORMAT(5I10)
c      WRITE(4,197)
c  197 FORMAT('SCALE , INNOVATION VARIANCE')
c      WRITE(4,198) SCALE,SIGSQ
c  198 FORMAT(5E16.8)
c      WRITE(4,199)
c  199 FORMAT('PARAMETERS')
c      WRITE(4,198) (B(I),I=1,NP)
c      WRITE(4,200)
c  200 FORMAT('STANDARD ERRORS')
c      WRITE(4,198) (DELB(I),I=1,NP)
c      WRITE(4,201)
c  201 FORMAT('CORRELATIONS')
c      WRITE(4,198) ((ESSP(I,J),J=1,I),I=1,NP)
c      WRITE(4,202)
      CALL ROOTS
c  202 FORMAT('ROOTS:REAL PARTS THEN IMAGINARY')
c      WRITE(4,198) (ROOTR(I),I=1,ARP)
c      WRITE(4,198) (ROOTI(I),I=1,ARP)
c      CLOSE(UNIT=4)
c added by Z.W.
c     the last element of phi.dat is the estimate of the mean
c      OPEN(UNIT=4,FILE='phi.dat',STATUS='NEW')
c      OPEN(UNIT=4,FILE='phi.dat',STATUS='unknown')
c      WRITE(4,300) (B(I),I=1,NP)
c  300 FORMAT(5E16.8)
c      CLOSE(UNIT=4)
c end by Z.W.
      RETURN
      END

*UPTODATE BALANCTEXT
      SUBROUTINE BALANC(NM,N,A,LOW,IGH,SCALE)
C
C     .. Scalar Arguments ..
      INTEGER IGH,LOW,N,NM
C     .. Array Arguments ..
      DOUBLE PRECISION A(NM,N),SCALE(N)
C     .. Scalars in Common ..
      DOUBLE PRECISION BASE,BIG,EPS,SMALL
C     .. Local Scalars ..
      DOUBLE PRECISION B2,C,F,G,R,RADIX,S
      INTEGER I,IEXC,J,JJ,K,L,M
      LOGICAL NOCONV
C     .. External Subroutines ..
      EXTERNAL SDATA
C     .. Intrinsic Functions ..
      INTRINSIC ABS
C     .. Common blocks ..
      COMMON /MDEPS/BASE,BIG,SMALL,EPS
C     .. Executable Statements ..
C
C     THIS SUBROUTINE IS A TRANSLATION OF THE ALGOL PROCEDURE BALANCE,
C     NUM. MATH. 13, 293-304(1969) BY PARLETT AND REINSCH.
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 315-326(1971).
C
C     THIS SUBROUTINE BALANCES A REAL MATRIX AND ISOLATES
C     EIGENVALUES WHENEVER POSSIBLE.
C
C     ON INPUT-
C
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
C          DIMENSION STATEMENT,
C
C        N IS THE ORDER OF THE MATRIX,
C
C        A CONTAINS THE INPUT MATRIX TO BE BALANCED.
C
C     ON OUTPUT-
C
C        A CONTAINS THE BALANCED MATRIX,
C
C        LOW AND IGH ARE TWO INTEGERS SUCH THAT A(I,J)
C          IS EQUAL TO ZERO IF
C           (1) I IS GREATER THAN J AND
C           (2) J=1,...,LOW-1 OR I=IGH+1,...,N,
C
C        SCALE CONTAINS INFORMATION DETERMINING THE
C           PERMUTATIONS AND SCALING FACTORS USED.
C
C     SUPPOSE THAT THE PRINCIPAL SUBMATRIX IN ROWS LOW THROUGH IGH
C     HAS BEEN BALANCED, THAT P(J) DENOTES THE INDEX INTERCHANGED
C     WITH J DURING THE PERMUTATION STEP, AND THAT THE ELEMENTS
C     OF THE DIAGONAL MATRIX USED ARE DENOTED BY D(I,J).  THEN
C        SCALE(J) = P(J),    FOR J = 1,...,LOW-1
C                 = D(J,J),      J = LOW,...,IGH
C                 = P(J)         J = IGH+1,...,N.
C     THE ORDER IN WHICH THE INTERCHANGES ARE MADE IS N TO IGH+1,
C     THEN 1 TO LOW-1.
C
C     NOTE THAT 1 IS RETURNED FOR IGH IF IGH IS ZERO FORMALLY.
C
C     THE ALGOL PROCEDURE EXC CONTAINED IN BALANCE APPEARS IN
C     BALANC  IN LINE.  (NOTE THAT THE ALGOL ROLES OF IDENTIFIERS
C     K,L HAVE BEEN REVERSED.)
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO B. S. GARBOW,
C     APPLIED MATHEMATICS DIVISION, ARGONNE NATIONAL LABORATORY
C
C     ------------------------------------------------------------------
C
      CALL SDATA
      RADIX = BASE
C
      B2 = RADIX*RADIX
      K = 1
      L = N
      GO TO 120
C     ********** IN-LINE PROCEDURE FOR ROW AND
C                COLUMN EXCHANGE **********
   20 SCALE(M) = J
      IF (J.EQ.M) GO TO 80
C
      DO 40 I = 1,L
          F = A(I,J)
          A(I,J) = A(I,M)
          A(I,M) = F
   40 CONTINUE
C
      DO 60 I = K,N
          F = A(J,I)
          A(J,I) = A(M,I)
          A(M,I) = F
   60 CONTINUE
C
   80 IF (IEXC.EQ.1) THEN
          GO TO 100
      ELSE
          GO TO 180
      END IF
C     ********** SEARCH FOR ROWS ISOLATING AN EIGENVALUE
C                AND PUSH THEM DOWN **********
  100 IF (L.EQ.1) GO TO 460
      L = L - 1
C     ********** FOR J=L STEP -1 UNTIL 1 DO -- **********
  120 DO 160 JJ = 1,L
          J = L + 1 - JJ
C
          DO 140 I = 1,L
              IF (I.EQ.J) GO TO 140
              IF (A(J,I).NE.0.0D0) GO TO 160
  140     CONTINUE
C
          M = L
          IEXC = 1
          GO TO 20
  160 CONTINUE
C
      GO TO 200
C     ********** SEARCH FOR COLUMNS ISOLATING AN EIGENVALUE
C                AND PUSH THEM LEFT **********
  180 K = K + 1
C
  200 DO 240 J = K,L
C
          DO 220 I = K,L
              IF (I.EQ.J) GO TO 220
              IF (A(I,J).NE.0.0D0) GO TO 240
  220     CONTINUE
C
          M = K
          IEXC = 2
          GO TO 20
  240 CONTINUE
C     ********** NOW BALANCE THE SUBMATRIX IN ROWS K TO L **********
      DO 260 I = K,L
          SCALE(I) = 1.0D0
  260 CONTINUE
C     ********** ITERATIVE LOOP FOR NORM REDUCTION **********
  280 NOCONV = .FALSE.
C
      DO 440 I = K,L
          C = 0.0D0
          R = 0.0D0
C
          DO 300 J = K,L
              IF (J.EQ.I) GO TO 300
              C = C + ABS(A(J,I))
              R = R + ABS(A(I,J))
  300     CONTINUE
C     ********** GUARD AGAINST ZERO C OR R DUE TO UNDERFLOW **********
          IF (C.EQ.0.0D0 .OR. R.EQ.0.0D0) GO TO 440
          G = R/RADIX
          F = 1.0D0
          S = C + R
  320     IF (C.GE.G) GO TO 340
          F = F*RADIX
          C = C*B2
          GO TO 320
  340     G = R*RADIX
  360     IF (C.LT.G) GO TO 380
          F = F/RADIX
          C = C/B2
          GO TO 360
C     ********** NOW BALANCE **********
  380     IF ((C+R)/F.GE.0.95D0*S) GO TO 440
          G = 1.0D0/F
          SCALE(I) = SCALE(I)*F
          NOCONV = .TRUE.
C
          DO 400 J = K,N
              A(I,J) = A(I,J)*G
  400     CONTINUE
C
          DO 420 J = 1,L
              A(J,I) = A(J,I)*F
  420     CONTINUE
C
  440 CONTINUE
C
      IF (NOCONV) GO TO 280
C
  460 LOW = K
      IGH = L
      RETURN
C     ********** LAST CARD OF BALANC **********
      END

      SUBROUTINE SEARCH
      IMPLICIT NONE
      INTEGER I,J
      DOUBLE PRECISION U,V
c      INCLUDE 'setcon.txt'
c      INCLUDE 'veccom.txt'
c      INCLUDE 'redcom.txt'
c      INCLUDE 'seacom.txt'
C*****setcon.txt
      LOGICAL CONV,FAIL
      INTEGER NP,ITCT,PPIND
      DOUBLE PRECISION CSO,CSZ,LAM,SSOLD,GMOLD,GMNEW,SIGSQ,OLDB(22)
c      COMMON/SETCON/CONV,FAIL,NP,ITCT,PPIND,CSO,CSZ,LAM,SSOLD,
c     *GMOLD,GMNEW,SIGSQ,OLDB
      COMMON/SETCON/CSO,CSZ,LAM,SSOLD,
     *GMOLD,GMNEW,SIGSQ,OLDB,CONV,FAIL,NP,ITCT,PPIND
C*****veccom.txt
      DOUBLE PRECISION B(22),NEWB(22),PERB(22),DELB(22),
     * ERR(5000),RES(5000),PRES(22,5000),sres(5000)
      COMMON/VECCOM/B,NEWB,PERB,DELB,ERR,RES,PRES,sres
C*****redcom.txt
      DOUBLE PRECISION GRAD(22),SSP(22,22)
      COMMON/REDCOM/GRAD,SSP
C*****seacom.txt
      LOGICAL SOLV
      DOUBLE PRECISION ESSP(22,22),ECOV(22,22)
c      COMMON/SEACOM/SOLV,ESSP,ECOV
      COMMON/SEACOM/ESSP,ECOV,SOLV

      V=CSO+LAM
      DO I=1,NP
      DELB(I)=GRAD(I)
      DO J=1,NP
      U=SSP(I,J)
      IF(I.EQ.J)U=U*V
      ESSP(I,J)=U
      END DO
      END DO
      CALL SIMI
      IF(SOLV)THEN
        DO 101 I=1,NP
        NEWB(I)=OLDB(I)-DELB(I)
  101   CONTINUE
      END IF
      RETURN
      END

kalsmo <- function(object)
    UseMethod("kalsmo")

"kalsmo.car" <- function(object)
{
  par <- object$phi
  n <- object$n.used
  p <- object$order
  k <- object$scale
    TRAN <- array(0, dim = c(p, p, n))
    SSCV <- array(0, dim = c(p, p, n))
    PSCV <- array(0, dim = c(p, p, n))
    FSCV <- array(0, dim = c(p, p, n))
    FSES <- matrix(0, n, p)
    PSES <- matrix(0, n, p)
    SSES <- matrix(0, n, p)
    SSER <- matrix(0, n, p)
    SVAR <- rep(0, n)
    filcomp <- matrix(0, n, p)
    dl <- as.complex(rep(0, p))
#    delta <- diff(series[, 1])
  delta <- diff(object$tim)
    delta <- c(0, delta)
#    z.obs <- series[, 2] - par[p + 1]
  z.obs <- object$ser - object$x.mean# detrend mean
    K <- c(rep(0, p - 1), 1)
    H <- choose(p - 1, k = 0:(p - 1))/k^(0:(p - 1))
#    w <- sort(polyroot(rev(c(1, par[1:p]))))
    w <- sort(round(polyroot(rev(c(1, par[1:p]))),4)) 
#   sortind <- sort.list(abs(object$rootr), decreasing = TRUE)
#    carroot <- complex(real = object$rootr, imag = object$rooti)
#    w <- carroot <- rev(sort(carroot, partial = sortind))
    r <- -k * (1 - w)/(1 + w)
    sortind <- sort.list(abs(Re(r)), decreasing = TRUE)
    r <- rev(r[sortind]) #changed  
    ###r <- rev(sort(r, partial = sortind))
    R <- matrix(0, p, p)
    E <- matrix(0, p, p)
    P <- matrix(0, p, p)
    for (i in 1:p) R[i, ] <- r^(i - 1)
    R <- round(R, 4)
    D <- diag(r)
   J <- solve(R)[, p]
    G <- H %*% R
    state <- matrix(0, n + 1, p)
    innovation <- v <- rep(0, n)
    z.est <- rep(0, n)
    sse <- 0
    P0 <- matrix(0, p, p)
    for (i in 1:p) for (j in 1:p) P0[i, j] <- -J[i] * Conj(J[j])/(r[i] + 
        Conj(r[j]))
    P <- P0
    i <- 1
    while (i < n + 1) {
        dl <- exp(r * delta[i])
        E <- diag(dl)
        TRAN[, , i] <- E
        state[i, ] <- dl * state[i, ]
        PSES[i, ] <- state[i, ]
        P <- E %*% (P - P0) %*% Conj(E) + P0
        PSCV[, , i] <- P
        z.est[i] <- Re(G %*% state[i, ])
        innovation[i] <- z.obs[i] - z.est[i]
        v[i] <- Re(G %*% P %*% t(Conj(G)))
        gain <- P %*% Conj(t(G))/v[i]
        state[i + 1, ] <- state[i, ] + gain * innovation[i]
        FSES[i, ] <- state[i + 1, ]
        filcomp[i, ] <- G * state[i + 1, ]
        sse <- sse + innovation[i]^2/v[i]
        P <- P - gain %*% Conj(t(gain)) * v[i]
        FSCV[, , i] <- P
        i <- i + 1
    }
    i <- n
    while (i > 0) {
        if (i < n) {
            P.star <- FSCV[, , i] %*% Conj(TRAN[, , (i + 1)]) %*% 
                solve(PSCV[, , (i + 1)])
        }
        SSES[i, ] <- FSES[i, ]
        SSCV[, , i] <- FSCV[, , i]
        if (i < n) {
            SSES[i, ] <- FSES[i, ] + P.star %*% (SSES[i + 1, 
                ] - PSES[i + 1, ])
            SSCV[, , i] <- FSCV[, , i] + P.star %*% (SSCV[, , 
                (i + 1)] - PSCV[, , (i + 1)]) %*% Conj(P.star)
        }
        SSER[i, ] <- G * SSES[i, ]
        SVAR[i] <- Re(G %*% SSCV[, , i] %*% t(Conj(G)))
        i <- i - 1
    }
    innovation <- round(innovation, 4)
    v <- round(v, 4)
    state <- round(state, 4)
    vwhite <- sse/(n - object$np)
    RET <- list(tim=object$tim,ser=object$ser,sse = Re(sse), vwhite = Re(vwhite), state = state, innovation = innovation, 
        filcomp = filcomp, v = v, roots = r, sses = SSES, sser = SSER, 
        svar = SVAR)
}

*UPTODATE LYCSRTEXT
      SUBROUTINE LYCSR(N,A,NA,C,NC,IERR)
C
C        MARK 2.2 RELEASE.  NAG COPYRIGHT 1987.
C
C  WARNING  -This routine is intended to be called only from
C            SLICE   routine  LYBSC .
C
C     .. Scalar Arguments ..
      IMPLICIT NONE
      INTEGER IERR,N,NA,NC
C     .. Array Arguments ..
      DOUBLE PRECISION A(NA,N),C(NC,N)
C*PT*WARNING* Already double-precision
C     .. Local Scalars ..
      DOUBLE PRECISION DPREC
      INTEGER DK,DL,I,IA,INFO,J,JOB,K,KK,KM1,L,LDIM,LDL,LL,NSYS
C     .. Local Arrays ..
      DOUBLE PRECISION P(4),T(4,4)
      INTEGER IPVT(4)
C     .. External Subroutines ..
C      EXTERNAL SGEFA,SGESL
C      EXTERNAL DGETRF, DGETRS
C     .. Executable Statements ..
C
C     This routine solves the continuous Lyapunov equation where
C     the matrix  A  has been transformed to quasi-triangular form.
C

      IERR = 0
      LDIM = 4
      JOB = 0
      L = 1
   20 DL = 1
      IF (L.EQ.N) GO TO 40
      IF (A(L+1,L).NE.0.0D0) DL = 2
   40 LL = L + DL - 1
      K = L
   60 KM1 = K - 1
      DK = 1
      IF (K.EQ.N) GO TO 80
      IF (A(K+1,K).NE.0.0D0) DK = 2
   80 KK = K + DK - 1
      IF (K.EQ.L) GO TO 160
C
      DO 140 I = K,KK
C
          DO 120 J = L,LL
C*PT*WARNING* Constant already double-precision
              DPREC = 0.0D0
              DO 100 IA = L,KM1
                  DPREC = DPREC + A(IA,I)*C(IA,J)
  100         CONTINUE
              C(I,J) = C(I,J) - DPREC
  120     CONTINUE
  140 CONTINUE
C  
  160 IF (DL.EQ.2) GO TO 200
      IF (DK.EQ.2) GO TO 180
      T(1,1) = A(K,K) + A(L,L)
      IF (T(1,1).EQ.0.0D0) GO TO 360
      C(K,L) = C(K,L)/T(1,1)
      GO TO 260
  180 T(1,1) = A(K,K) + A(L,L)
      T(1,2) = A(KK,K)
      T(2,1) = A(K,KK)
      T(2,2) = A(KK,KK) + A(L,L)
      P(1) = C(K,L)
      P(2) = C(KK,L)
      NSYS = 2
      CALL DGETRF(NSYS, NSYS, T, LDIM,IPVT,INFO)
C      CALL SGEFA(T,LDIM,NSYS,IPVT,INFO)
      IF (INFO.NE.0) GO TO 360
C      CALL SGESL(T,LDIM,NSYS,IPVT,P,JOB)
      CALL DGETRS('N',NSYS, 1, T,LDIM, IPVT,P, LDIM, INFO)

      C(K,L) = P(1)
      C(KK,L) = P(2)
      GO TO 260
  200 IF (DK.EQ.2) GO TO 220
      T(1,1) = A(K,K) + A(L,L)
      T(1,2) = A(LL,L)
      T(2,1) = A(L,LL)
      T(2,2) = A(K,K) + A(LL,LL)
      P(1) = C(K,L)
      P(2) = C(K,LL)
      NSYS = 2
      CALL DGETRF(NSYS, NSYS, T, LDIM,IPVT,INFO)
C      CALL SGEFA(T,LDIM,NSYS,IPVT,INFO)
      IF (INFO.NE.0) GO TO 360
      CALL DGETRS('N',NSYS, 1, T,LDIM, IPVT,P, LDIM, INFO)
C      CALL SGESL(T,LDIM,NSYS,IPVT,P,JOB)
      C(K,L) = P(1)
      C(K,LL) = P(2)
      GO TO 260
  220 IF (K.NE.L) GO TO 240
      T(1,1) = A(L,L)
      T(1,2) = A(LL,L)
      T(1,3) = 0.0D0
      T(2,1) = A(L,LL)
      T(2,2) = A(L,L) + A(LL,LL)
      T(2,3) = T(1,2)
      T(3,1) = 0.0D0
      T(3,2) = T(2,1)
      T(3,3) = A(LL,LL)
      P(1) = C(L,L)*0.5D0
      P(2) = C(LL,L)
      P(3) = C(LL,LL)*0.5D0
      NSYS = 3
      CALL DGETRF(NSYS, NSYS, T, LDIM,IPVT,INFO)
C      CALL SGEFA(T,LDIM,NSYS,IPVT,INFO)
      IF (INFO.NE.0) GO TO 360
C      CALL SGESL(T,LDIM,NSYS,IPVT,P,JOB)
      CALL DGETRS('N',NSYS, 1, T,LDIM, IPVT,P, LDIM, INFO)
      C(L,L) = P(1)
      C(LL,L) = P(2)
      C(L,LL) = P(2)
      C(LL,LL) = P(3)
      GO TO 260
  240 T(1,1) = A(K,K) + A(L,L)
      T(1,2) = A(KK,K)
      T(1,3) = A(LL,L)
      T(1,4) = 0.0D0
      T(2,1) = A(K,KK)
      T(2,2) = A(KK,KK) + A(L,L)
      T(2,3) = 0.0D0
      T(2,4) = T(1,3)
      T(3,1) = A(L,LL)
C      call intpr('L', 1, L, 1)
C      call intpr('LL', 2, LL, 1)
C      call dblepr('T(3,1)', 6, T(3,1), 1)
      T(3,2) = 0.0D0
      T(3,3) = A(K,K) + A(LL,LL)
      T(3,4) = T(1,2)
      T(4,1) = 0.0D0
      T(4,2) = T(3,1)
C      call dblepr('T(4,2)', 6, T(4,2), 1)
      T(4,3) = T(2,1)
      T(4,4) = A(KK,KK) + A(LL,LL)
      P(1) = C(K,L)
      P(2) = C(KK,L)
      P(3) = C(K,LL)
      P(4) = C(KK,LL)
      NSYS = 4
      
      CALL DGETRF(NSYS, NSYS, T, LDIM,IPVT,INFO)
C      CALL SGEFA(T,LDIM,NSYS,IPVT,INFO)
      IF (INFO.NE.0) GO TO 360
C      CALL SGESL(T,LDIM,NSYS,IPVT,P,JOB)
      CALL DGETRS('N',NSYS, 1, T,LDIM, IPVT,P, LDIM, INFO)
      C(K,L) = P(1)
      C(KK,L) = P(2)
      C(K,LL) = P(3)
      C(KK,LL) = P(4)
  260 K = K + DK
      IF (K.LE.N) GO TO 60
      LDL = L + DL
      IF (LDL.GT.N) RETURN
C
      DO 340 J = LDL,N
C
          DO 280 I = L,LL
              C(I,J) = C(J,I)
  280     CONTINUE
C
          DO 320 I = J,N
C*PT*WARNING* Constant already double-precision
              DPREC = 0.0D0
              DO 300 K = L,LL
                  DPREC = DPREC + C(I,K)*A(K,J)
                  DPREC = DPREC + A(K,I)*C(K,J)
  300         CONTINUE
              C(I,J) = C(I,J) - DPREC
              C(J,I) = C(I,J)
  320     CONTINUE
  340 CONTINUE
C
      L = LDL
      GO TO 20
C
  360 IERR = 1
      RETURN
      END

"spec.ls" <-
function (x, y=NULL, spans = NULL, kernel = NULL, taper = 0.1, pad = 0, 
    fast = TRUE, type = "lomb", demean = FALSE, detrend = TRUE, 
    plot.it = TRUE, na.action = na.fail, ...) 
{
#    series <- deparse(substitute(x))
    if (NCOL(x)==2){
      series <- deparse(substitute(x))
      ti <- x[,1]
      x <- x[,2]}
#    series <- deparse(substitute(y))}
    else {
       series <- deparse(substitute(y))
      ti <- x
      x <- y
    }
    x <- na.action(as.ts(x))
    xfreq <- frequency(x)
    x <- as.matrix(x)
    N <- nrow(x)
    nser <- ncol(x)
    if (!is.null(spans)) 
        if (is.tskernel(spans)) 
            kernel <- spans
        else kernel <- kernel("modified.daniell", spans%/%2)
    if (!is.null(kernel) && !is.tskernel(kernel)) 
        stop("must specify spans or a valid kernel")
    if (detrend) {
        t <- ti - mean(ti)
        sumt2 <- sum(t^2)
        for (i in 1:ncol(x)) x[, i] <- x[, i] - mean(x[, i]) - 
            sum(x[, i] * t) * t/sumt2
    }
    else if (demean) {
        x <- sweep(x, 2, colMeans(x))
    }
    x <- spec.taper(x, taper)
    u2 <- (1 - (5/8) * taper * 2)
    u4 <- (1 - (93/128) * taper * 2)
    if (pad > 0) {
        x <- rbind(x, matrix(0, nrow = N * pad, ncol = ncol(x)))
        N <- nrow(x)
    }
    NewN <- if (fast) 
        nextn(N)
    else N
    x <- rbind(x, matrix(0, nrow = (NewN - N), ncol = ncol(x)))
    N <- nrow(x)
    Nspec <- floor(N/2)
    freq <- seq(from = xfreq/N, by = xfreq/N, length = Nspec)
    pgram <- array(NA, dim = c(N, ncol(x), ncol(x)))
    freq.temp <- seq(from = xfreq/N, by = xfreq/N, length = N)
    if (type == "lomb") {
        for (i in 1:ncol(x)) {
            for (j in 1:ncol(x)) {
                for (k in 1:length(freq.temp)) {
                  tao <- atan(sum(sin(2 * pi * freq.temp[k] * 
                                      ti))/sum(cos(2 * pi * freq.temp[k] * ti)))/(2 * freq.temp[k])
                  pgram[k, i, j] <- 0.5 * ((sum(x[1:length(ti)]* cos(2 * pi * freq.temp[k] * (ti - tao))))^2/sum((cos(2 * 
                                                                                                         pi * freq.temp[k] * (ti - tao)))^2) + (sum(x[1:length(ti)] *  sin(2 * pi * freq.temp[k] * (ti - tao))))^2/sum((sin(2 * pi * freq.temp[k] * (ti - tao)))^2))
                  pgram[1, i, j] <- 0.5 * (pgram[2, i, j] + pgram[N, 
                                                                  i, j])
                }
              }
          }
      }
    if (type == "ft") {
        for (i in 1:ncol(x)) {
            for (j in 1:ncol(x)) {
                for (k in 1:length(freq.temp)) pgram[k, i, j] <- ((sum(x * 
                  cos(2 * pi * freq.temp[k] * ti)))^2 + (sum(x * 
                  sin(2 * pi * freq.temp[k] * ti)))^2)/N
            }
        }
    }
    if (!is.null(kernel)) {
        for (i in 1:ncol(x)) for (j in 1:ncol(x)) pgram[, i, 
            j] <- kernapply(pgram[, i, j], kernel, circular = TRUE)
        df <- df.kernel(kernel)/(u4/u2^2)
        bandwidth <- bandwidth.kernel(kernel) * xfreq/N
    }
    else {
        df <- 2/(u4/u2^2)
        bandwidth <- sqrt(1/12) * xfreq/N
    }
    pgram <- pgram[1:Nspec, , , drop = FALSE]
    spec <- matrix(NA, nrow = Nspec, ncol = nser)
    for (i in 1:nser) spec[, i] <- Re(pgram[1:Nspec, i, i])
    if (nser == 1) {
        coh <- phase <- NULL
    }
    else {
        coh <- phase <- matrix(NA, nrow = Nspec, ncol = nser * 
            (nser - 1)/2)
        for (i in 1:(nser - 1)) {
            for (j in (i + 1):nser) {
                coh[, i + (j - 1) * (j - 2)/2] <- Mod(pgram[, 
                  i, j])^2/(spec[, i] * spec[, j])
                phase[, i + (j - 1) * (j - 2)/2] <- Arg(pgram[, 
                  i, j])
            }
        }
    }
    for (i in 1:nser) spec[, i] <- spec[, i]/u2
    spec <- drop(spec)
    spg.out <- list(freq = freq, spec = spec, coh = coh, phase = phase, 
        kernel = kernel, df = df, bandwidth = bandwidth, n.used = nrow(x), 
        series = series, snames = colnames(x), method = ifelse(!is.null(kernel), 
            "Smoothed Lomb-Scargle Periodogram", "Raw Lomb-Scargle Periodogram"), 
        taper = taper, pad = pad, detrend = detrend, demean = demean)
    class(spg.out) <- "spec"
    if (plot.it) {
        plotSpecLs(spg.out, ...)
        return(invisible(spg.out))
    }
    else return(spg.out)
}

kalsmoComp <- 
  function (x, comp=NULL, plot.it = TRUE, xlab="time",ylab="",na.action = na.fail, ...) {
    cn <- match(c("sser"), names(x))
    if (any(is.na(cn))) 
      stop("x must be a kalsmo.car() object")
    if(length(comp)==1)
    compser <- Re(x$sser[,comp])
    else
    compser <- Re(apply(x$sser[,comp],1,sum))
    ntim <- length(x$ser)
    spg.out <- list(tim=x$tim[1:ntim],compser=compser, method = paste("CAR (", 
                               comp, ") component ", sep = ""))
    class(spg.out) <- "kalsmo.comp"
    if (plot.it) {
      plot(spg.out$tim,spg.out$compser,xlab=xlab,ylab=ylab,type="l",...)
      return(invisible(spg.out))
    }
    else return(spg.out)
  }

      SUBROUTINE MPRINT(N,A,K,TX)
      IMPLICIT NONE
      INTEGER N,K,I
      DOUBLE PRECISION A(K,*)
      CHARACTER(LEN=*) TX
      I = LEN(TX)
      IF (N .LT. 0 .AND. K .GT. 0) A(1,1) = A(1,1)
C     WRITE(*,*) 'MATRIX ', TX
      DO 10 I=1,N
C        WRITE(*,*) A(I,1:N)
   10 CONTINUE
      END
C***********   
      SUBROUTINE MPROD(N,A,B,C,K)
      INTEGER N,K,I,J,L
      DOUBLE PRECISION A(K,*),B(K,*),C(K,*),U
      DO I=1,N
        DO J=1,N
          U=0.D0
          DO L=1,N
            U=U+B(I,L)*C(L,J)
          END DO
          A(I,J)=U
        END DO
      END DO
      END
C**********
      SUBROUTINE MTPROD(N,A,B,C,K)
      INTEGER N,K,J,L
      DOUBLE PRECISION A(K,*),B(K,*),C(K,*),U
      DO I=1,N
        DO J=1,N
          U=0.D0
          DO L=1,N
            U=U+B(L,I)*C(L,J)
          END DO
          A(I,J)=U
        END DO
      END DO
      END
C***********
      SUBROUTINE MDIFF(N,A,B,C,K)
      INTEGER N,K,I,J
      DOUBLE PRECISION A(K,*),B(K,*),C(K,*)
      DO I=1,N
        DO J=1,N
          A(I,J)=B(I,J)-C(I,J)
        END DO
      END DO
      END
C*********
      SUBROUTINE MSPROD(N,A,B,C,K)
      INTEGER N,K,I,J,L
      DOUBLE PRECISION A(K,*),B(K,*),C(K,*),U
      DO I=1,N
        DO J=1,N
          U=0.D0
          DO L=1,N
            U=U+C(I,L)*B(J,L)
          END DO
          A(I,J)=U
        END DO
      END DO
      DO I=1,N
        DO J=I,N
          U=0.D0
          DO L=1,N
            U=U+B(I,L)*A(L,J)
          END DO
          A(J,I)=U
        END DO
      END DO
      IF (N.GT.1)THEN
        M=N-1
        DO J=2,N
          DO I=1,J-1
            A(I,J)=A(J,I)
          END DO
        END DO
      ENDIF 
      END

### Adopted from plot.spec R code
 spec.ci <- function (spec.obj, coverage = 0.95)
    {
        ## A utility function for plot.spec which calculates the confidence
        ## interval (centred around zero). We use a conditional argument to
        ## ensure that the ci always contains zero.

        if (coverage < 0 || coverage >= 1)
            stop("coverage probability out of range [0,1)")
        tail <- (1 - coverage)
        df <- spec.obj$df
        upper.quantile <- 1 - tail * pchisq(df, df, lower.tail = FALSE)
        lower.quantile <- tail * pchisq(df, df)
        1/(qchisq(c(upper.quantile, lower.quantile), df)/df)
    }

"plotSpecLs" <-
function (x, add = FALSE, ci = 0.95, log = c("yes", "dB", "no"), 
    xlab = "frequency", ylab = NULL, type = "l",
    main = NULL, sub = NULL, ...) 
{
    log <- match.arg(log)
    if (is.null(ylab)) 
        ylab <- if (log == "dB") 
            "spectrum (dB)"
        else "spectrum"
    if (is.logical(log)) 
        log <- if (log) 
            "yes"
        else "no"
    if (missing(log) && getOption("ts.S.compat")) 
        log <- "dB"
    log <- match.arg(log)
    ylog <- ""
    if (log == "dB") 
        x$spec <- 10 * log10(x$spec)
    if (log == "yes") 
        ylog <- "y"
    if (add) {
        matplot(x$freq, x$spec, type = type, add = TRUE, ...)
    }
    else {
        matplot(x$freq, x$spec, xlab = xlab, ylab = ylab, type = type, 
            log = ylog, ...)
        if (ci <= 0 || !is.numeric(x$df) || log == "no") {
            ci.text <- ""
        }
        else {
            conf.lim <- spec.ci(x, coverage = ci)
            if (log == "dB") {
                conf.lim <- 10 * log10(conf.lim)
                conf.y <- max(x$spec) - conf.lim[2]
                conf.x <- max(x$freq) - x$bandwidth
                lines(rep(conf.x, 2), conf.y + conf.lim, ...)
                lines(conf.x + c(-0.5, 0.5) * x$bandwidth, rep(conf.y, 
                  2))
                ci.text <- paste(", ", round(100 * ci, 2), "% C.I. is (", 
                  paste(format(conf.lim, digits = 3), collapse = ","), 
                  ")dB", sep = "")
            }
            else {
                ci.text <- ""
                conf.y <- max(x$spec)/conf.lim[2]
                conf.x <- max(x$freq) - x$bandwidth
                lines(rep(conf.x, 2), conf.y * conf.lim)
                lines(conf.x + c(-0.5, 0.5) * x$bandwidth, rep(conf.y, 
                  2))
            }
        }
        if (is.null(main)) 
            main <- paste(if (!is.null(x$series)) 
                paste("Series:", x$series)
            else "Lomb-Scargle peridogram", x$method, sep = "\n")
        if (is.null(sub) && is.numeric(x$bandwidth)) 
            sub <- paste("bandwidth = ", format(x$bandwidth, 
                digits = 3), ci.text, sep = "")
        title(main = main, sub = sub)
    }
    invisible(x)
}

*UPTODATE SSCALTEXT
      SUBROUTINE SSCAL(N,SA,SX,INCX)
C
C     SCALES A VECTOR BY A CONSTANT.
C     USES UNROLLED LOOPS FOR INCREMENT EQUAL TO 1.
C     JACK DONGARRA, LINPACK, 3/11/78.
C
C     .. Scalar Arguments ..
      DOUBLE PRECISION SA
      INTEGER INCX,N
C     .. Array Arguments ..
      DOUBLE PRECISION SX(*)
C     .. Local Scalars ..
      INTEGER I,M,MP1,NINCX
C     .. Intrinsic Functions ..
      INTRINSIC MOD
C     .. Executable Statements ..
C
      IF (N.LE.0) RETURN
      IF (INCX.EQ.1) GO TO 40
C
C        CODE FOR INCREMENT NOT EQUAL TO 1
C
      NINCX = N*INCX
      DO 20 I = 1,NINCX,INCX
          SX(I) = SA*SX(I)
   20 CONTINUE
      RETURN
C
C        CODE FOR INCREMENT EQUAL TO 1
C
C
C        CLEAN-UP LOOP
C
   40 M = MOD(N,5)
      IF (M.EQ.0) GO TO 80
      DO 60 I = 1,M
          SX(I) = SA*SX(I)
   60 CONTINUE
      IF (N.LT.5) RETURN
   80 MP1 = M + 1
      DO 100 I = MP1,N,5
          SX(I) = SA*SX(I)
          SX(I+1) = SA*SX(I+1)
          SX(I+2) = SA*SX(I+2)
          SX(I+3) = SA*SX(I+3)
          SX(I+4) = SA*SX(I+4)
  100 CONTINUE
      RETURN
      END

"plot.car" <-
function (x, type=c("spec", "pred", "diag"), ...)  
{
    if(!inherits(x, "car"))
    stop("Not a 'car' object\n")
    type <- match.arg(type)
    if(type=="spec")
    plot(x, ...)
    #plot.spec(x, ...)
    if(type=="pred")
    plot.predict.car(x, ...)
    if(type=="diag")
    diag(x, ...)
    invisible(x)
}

*UPTODATE ENVIRNTEXT
      SUBROUTINE ENVIRN(BETA,T,EPS,EMIN,RMIN)
C
C        MARK 2.2 RELEASE.  NAG COPYRIGHT 1987.
C
C     ENVIRN returns the machine parameters given by:
C
C        BETA - INTEGER.
C               The base of the machine.
C
C        T    - INTEGER.
C               The number of ( BETA ) digits in the mantissa.
C
C        EPS  - REAL.
C               The smallest positive number such that
C
C                  fl( 1.0 - EPS ) .lt. 1.0,
C
C               where fl denotes the computed value.
C
C        EMIN - INTEGER.
C               The minimum exponent before (gradual) underflow occurs.
C
C        RMIN - REAL.
C               The smallest normalized number for the machine given by
C               BASE**( EMIN - 1 ), where BASE is the floating point
C               value of BETA.
C
C
C     The computation of EPS, EMIN and RMIN is based on a routine,
C     PARANOIA by W. Kahan of the University of California at Berkeley.
C
C
C  Nag Fortran 77 O( 1 ) basic linear algebra routine.
C
C  -- Written on 2-June-1987.
C     Sven Hammarling, Mick Pont and Janet Welding, Nag Central Office.
C
C
C     .. Scalar Arguments ..
      DOUBLE PRECISION EPS,RMIN
      INTEGER BETA,EMIN,T
C     .. Local Scalars ..
      DOUBLE PRECISION A,B,LEPS,LRMIN,ONE,RBASE,SMALL,TWO,ZERO
      INTEGER GNMIN,GPMIN,I,LBETA,LEMIN,LT,NGNMIN,NGPMIN,NOUT
      LOGICAL FIRST,IWARN,LRND
C     .. External Functions ..
      DOUBLE PRECISION STORE
      EXTERNAL STORE
C     .. External Subroutines ..
C      EXTERNAL ENVRON,GETMIN
      EXTERNAL GETMIN
C     .. Intrinsic Functions ..
      INTRINSIC ABS,MAX,MIN
C     .. Save statement ..
      SAVE FIRST,IWARN,LBETA,LEMIN,LEPS,LRMIN,LT
C     .. Data statements ..
      DATA FIRST/.TRUE./,IWARN/.FALSE./
      DATA NOUT/6/
C     .. Executable Statements ..
C
      IF (FIRST) THEN
          FIRST = .FALSE.
          ZERO = 0
          ONE = 1
          TWO = 2
C
C        LBETA, LT, LEPS, LEMIN and LRMIN are the local values of BETA,
C        T, EPS, EMIN and RMIN.
C
C        Throughout this routine we use the function STORE to ensure
C        that relevant values are stored and not held in registers, or
C        are not affected by optimizers.
C
C        ENVRON returns the parameters LBETA and LT. ( LRND is not used
C        here. )
C
c          call envron(lbeta,lt,lrnd)
          lbeta = 2
          lt = 53
          lrnd = .FALSE.
C
C        Start to find EPS.
C
          B = LBETA
          IF (LRND) THEN
              LEPS = (B** (1-LT))/TWO
          ELSE
              LEPS = B** (1-LT)
          END IF
C
C        Computation of EPS complete. Now find EMIN.
C        Let a = + or - 1, and + or - (1 + base**(-3)).
C        Keep dividing a by BETA until (gradual) underflow occurs.
C        This is detected when we cannot recover the previous a.
C
          RBASE = ONE/LBETA
          SMALL = ONE
          DO 20 I = 1,3
              SMALL = STORE(SMALL*RBASE,ZERO)
   20     CONTINUE
          A = STORE(ONE,SMALL)
          CALL GETMIN(NGPMIN,ONE,LBETA)
          CALL GETMIN(NGNMIN,-ONE,LBETA)
          CALL GETMIN(GPMIN,A,LBETA)
          CALL GETMIN(GNMIN,-A,LBETA)
C
          IF (NGPMIN.EQ.NGNMIN .AND. GPMIN.EQ.GNMIN) THEN
              IF (NGPMIN.EQ.GPMIN) THEN
                  LEMIN = NGPMIN
C            ( Non twos-complement machines, no gradual underflow;
C              eg VAX )
              ELSE IF (GPMIN-NGPMIN.EQ.3) THEN
                  LEMIN = NGPMIN - 1 + LT
C            ( Non twos-complement machines, with gradual underflow;
C              eg IEEE standard followers )
              ELSE
                  LEMIN = MIN(NGPMIN,GPMIN)
C            ( A guess; no known machine )
                  IWARN = .TRUE.
              END IF
          ELSE IF (NGPMIN.EQ.GPMIN .AND. NGNMIN.EQ.GNMIN) THEN
              IF (ABS(NGPMIN-NGNMIN).EQ.1) THEN
                  LEMIN = MAX(NGPMIN,NGNMIN)
C            ( Twos-complement machines, no gradual underflow;
C              eg Cyber 205 )
              ELSE
                  LEMIN = MIN(NGPMIN,NGNMIN)
C            ( A guess; no known machine )
                  IWARN = .TRUE.
              END IF
          ELSE IF (ABS(NGPMIN-NGNMIN).EQ.1 .AND. GPMIN.EQ.GNMIN) THEN
              IF (GPMIN-MIN(NGPMIN,NGNMIN).EQ.3) THEN
                  LEMIN = MAX(NGPMIN,NGNMIN) - 1 + LT
C            ( Twos-complement machines with gradual underflow;
C              no known machine )
              ELSE
                  LEMIN = MAX(NGPMIN,NGNMIN)
C            ( A guess; no known machine )
                  IWARN = .TRUE.
              END IF
          ELSE
              LEMIN = MIN(NGPMIN,NGNMIN,GPMIN,GNMIN)
C         ( A guess; no known machine )
              IWARN = .TRUE.
          END IF
C**
C Comment out this IF block if Emin is ok
          IF (IWARN) THEN
              FIRST = .TRUE.
              call rwarn('The value Emin may be incorrect.
     +       If after inspection the value Emin looks acceptable,
     +       please comment out the IF block as marked within 
     +       the code of routine envirn,
     +       otherwise contact NAG Central Office.')
              call intpr('LEMIN=', -1, LEMIN, 1)
C              WRITE (NOUT,FMT=99999) LEMIN
          END IF
C**
C
C        Finally compute RMIN by successive division by BETA.
C        We could compute RMIN as base**( EMIN - 1 ), but some machines
C        underflow during this computation.
C
          LRMIN = 1
          DO 40 I = 1,1 - LEMIN
              LRMIN = LRMIN/LBETA
   40     CONTINUE
      END IF
C
      BETA = LBETA
      T = LT
      EPS = LEPS
      EMIN = LEMIN
      RMIN = LRMIN
      RETURN
C
C
C     End of ENVIRN.
C

      END

      SUBROUTINE D5RPLY(TYPE, QP, K, QK)
C COMPUTES THE NEXT K POLYNOMIALS USING SCALARS
C COMPUTED IN D2RPLY
C
      DOUBLE PRECISION SR, SI, U, V, A, B, C, D, A1, A2, A3, A6, A7
      DOUBLE PRECISION E, F, G, H, SZR, SZI, LZR, LZI, ETA, ARE, MRE
      INTEGER N, NN
      COMMON /P77PLY/ SR, SI, U,
     1 V, A, B, C, D, A1, A2, A3, A6, A7, E, F, G,
     2 H, SZR, SZI, LZR, LZI, ETA, ARE, MRE, N, NN
C
      INTEGER I
      INTEGER TYPE
C
      DOUBLE PRECISION QP(*), K(*), QK(*)
      DOUBLE PRECISION TEMP
      IF (TYPE.EQ.3) GO TO 40
      TEMP = A
      IF (TYPE.EQ.1) TEMP = B
      IF (DABS(A1).GT.DABS(TEMP)*ETA*10.) GO TO 20
C IF A1 IS NEARLY ZERO THEN USE A SPECIAL FORM OF THE
C RECURRENCE
      K(1) = 0.D0
      K(2) = -A7*QP(1)
      DO 10 I=3,N
        K(I) = A3*QK(I-2) - A7*QP(I-1)
   10 CONTINUE
      RETURN
C USE SCALED FORM OF THE RECURRENCE
   20 A7 = A7/A1
      A3 = A3/A1
      K(1) = QP(1)
      K(2) = QP(2) - A7*QP(1)
      DO 30 I=3,N
        K(I) = A3*QK(I-2) - A7*QP(I-1) + QP(I)
   30 CONTINUE
      RETURN
C USE UNSCALED FORM OF THE RECURRENCE IF TYPE IS 3
   40 K(1) = 0.D0
      K(2) = 0.D0
      DO 50 I=3,N
        K(I) = QK(I-2)
   50 CONTINUE
      RETURN
      END

#AIC <- function(object, ...)
#    UseMethod("AIC")
"AIC.car" <-
function (object, ..., verbose = TRUE, k = 2)
{
    phi <- object$phi
    covmat <- object$ecov
    ARP <- length(phi)
    tphi <- chol(solve(covmat)) %*% phi
    cums <- cumsum(tphi^2)
    if (is.null(k) || length(k) != 1L || !is.finite(k) || k < 0) {
        stop("k must be a single nonnegative finite number")
    }
    aic <- -cums + k * seq_len(length(phi))

    res <- data.frame(order = seq_len(length(phi)),
                      t.statistic = round(as.vector(tphi), 2),
                      AIC = round(aic, 2))

    if (isTRUE(verbose)) {
        msg <- c(
            paste0("Call:\n", paste(deparse(object$call), collapse = "\n")),
            "Model selection statistics:",
            paste(capture.output(print(res, row.names = FALSE)), collapse = "\n")
        )
        message(paste(msg, collapse = "\n"))
    }

    res
}

*UPTODATE ORTHESTEXT
      SUBROUTINE ORTHES(NM,N,LOW,IGH,A,ORT)
C
C     .. Scalar Arguments ..
      INTEGER IGH,LOW,N,NM
C     .. Array Arguments ..
      DOUBLE PRECISION A(NM,N),ORT(IGH)
C     .. Local Scalars ..
      DOUBLE PRECISION F,G,H,SCALE
      INTEGER I,II,J,JJ,KP1,LA,M,MP
C     .. Intrinsic Functions ..
      INTRINSIC ABS,SIGN,SQRT
C     .. Executable Statements ..
C
C     THIS SUBROUTINE IS A TRANSLATION OF THE ALGOL PROCEDURE ORTHES,
C     NUM. MATH. 12, 349-368(1968) BY MARTIN AND WILKINSON.
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 339-358(1971).
C
C     GIVEN A REAL GENERAL MATRIX, THIS SUBROUTINE
C     REDUCES A SUBMATRIX SITUATED IN ROWS AND COLUMNS
C     LOW THROUGH IGH TO UPPER HESSENBERG FORM BY
C     ORTHOGONAL SIMILARITY TRANSFORMATIONS.
C
C     ON INPUT-
C
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
C          DIMENSION STATEMENT,
C
C        N IS THE ORDER OF THE MATRIX,
C
C        LOW AND IGH ARE INTEGERS DETERMINED BY THE BALANCING
C          SUBROUTINE  BALANC.  IF  BALANC  HAS NOT BEEN USED,
C          SET LOW=1, IGH=N,
C
C        A CONTAINS THE INPUT MATRIX.
C
C     ON OUTPUT-
C
C        A CONTAINS THE HESSENBERG MATRIX.  INFORMATION ABOUT
C          THE ORTHOGONAL TRANSFORMATIONS USED IN THE REDUCTION
C          IS STORED IN THE REMAINING TRIANGLE UNDER THE
C          HESSENBERG MATRIX,
C
C        ORT CONTAINS FURTHER INFORMATION ABOUT THE TRANSFORMATIONS.
C          ONLY ELEMENTS LOW THROUGH IGH ARE USED.
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO B. S. GARBOW,
C     APPLIED MATHEMATICS DIVISION, ARGONNE NATIONAL LABORATORY
C
C     ------------------------------------------------------------------
C
      LA = IGH - 1
      KP1 = LOW + 1
      IF (LA.LT.KP1) GO TO 200
C
      DO 180 M = KP1,LA
          H = 0.0D0
          ORT(M) = 0.0D0
          SCALE = 0.0D0
C     ********** SCALE COLUMN (ALGOL TOL THEN NOT NEEDED) **********
          DO 20 I = M,IGH
              SCALE = SCALE + ABS(A(I,M-1))
   20     CONTINUE
C
          IF (SCALE.EQ.0.0D0) GO TO 180
          MP = M + IGH
C     ********** FOR I=IGH STEP -1 UNTIL M DO -- **********
          DO 40 II = M,IGH
              I = MP - II
              ORT(I) = A(I,M-1)/SCALE
              H = H + ORT(I)*ORT(I)
   40     CONTINUE
C
          G = -SIGN(SQRT(H),ORT(M))
          H = H - ORT(M)*G
          ORT(M) = ORT(M) - G
C     ********** FORM (I-(U*UT)/H) * A **********
          DO 100 J = M,N
              F = 0.0D0
C     ********** FOR I=IGH STEP -1 UNTIL M DO -- **********
              DO 60 II = M,IGH
                  I = MP - II
                  F = F + ORT(I)*A(I,J)
   60         CONTINUE
C
              F = F/H
C
              DO 80 I = M,IGH
                  A(I,J) = A(I,J) - F*ORT(I)
   80         CONTINUE
C
  100     CONTINUE
C     ********** FORM (I-(U*UT)/H)*A*(I-(U*UT)/H) **********
          DO 160 I = 1,IGH
              F = 0.0D0
C     ********** FOR J=IGH STEP -1 UNTIL M DO -- **********
              DO 120 JJ = M,IGH
                  J = MP - JJ
                  F = F + ORT(J)*A(I,J)
  120         CONTINUE
C
              F = F/H
C
              DO 140 J = M,IGH
                  A(I,J) = A(I,J) - F*ORT(J)
  140         CONTINUE
C
  160     CONTINUE
C
          ORT(M) = SCALE*ORT(M)
          A(M,M-1) = SCALE*G
  180 CONTINUE
C
  200 RETURN
C     ********** LAST CARD OF ORTHES **********
      END

*UPTODATE HQRORTTEXT
      SUBROUTINE HQRORT(NM,N,LOW,IGH,H,WR,WI,Z,IERR)
C
C     .. Scalar Arguments ..
      INTEGER IERR,IGH,LOW,N,NM
C     .. Array Arguments ..
      DOUBLE PRECISION H(NM,N),WI(N),WR(N),Z(NM,N)
C     .. Local Scalars ..
      DOUBLE PRECISION NORM,P,Q,R,S,T,TST1,TST2,W,X,Y,ZZ
      INTEGER EN,ENM2,I,ITN,ITS,J,K,L,LL,M,MM,MP2,NA
      LOGICAL NOTLAS
C     .. Intrinsic Functions ..
      INTRINSIC ABS,MIN,SIGN,SQRT
C     .. Executable Statements ..
C
C     THIS SUBROUTINE IS A TRANSLATION OF THE ALGOL PROCEDURE HQR2,
C     NUM. MATH. 16, 181-204(1970) BY PETERS AND WILKINSON.
C     HANDBOOK FOR AUTO. COMP., VOL.II-LINEAR ALGEBRA, 372-395(1971).
C
C     THIS VERSION INCORPORATES THE SUGGESTED EISPACK MODIFICATION
C     OF HQR2 TO PRODUCE HQRORT.
C
C     THIS SUBROUTINE FINDS THE EIGENVALUES AND THE SCHUR FACTOR Q
C     OF A REAL UPPER HESSENBERG MATRIX BY THE QR METHOD.  THE
C     THE SCHUR FACTOR OF A REAL GENERAL MATRIX CAN ALSO BE FOUND
C     IF  ORTHES  AND  ORTRAN  HAVE
C     BEEN USED TO REDUCE THIS GENERAL MATRIX TO HESSENBERG FORM
C     AND TO ACCUMULATE THE SIMILARITY TRANSFORMATIONS.
C
C     ON INPUT
C
C        NM MUST BE SET TO THE ROW DIMENSION OF TWO-DIMENSIONAL
C          ARRAY PARAMETERS AS DECLARED IN THE CALLING PROGRAM
C          DIMENSION STATEMENT.
C
C        N IS THE ORDER OF THE MATRIX.
C
C        LOW AND IGH ARE INTEGERS DETERMINED BY THE BALANCING
C          SUBROUTINE  BALANC.  IF  BALANC  HAS NOT BEEN USED,
C          SET LOW=1, IGH=N.
C
C        H CONTAINS THE UPPER HESSENBERG MATRIX.
C
C        Z CONTAINS THE TRANSFORMATION MATRIX PRODUCED BY  ELTRAN
C          AFTER THE REDUCTION BY  ELMHES, OR BY  ORTRAN  AFTER THE
C          REDUCTION BY  ORTHES, IF PERFORMED.  IF THE SCHUR FACTOR
C          OF THE HESSENBERG MATRIX IS DESIRED, Z MUST CONTAIN THE
C          IDENTITY MATRIX.
C
C     ON OUTPUT
C
C        H CONTAINS THE QUASI-TRIANGULAR SCHUR FACTOR.
C
C        WR AND WI CONTAIN THE REAL AND IMAGINARY PARTS,
C          RESPECTIVELY, OF THE EIGENVALUES.  THE EIGENVALUES
C          ARE UNORDERED EXCEPT THAT COMPLEX CONJUGATE PAIRS
C          OF VALUES APPEAR CONSECUTIVELY WITH THE EIGENVALUE
C          HAVING THE POSITIVE IMAGINARY PART FIRST.  IF AN
C          ERROR EXIT IS MADE, THE EIGENVALUES SHOULD BE CORRECT
C          FOR INDICES IERR+1,...,N.
C
C        Z CONTAINS THE TRANSFORMED MATRIX Q*Z.
C
C        IERR IS SET TO
C          ZERO       FOR NORMAL RETURN,
C          J          IF THE LIMIT OF 30*N ITERATIONS IS EXHAUSTED
C                     WHILE THE J-TH EIGENVALUE IS BEING SOUGHT.
C
C     CALLS CDIV FOR COMPLEX DIVISION.
C
C     QUESTIONS AND COMMENTS SHOULD BE DIRECTED TO BURTON S. GARBOW,
C     MATHEMATICS AND COMPUTER SCIENCE DIV, ARGONNE NATIONAL LABORATORY
C
C     THIS VERSION DATED AUGUST 1983.
C
C     ------------------------------------------------------------------
C
      IERR = 0
      NORM = 0.0D0
      K = 1
C     .......... STORE ROOTS ISOLATED BY BALANC
C                AND COMPUTE MATRIX NORM ..........
      DO 40 I = 1,N
C
          DO 20 J = K,N
              NORM = NORM + ABS(H(I,J))
   20     CONTINUE
C
          K = I
          IF (I.GE.LOW .AND. I.LE.IGH) GO TO 40
          WR(I) = H(I,I)
          WI(I) = 0.0D0
   40 CONTINUE
C
      EN = IGH
      T = 0.0D0
      ITN = 30*N
C     .......... SEARCH FOR NEXT EIGENVALUES ..........
   60 IF (EN.LT.LOW) GO TO 640
      ITS = 0
      NA = EN - 1
      ENM2 = NA - 1
      L = LOW
C     .......... LOOK FOR SINGLE SMALL SUB-DIAGONAL ELEMENT
C                FOR L=EN STEP -1 UNTIL LOW DO -- ..........
   80 DO 100 LL = LOW,EN
          L = EN + LOW - LL
          IF (L.EQ.LOW) GO TO 120
          S = ABS(H(L-1,L-1)) + ABS(H(L,L))
          IF (S.EQ.0.0D0) S = NORM
          TST1 = S
          TST2 = TST1 + ABS(H(L,L-1))
          IF (TST2.EQ.TST1) GO TO 120
  100 CONTINUE
C     .......... FORM SHIFT ..........
  120 X = H(EN,EN)
      IF (L.EQ.EN) GO TO 480
      Y = H(NA,NA)
      W = H(EN,NA)*H(NA,EN)
      IF (L.EQ.NA) GO TO 5000
      IF (ITN.EQ.0) GO TO 620
      IF (ITS.NE.10 .AND. ITS.NE.20) GO TO 160
C     .......... FORM EXCEPTIONAL SHIFT ..........
      T = T + X
C
      DO 140 I = LOW,EN
          H(I,I) = H(I,I) - X
  140 CONTINUE
C
      S = ABS(H(EN,NA)) + ABS(H(NA,ENM2))
      X = 0.75D0*S
      Y = X
      W = -0.4375D0*S*S
  160 ITS = ITS + 1
      ITN = ITN - 1
C     .......... LOOK FOR TWO CONSECUTIVE SMALL
C                SUB-DIAGONAL ELEMENTS.
C                FOR M=EN-2 STEP -1 UNTIL L DO -- ..........
      DO 180 MM = L,ENM2
          M = ENM2 + L - MM
          ZZ = H(M,M)
          R = X - ZZ
          S = Y - ZZ
          P = (R*S-W)/H(M+1,M) + H(M,M+1)
          Q = H(M+1,M+1) - ZZ - R - S
          R = H(M+2,M+1)
          S = ABS(P) + ABS(Q) + ABS(R)
          P = P/S
          Q = Q/S
          R = R/S
          IF (M.EQ.L) GO TO 200
          TST1 = ABS(P)* (ABS(H(M-1,M-1))+ABS(ZZ)+ABS(H(M+1,M+1)))
          TST2 = TST1 + ABS(H(M,M-1))* (ABS(Q)+ABS(R))
          IF (TST2.EQ.TST1) GO TO 200
  180 CONTINUE
C
  200 MP2 = M + 2
C
      DO 220 I = MP2,EN
          H(I,I-2) = 0.0D0
          IF (I.EQ.MP2) GO TO 220
          H(I,I-3) = 0.0D0
  220 CONTINUE
C     .......... DOUBLE QR STEP INVOLVING ROWS L TO EN AND
C                COLUMNS M TO EN ..........
      DO 460 K = M,NA
          NOTLAS = K .NE. NA
          IF (K.EQ.M) GO TO 240
          P = H(K,K-1)
          Q = H(K+1,K-1)
          R = 0.0D0
          IF (NOTLAS) R = H(K+2,K-1)
          X = ABS(P) + ABS(Q) + ABS(R)
          IF (X.EQ.0.0D0) GO TO 460
          P = P/X
          Q = Q/X
          R = R/X
  240     S = SIGN(SQRT(P*P+Q*Q+R*R),P)
          IF (K.EQ.M) GO TO 260
          H(K,K-1) = -S*X
          GO TO 280
  260     IF (L.NE.M) H(K,K-1) = -H(K,K-1)
  280     P = P + S
          X = P/S
          Y = Q/S
          ZZ = R/S
          Q = Q/P
          R = R/P
          IF (NOTLAS) GO TO 360
C     .......... ROW MODIFICATION ..........
          DO 300 J = K,N
              P = H(K,J) + Q*H(K+1,J)
              H(K,J) = H(K,J) - P*X
              H(K+1,J) = H(K+1,J) - P*Y
  300     CONTINUE
C
          J = MIN(EN,K+3)
C     .......... COLUMN MODIFICATION ..........
          DO 320 I = 1,J
              P = X*H(I,K) + Y*H(I,K+1)
              H(I,K) = H(I,K) - P
              H(I,K+1) = H(I,K+1) - P*Q
  320     CONTINUE
C     .......... ACCUMULATE TRANSFORMATIONS ..........
          DO 340 I = LOW,IGH
              P = X*Z(I,K) + Y*Z(I,K+1)
              Z(I,K) = Z(I,K) - P
              Z(I,K+1) = Z(I,K+1) - P*Q
  340     CONTINUE
          GO TO 440
  360     CONTINUE
C     .......... ROW MODIFICATION ..........
          DO 380 J = K,N
              P = H(K,J) + Q*H(K+1,J) + R*H(K+2,J)
              H(K,J) = H(K,J) - P*X
              H(K+1,J) = H(K+1,J) - P*Y
              H(K+2,J) = H(K+2,J) - P*ZZ
  380     CONTINUE
C
          J = MIN(EN,K+3)
C     .......... COLUMN MODIFICATION ..........
          DO 400 I = 1,J
              P = X*H(I,K) + Y*H(I,K+1) + ZZ*H(I,K+2)
              H(I,K) = H(I,K) - P
              H(I,K+1) = H(I,K+1) - P*Q
              H(I,K+2) = H(I,K+2) - P*R
  400     CONTINUE
C     .......... ACCUMULATE TRANSFORMATIONS ..........
          DO 420 I = LOW,IGH
              P = X*Z(I,K) + Y*Z(I,K+1) + ZZ*Z(I,K+2)
              Z(I,K) = Z(I,K) - P
              Z(I,K+1) = Z(I,K+1) - P*Q
              Z(I,K+2) = Z(I,K+2) - P*R
  420     CONTINUE
  440     CONTINUE
C
  460 CONTINUE
C
      GO TO 80
C     .......... ONE ROOT FOUND ..........
  480 H(EN,EN) = X + T
      WR(EN) = H(EN,EN)
      WI(EN) = 0.0D0
      EN = NA
      GO TO 60
C     .......... TWO ROOTS FOUND ..........
 5000 P = (Y-X)/2.0D0
      Q = P*P + W
      ZZ = SQRT(ABS(Q))
      H(EN,EN) = X + T
      X = H(EN,EN)
      H(NA,NA) = Y + T
      IF (Q.LT.0.0D0) GO TO 580
C     .......... REAL PAIR ..........
      ZZ = P + SIGN(ZZ,P)
      WR(NA) = X + ZZ
      WR(EN) = WR(NA)
      IF (ZZ.NE.0.0D0) WR(EN) = X - W/ZZ
      WI(NA) = 0.0D0
      WI(EN) = 0.0D0
      X = H(EN,NA)
      S = ABS(X) + ABS(ZZ)
      P = X/S
      Q = ZZ/S
      R = SQRT(P*P+Q*Q)
      P = P/R
      Q = Q/R
C     .......... ROW MODIFICATION ..........
      DO 520 J = NA,N
          ZZ = H(NA,J)
          H(NA,J) = Q*ZZ + P*H(EN,J)
          H(EN,J) = Q*H(EN,J) - P*ZZ
  520 CONTINUE
C     .......... COLUMN MODIFICATION ..........
      DO 540 I = 1,EN
          ZZ = H(I,NA)
          H(I,NA) = Q*ZZ + P*H(I,EN)
          H(I,EN) = Q*H(I,EN) - P*ZZ
  540 CONTINUE
C     .......... ACCUMULATE TRANSFORMATIONS ..........
      DO 560 I = LOW,IGH
          ZZ = Z(I,NA)
          Z(I,NA) = Q*ZZ + P*Z(I,EN)
          Z(I,EN) = Q*Z(I,EN) - P*ZZ
  560 CONTINUE
C
      GO TO 600
C     .......... COMPLEX PAIR ..........
  580 WR(NA) = X + P
      WR(EN) = X + P
      WI(NA) = ZZ
      WI(EN) = -ZZ
  600 EN = ENM2
      GO TO 60
  620 IERR = EN
  640 RETURN
      END

      SUBROUTINE DISPLAY
      IMPLICIT NONE
      CHARACTER(LEN=4) ST0
      INTEGER I
c      INCLUDE 'model.txt'
c      INCLUDE 'series.txt'
c      INCLUDE 'conpar.txt'
C*****model.txt
      INTEGER PFI,ARP,VRI,CCV,LYAP,SCC,fct
      DOUBLE PRECISION SCALE,VR,CONST,PHI(20),PRDG
      COMMON/MODEL/SCALE,VR,CONST,PHI,PRDG,PFI,ARP,VRI,CCV,LYAP,SCC,fct
      integer for, fty
      common/model/for, fty
C*****series.txt
      CHARACTER(LEN=40) NAME
      INTEGER LEN
      DOUBLE PRECISION TIM(5000),SER(5000),TDIF(5000)
c      COMMON/SERIES/NAME,LEN,TIM,SER,TDIF
      COMMON/SERIES/TIM,SER,TDIF,NAME,LEN
C*****conpar.txt    
      INTEGER NIT,OPM,RGM,KUP,KSP,KST
      DOUBLE PRECISION REQ,CONCRIT,RPERT,IVLAM,FAC,STLAM,SMLAM,GTLAM
c      COMMON/CONPAR/NIT,OPM,RGM,REQ,CONCRIT,RPERT,IVLAM,FAC,STLAM,
c     * SMLAM,GTLAM,KUP,KSP,KST
      COMMON/CONPAR/REQ,CONCRIT,RPERT,IVLAM,FAC,STLAM,
     * SMLAM,GTLAM,NIT,OPM,RGM,KUP,KSP,KST

      integer ARI, tra
      common/model/ari,tra
      
C      CALL NEWLINE
C      CALL NEWLINE
C  DISPLAY MODEL SETUP
C      PRINT *,'INPUT FROM MODEL FILE IS:'
C      CALL NEWLINE
      IF(PFI.EQ.1)THEN
        ST0='QLFA'
      ELSE IF(PFI.EQ.2)THEN
        ST0='QLFS'
      ELSE IF(PFI.EQ.3)THEN
        ST0='DIRA'
      ELSE IF(PFI.EQ.4)THEN
        ST0='DIRS'
      ELSE IF(PFI.EQ.5)THEN
        ST0='MAPA'
      ELSE
        ST0='MAPS'
      END IF
C      PRINT *,'FORM OF PARAMETERIZATION: ',ST0
C      PRINT *,'SCALE: ',SCALE
C      PRINT *,'ORDER OF AUTOREGRESSION: ',ARP
      IF(ARI.EQ.0)THEN
        ST0='NO  '
      ELSE
        ST0='YES '
      END IF
C      PRINT *,'INITIAL VALUES SUPPLIED? ',ST0

      IF(ARI.EQ.1)THEN
C        PRINT *,'VALUES SUPPLIED ARE:'
        DO 100 I=1,ARP
C        PRINT *,I,PHI(I)
  100   CONTINUE
      ELSE
C        PRINT *,'INITIAL VALUES ARE ZERO'
      END IF
      IF(VRI.EQ.0)THEN
        ST0='NO  '
      ELSE
        ST0='YES '
      END IF
C      PRINT *,'OBSERVATION VARIANCE RATIO INCLUDED? ',ST0
      IF(VRI.EQ.1)THEN
C        PRINT *,'OBSERVATION VARIANCE RATIO: ',VR
      END IF
      IF(CCV.EQ.0)THEN
        ST0='NULL'
      ELSE IF(CCV.EQ.1)THEN
        ST0='MNCT'
      ELSE
        ST0='CTES'
      END IF
C      PRINT *,'METHOD OF MEAN ESTIMATION: ',ST0
      IF(CCV.GT.0)THEN
C        PRINT *,'CONSTANT: ',CONST
      END IF        
      IF(LYAP.EQ.0)THEN
        ST0='NO  '
      ELSE
        ST0='YES '
      END IF
C      PRINT *,'STATIONARY STATE ASSUMED AT START ? ',ST0
      IF(LYAP.EQ.0)THEN
C        PRINT *,'STATE VARIANCE AT START SET TO DIAGONAL ',VR
      END IF
      IF(SCC.EQ.0)THEN
        ST0='NO'
      ELSE
        ST0='YES'
      ENDIF
C      PRINT *,'STATIONARITY CONSTRAINT CHECKING ? ',ST0
C  DISPLAY SERIES SETUP
C      CALL NEWLINE
C      CALL NEWLINE
C      PRINT *,'INPUT FROM SERIES FILE IS:'
C      CALL NEWLINE
C      PRINT *,'SERIES NAME: ',NAME
C      PRINT *,'SERIES LENGTH: ',LEN
C  DISPLAY CONTROL PARAMETER SETUP
C      CALL NEWLINE
C      CALL NEWLINE
C      PRINT *,'INPUT FROM CONTROL PARAMETER FILE IS:'
C      CALL NEWLINE
C      PRINT *,'NUMBER OF ITERATIONS: ',NIT
C      PRINT *,'OPTIMISATION METHOD: ',OPM
C      PRINT *,'RESIDUAL GENERATION METHOD: ',RGM
C      PRINT *,'ROOT EQUALITY CRITERION: ',REQ
C      PRINT *,'SUM OF SQUARES CONVERGENCE CRITERION: ',CONCRIT
C      PRINT *,'PARAMETER PERTURBATION MODIFICATION RATIO: ',RPERT
C      PRINT *,'STEP-SIZE CONSTRAINT PARAMETER INITIAL VALUE: ',IVLAM
C      PRINT *,'CONSTRAINED MODIFICATION FACTOR STEP-SIZE: ',FAC
C      PRINT *,'SMALLEST LAMBDA VALUE: ',STLAM
C      PRINT *,'SMALL LAMBDA VALUE: ',SMLAM
C      PRINT *,'GREATEST LAMBDA VALUE: ',GTLAM
C      IF (KST.EQ.1) PRINT *,'STATE ESTIMATE FILE REQUESTED'
C      IF (KSP.EQ.1) PRINT *,'MODEL SPECTRUM FILE REQUESTED'
C      IF (KUP.EQ.1) PRINT *,'UPDATED MODEL FILE REQUESTED'
      CALL NEWLINE
C      PRINT *,'END OF SET-UP PROCESS'
      RETURN
      END
 

      subroutine setkfilsm(FSER1,FVAR1,SSER1,SVAR1,
c     *ERR1,SRES1)
     *SRES1) 
      IMPLICIT NONE
      INTEGER I

c      INCLUDE 'conpar.txt'
c      INCLUDE 'setcon.txt'
c      INCLUDE 'series.txt'
c      INCLUDE 'veccom.txt'
c      INCLUDE 'redcom.txt'
c      INCLUDE 'seacom.txt'
c      INCLUDE 'repar3.txt'
c      INCLUDE 'kfasav.txt'
C*****conpar.txt    
      INTEGER NIT,OPM,RGM,KUP,KSP,KST
      DOUBLE PRECISION REQ,CONCRIT,RPERT,IVLAM,FAC,STLAM,SMLAM,GTLAM
c      COMMON/CONPAR/NIT,OPM,RGM,REQ,CONCRIT,RPERT,IVLAM,FAC,STLAM,
c     * SMLAM,GTLAM,KUP,KSP,KST
      COMMON/CONPAR/REQ,CONCRIT,RPERT,IVLAM,FAC,STLAM,
     * SMLAM,GTLAM,NIT,OPM,RGM,KUP,KSP,KST

C*****setcon.txt
      LOGICAL CONV,FAIL
      INTEGER NP,ITCT,PPIND
      DOUBLE PRECISION CSO,CSZ,LAM,SSOLD,GMOLD,GMNEW,SIGSQ,OLDB(22)
c      COMMON/SETCON/CONV,FAIL,NP,ITCT,PPIND,CSO,CSZ,LAM,SSOLD,
c     *GMOLD,GMNEW,SIGSQ,OLDB
      COMMON/SETCON/CSO,CSZ,LAM,SSOLD,
     *GMOLD,GMNEW,SIGSQ,OLDB,CONV,FAIL,NP,ITCT,PPIND
C*****series.txt
      CHARACTER(LEN=40) NAME
      INTEGER LEN
      DOUBLE PRECISION TIM(5000),SER(5000),TDIF(5000)
c      COMMON/SERIES/NAME,LEN,TIM,SER,TDIF
      COMMON/SERIES/TIM,SER,TDIF,NAME,LEN

C*****veccom.txt
      DOUBLE PRECISION B(22),NEWB(22),PERB(22),DELB(22),
     * ERR(5000),RES(5000),PRES(22,5000),sres(5000)
      COMMON/VECCOM/B,NEWB,PERB,DELB,ERR,RES,PRES,sres

C*****redcom.txt
      DOUBLE PRECISION GRAD(22),SSP(22,22)
      COMMON/REDCOM/GRAD,SSP
C*****seacom.txt
      LOGICAL SOLV
      DOUBLE PRECISION ESSP(22,22),ECOV(22,22)
c      COMMON/SEACOM/SOLV,ESSP,ECOV
      COMMON/SEACOM/ESSP,ECOV,SOLV

C*****repar3.txt
      DOUBLE PRECISION ALPHA(21),ROOTR(20),ROOTI(20),XVECR(20),
     *XVECI(20)
      COMMON/REPAR3/ALPHA,ROOTR,ROOTI,XVECR,XVECI

C*****kfasav.txt
      DOUBLE PRECISION PRE(5000),PRV(5000),FSER(5000),FVAR(5000)
     1,SSER(5000)
     1,SVAR(5000)
      DOUBLE PRECISION FSES(5000,20),FSCV(5000,20,20),SSES(5000,20)
     1,SSCV(5000,20,20),PSES(5000,20),PSCV(5000,20,20),TRAN(5000,20,20)
c      COMMON/KFASAV/PRE,PRV,FSER,FVAR,SSER,SVAR,FSES,FSCV,SSES,SSCV
c     1,PSES,PSCV,TRAN
      COMMON/KFASAV/FSES,FSCV,SSES,SSCV
     1,PSES,PSCV,TRAN,PRE,PRV,FSER,FVAR,SSER,SVAR


      INTEGER PFI,ARP,VRI,CCV,LYAP,SCC,fct
      DOUBLE PRECISION SCALE,VR,CONST,PHI(20),PRDG
c      COMMON/MODEL/PFI,ARP,VRI,CCV,SCALE,VR,CONST,PHI,LYAP,PRDG,SCC,fct
      COMMON/MODEL/SCALE,VR,CONST,PHI,PRDG,PFI,ARP,VRI,CCV,LYAP,SCC,fct
      integer for, fty
      common/model/for, fty
      integer ARI, tra
      common/model/ari,tra      
      DOUBLE PRECISION FSER1(5000),FVAR1(5000),
     *SSER1(5000),SVAR1(5000)
      DOUBLE PRECISION SRES1(5000)

      DO 100 I=1,LEN
         FSER1(I)=FSER(I)
 100  continue
      DO 101 I=1,LEN
         FVAR1(I)=FVAR(I)
 101  CONTINUE
c         DO 102 I=1,LEN
c            PRE1(I)=PRE(I)
c 102     CONTINUE
c         DO 103 I=1,LEN
c            PRV1(I)=PRV(I)
c 103     CONTINUE
      DO 104 I=1,LEN
         SSER1(I)=SSER(I)
 104  CONTINUE
      DO 105 I=1,LEN
         SVAR1(I)=SVAR(I)
 105  CONTINUE
c         DO 106 I=1,LEN
c            ERR1(I)=ERR(I)
c 106     CONTINUE
      DO 107 I=1,LEN
         SRES1(I)=SRES(I)
 107  CONTINUE
      RETURN
      END

      SUBROUTINE LOOP(ss, bit, ERRNO)
      IMPLICIT NONE
      LOGICAL ADV
      INTEGER I, errno
      DOUBLE PRECISION U,PU,SSNEW,DDUM(1)
c      INCLUDE 'model.txt'
c      INCLUDE 'series.txt'
c      INCLUDE 'conpar.txt'
c      INCLUDE 'setcon.txt'
c      INCLUDE 'veccom.txt'
c      INCLUDE 'repcom.txt'
C*****model.txt
      INTEGER PFI,ARP,VRI,CCV,LYAP,SCC,fct
      DOUBLE PRECISION SCALE,VR,CONST,PHI(20),PRDG
      COMMON/MODEL/SCALE,VR,CONST,PHI,PRDG,PFI,ARP,VRI,CCV,LYAP,SCC,fct
      integer for, fty
      common/model/for, fty
      integer ARI, tra
      common/model/ari,tra

C*****repcom.txt
      LOGICAL PRPI
      INTEGER REQSW,KFSW
      DOUBLE PRECISION VOB
c      COMMON/REPCOM/PRPI,REQSW,VOB,KFSW
      COMMON/REPCOM/VOB,REQSW,KFSW,PRPI 
C*****conpar.txt    
      INTEGER NIT,OPM,RGM,KUP,KSP,KST
      DOUBLE PRECISION REQ,CONCRIT,RPERT,IVLAM,FAC,STLAM,SMLAM,GTLAM
c      COMMON/CONPAR/NIT,OPM,RGM,REQ,CONCRIT,RPERT,IVLAM,FAC,STLAM,
c     * SMLAM,GTLAM,KUP,KSP,KST
      COMMON/CONPAR/REQ,CONCRIT,RPERT,IVLAM,FAC,STLAM,
     * SMLAM,GTLAM,NIT,OPM,RGM,KUP,KSP,KST

C*****setcon.txt
      LOGICAL CONV,FAIL
      INTEGER NP,ITCT,PPIND
      DOUBLE PRECISION CSO,CSZ,LAM,SSOLD,GMOLD,GMNEW,SIGSQ,OLDB(22)
c      COMMON/SETCON/CONV,FAIL,NP,ITCT,PPIND,CSO,CSZ,LAM,SSOLD,
c     *GMOLD,GMNEW,SIGSQ,OLDB
      COMMON/SETCON/CSO,CSZ,LAM,SSOLD,
     *GMOLD,GMNEW,SIGSQ,OLDB,CONV,FAIL,NP,ITCT,PPIND
C*****veccom.txt
      DOUBLE PRECISION B(22),NEWB(22),PERB(22),DELB(22),
     * ERR(5000),RES(5000),PRES(22,5000),sres(5000)
      COMMON/VECCOM/B,NEWB,PERB,DELB,ERR,RES,PRES,sres
C*****series.txt
      CHARACTER(LEN=40) NAME
      INTEGER LEN
      DOUBLE PRECISION TIM(5000),SER(5000),TDIF(5000)
c      COMMON/SERIES/NAME,LEN,TIM,SER,TDIF
      COMMON/SERIES/TIM,SER,TDIF,NAME,LEN
C sum of squares and parameters in each iteration      
      DOUBLE PRECISION ss(NIT+1), BIT(NIT+1,22)
      DOUBLE PRECISION WK(20),VT(5000),BI(2,20,20),R(2,20,20),
     *RI(2,20,20)
      INTEGER ERRNO1
      COMMON/RESGN2/WK,VT,BI,R,RI,ERRNO1

C  FOR USE IN PERTURBATION CALCULATIONS
      ERRNO=0
      IF(PFI.LT.3)THEN
        PU=CSO
      ELSE
        PU=RPERT*1.0D-3
      END IF
C  START OF MAIN LOOP
      DO 100 ITCT=1,NIT
C  COPY OLDB TO B
      DO 200 I=1,NP
      B(I)=OLDB(I)
  200 CONTINUE
      IF(ITCT.EQ.1)THEN
        CALL REPAR
        IF(PRPI)THEN
C          PRINT *,'ITERATION 0: ROOT WITH POSITIVE REAL PART'
           call rexit('ITERATION 0: ROOT WITH POSITIVE REAL PART')
C          STOP
        ERRNO=1
        RETURN
        END IF
        CALL RESGEN
        SSOLD=CSZ
        DO 201 I=1,LEN
        SSOLD=SSOLD+RES(I)*RES(I)
  201   CONTINUE
        GMOLD=GMNEW
        ss(1)=SSOLD
        IF(tra.EQ.1)THEN
           call intpr('ITERATION 0:', 12, 1, 0)
           DDUM(1)=LAM
           call dblepr('LAMBDA = ', 9, DDUM, 1)
           DDUM(1)=SSOLD
           call dblepr('   INITIAL SUM OF SQUARES = ', 28, DDUM, 1)
           call dblepr('   INITIAL PARAMETER VALUES',  27, B(1), 1)
C        PRINT *,'ITERATION 0:'
C        PRINT *,'   LAMBDA = ',LAM
C        PRINT *,'   INITIAL SUM OF SQUARES = ',SSOLD
C        PRINT *,'   INITIAL PARAMETER VALUES'
c           call dblepr('  ',  2, B, ARP)
C        DO 202 I=1,ARP
C           call dblepr('  ',  2, B(I), 1)
C        PRINT *,'   ',I,'  ',B(I)
C  202   CONTINUE
        IF(VRI.EQ.1)THEN
          call dblepr('   INITIAL OBSERVATION VARIANCE RATIO = ', 40, 
     * B(ARP+1), 1)
C          PRINT *,'   INITIAL OBSERVATION VARIANCE RATIO = ',B(ARP+1)
C        END IF
        END IF
        ENDIF
        IF(CCV.EQ.2)THEN
          IF(VRI.EQ.1)THEN
            U=B(ARP+2)
          ELSE
            U=B(ARP+1)
          ENDIF
          IF(tra.EQ.1)THEN
           DDUM(1)=U
           call dblepr('   INITIAL VALUE OF CONSTANT TERM = ', 36, 
     *       DDUM, 1)
C           PRINT *,'   INITIAL VALUE OF CONSTANT TERM = ',U
          ENDIF
        ENDIF
      END IF
C  CALCULATE PERTURBATION VECTOR
      DO 300 I=1,NP
      PERB(I)=PU
  300 CONTINUE
      IF(PFI.EQ.3.OR.PFI.EQ.4)THEN
        DO 301 I=1,ARP
        PERB(I)=PERB(I)*(SCALE+DABS(B(I)))
  301   CONTINUE
      END IF
      DO 303 PPIND=1,NP
      U=PERB(PPIND)
      B(PPIND)=B(PPIND)+U
      CALL REPAR
      IF(PRPI)THEN
         call rexit('ROOT WITH POSITIVE REAL PART')
C        PRINT *,'ITERATION ',ITCT,': ROOT WITH POSITIVE REAL PART'
        ERRNO=1
        RETURN
C        STOP
      END IF
      CALL RESGEN
      ERRNO = ERRNO1
      DO 302 I=1,LEN
      PRES(PPIND,I)=(PRES(PPIND,I)-RES(I))/U
  302 CONTINUE
      B(PPIND)=OLDB(PPIND)
  303 CONTINUE
      PPIND=0
      CALL REJUCE
  304 CALL SEARCH
      DO 305 I=1,NP
      B(I)=NEWB(I)
  305 CONTINUE
      CALL REPAR
      IF(PRPI)THEN
        ADV=.FALSE.
      ELSE
        CALL RESGEN
        SSNEW=CSZ
        DO 306 I=1,LEN
        SSNEW=SSNEW+RES(I)*RES(I)
  306   CONTINUE
        ADV=SSNEW.LT.SSOLD
      END IF
      IF(ADV)THEN
        LAM=LAM/FAC
        IF(LAM.LT.STLAM)LAM=CSZ
        CONV=LAM.LT.SMLAM.AND.SSNEW.GT.(SSOLD*(CSO-CONCRIT))
        SSOLD=SSNEW
        GMOLD=GMNEW
        DO 307 I=1,NP
        OLDB(I)=NEWB(I)
  307   CONTINUE
        IF(CONV)THEN
          RETURN
        ELSE
          IF(tra.EQ.1)THEN
            call intpr('ITERATION :', 11, ITCT, 1)
            DDUM(1)=LAM
            call dblepr('   LAMBDA: ',11, DDUM, 1)
           DDUM(1)=SSOLD
           call dblepr('   SUM OF SQUARES = ', 20, DDUM, 1)
           call dblepr('   PARAMETER VALUES',  19, OLDB(1), 1)
C          PRINT *,'ITERATION :',ITCT
C          PRINT *,'   LAMBDA: ',LAM
C          PRINT *,'   SUM OF SQUARES: ',SSOLD
C          PRINT *,'   PARAMETER VALUES:'
C           call dblepr('  ',  2, OLDB, NP)
C          DO 308 I=1,NP
C           call intpr('I,', -1, I, 1)
C           call dblepr('  ',  2, OLDB(I), 1)
C          PRINT *,'  ',I,'  ',OLDB(I)
C          PRINT *,'  ',I,'  ',OLDB(I)
C  308     CONTINUE
          END IF
      ss(ITCT+1)=SSOLD
      DO 309 I=1,NP
      BIT(ITCT+1, I)=OLDB(I)
  309 CONTINUE    
          GOTO 100
        END IF
      ELSE
        LAM=LAM*FAC
        FAIL=LAM.GT.GTLAM
        IF(FAIL)THEN
          RETURN
        ELSE
          IF(LAM.LT.IVLAM)LAM=IVLAM
          GOTO 304
        END IF
      END IF
  100 CONTINUE
      RETURN
      END

      SUBROUTINE CINVERT
      IMPLICIT NONE
      INTEGER I,J
c      INCLUDE 'model.txt'
c      INCLUDE 'setcon.txt'
c      INCLUDE 'resgn2.txt'
C*****model.txt
      INTEGER PFI,ARP,VRI,CCV,LYAP,SCC,fct
      DOUBLE PRECISION SCALE,VR,CONST,PHI(20),PRDG
      COMMON/MODEL/SCALE,VR,CONST,PHI,PRDG,PFI,ARP,VRI,CCV,LYAP,SCC,fct
      integer for, fty
      common/model/for, fty
      integer ARI, tra
      common/model/ari,tra

C*****setcon.txt
      LOGICAL CONV,FAIL
      INTEGER NP,ITCT,PPIND
      DOUBLE PRECISION CSO,CSZ,LAM,SSOLD,GMOLD,GMNEW,SIGSQ,OLDB(22)
c      COMMON/SETCON/CONV,FAIL,NP,ITCT,PPIND,CSO,CSZ,LAM,SSOLD,
c     *GMOLD,GMNEW,SIGSQ,OLDB
      COMMON/SETCON/CSO,CSZ,LAM,SSOLD,
     *GMOLD,GMNEW,SIGSQ,OLDB,CONV,FAIL,NP,ITCT,PPIND
C*****resgn2.txt
      DOUBLE PRECISION WK(20),VT(5000),BI(2,20,20),R(2,20,20),
     *RI(2,20,20)
c       DOUBLE PRECISION WK(20),VT(5000),BI(2,20,20),R(20,20),RI(2,20,20)
      INTEGER ERRNO1
      COMMON/RESGN2/WK,VT,BI,R,RI,ERRNO1


c     added ZW
      integer IPIVOT(ARP)
      DO J=1,ARP
      DO I=1,ARP
      IF(I.EQ.J)THEN
        BI(1,I,J)=CSO
      ELSE
        BI(1,I,J)=CSZ
      END IF
      BI(2,I,J)=CSZ
      END DO
      END DO
      I=1
      J=20
c      CALL F04ADF(R,J,BI,J,ARP,ARP,RI,J,WK,I) changed ZW
      call zgesv(ARP,ARP,R,J,IPIVOT,BI,J,I)
      ERRNO1 = 0
      IF(I.NE.0)THEN
C      IF(I.NE.0)THEN
c        PRINT *,'PROGRAM FAILS IN F04ADF WITH ERROR: ',I
C         PRINT *,'PROGRAM FAILS IN ZGESV WITH ERROR: ',I
        call rexit('PROGRAM FAILS IN ZGESV')
C         STOP
      ERRNO1 = 2
      RETURN
      END IF
      RETURN
      END

      SUBROUTINE D2RPLY(TYPE, K, QK)
C THIS ROUTINE CALCULATES SCALAR QUANTITIES USED TO
C COMPUTE THE NEXT K POLYNOMIAL AND NEW ESTIMATES OF
C THE QUADRATIC COEFFICIENTS.
C TYPE - INTEGER VARIABLE SET HERE INDICATING HOW THE
C CALCULATIONS ARE NORMALIZED TO AVOID OVERFLOW
C
      COMMON /P77PLY/ SR, SI, U,
     1 V, A, B, C, D, A1, A2, A3, A6, A7, E, F, G,
     2 H, SZR, SZI, LZR, LZI, ETA, ARE, MRE, N, NN
C
      INTEGER N, NN
      INTEGER TYPE
      DOUBLE PRECISION ETA, ARE, MRE
C
      DOUBLE PRECISION K(*), QK(*), SR, SI, U, V, A, B, C, D,
     1 A1, A2, A3, A6, A7, E, F, G, H, SZR, SZI,
     2 LZR, LZI
C SYNTHETIC DIVISION OF K BY THE QUADRATIC 1,U,V
      CALL D8RPLY(N, U, V, K, QK, C, D)
      IF (DABS(C).GT.DABS(K(N))*100.*ETA) GO TO 10
      IF (DABS(D).GT.DABS(K(N-1))*100.*ETA) GO TO 10
      TYPE = 3
C TYPE=3 INDICATES THE QUADRATIC IS ALMOST A FACTOR
C OF K
      RETURN
   10 IF (DABS(D).LT.DABS(C)) GO TO 20
      TYPE = 2
C TYPE=2 INDICATES THAT ALL FORMULAS ARE DIVIDED BY D
      E = A/D
      F = C/D
      G = U*B
      H = V*B
      A3 = (A+G)*E + H*(B/D)
      A1 = B*F - A
      A7 = (F+U)*A + H
      RETURN
   20 TYPE = 1
C TYPE=1 INDICATES THAT ALL FORMULAS ARE DIVIDED BY C
      E = A/C
      F = D/C
      G = U*E
      H = V*B
      A3 = A*E + (H/C+G)*B
      A1 = B - A*(D/C)
      A7 = A + G*D + H*F
      RETURN
      END

*UPTODATE SAXPYTEXT
      SUBROUTINE SAXPY(N,SA,SX,INCX,SY,INCY)
C
C     CONSTANT TIMES A VECTOR PLUS A VECTOR.
C     USES UNROLLED LOOP FOR INCREMENTS EQUAL TO ONE.
C     JACK DONGARRA, LINPACK, 3/11/78.
C
C     .. Scalar Arguments ..
      DOUBLE PRECISION SA
      INTEGER INCX,INCY,N
C     .. Array Arguments ..
      DOUBLE PRECISION SX(*),SY(*)
C     .. Local Scalars ..
      INTEGER I,IX,IY,M,MP1
C     .. Intrinsic Functions ..
      INTRINSIC MOD
C     .. Executable Statements ..
C
      IF (N.LE.0) RETURN
      IF (SA.EQ.0.0D0) RETURN
      IF (INCX.EQ.1 .AND. INCY.EQ.1) GO TO 40
C
C        CODE FOR UNEQUAL INCREMENTS OR EQUAL INCREMENTS
C          NOT EQUAL TO 1
C
      IX = 1
      IY = 1
      IF (INCX.LT.0) IX = (-N+1)*INCX + 1
      IF (INCY.LT.0) IY = (-N+1)*INCY + 1
      DO 20 I = 1,N
          SY(IY) = SY(IY) + SA*SX(IX)
          IX = IX + INCX
          IY = IY + INCY
   20 CONTINUE
      RETURN
C
C        CODE FOR BOTH INCREMENTS EQUAL TO 1
C
C
C        CLEAN-UP LOOP
C
   40 M = MOD(N,4)
      IF (M.EQ.0) GO TO 80
      DO 60 I = 1,M
          SY(I) = SY(I) + SA*SX(I)
   60 CONTINUE
      IF (N.LT.4) RETURN
   80 MP1 = M + 1
      DO 100 I = MP1,N,4
          SY(I) = SY(I) + SA*SX(I)
          SY(I+1) = SY(I+1) + SA*SX(I+1)
          SY(I+2) = SY(I+2) + SA*SX(I+2)
          SY(I+3) = SY(I+3) + SA*SX(I+3)
  100 CONTINUE
      RETURN
      END

      SUBROUTINE REPAR
      IMPLICIT NONE
      INTEGER I,J,K,K1,K2,M
      DOUBLE PRECISION REV,S,T,U,V,W,X,Y,Z
c      INCLUDE 'model.txt'
c      INCLUDE 'conpar.txt'
c      INCLUDE 'setcon.txt'
c      INCLUDE 'veccom.txt'
c      INCLUDE 'repcom.txt'
c      INCLUDE 'repar2.txt'
c      INCLUDE 'repar3.txt'
C*****model.txt
      INTEGER PFI,ARP,VRI,CCV,LYAP,SCC,fct
      DOUBLE PRECISION SCALE,VR,CONST,PHI(20),PRDG
      COMMON/MODEL/SCALE,VR,CONST,PHI,PRDG,PFI,ARP,VRI,CCV,LYAP,SCC,fct
      integer for, fty
      common/model/for, fty
      integer ARI, tra
      common/model/ari,tra

C*****conpar.txt    
      INTEGER NIT,OPM,RGM,KUP,KSP,KST
      DOUBLE PRECISION REQ,CONCRIT,RPERT,IVLAM,FAC,STLAM,SMLAM,GTLAM
c      COMMON/CONPAR/NIT,OPM,RGM,REQ,CONCRIT,RPERT,IVLAM,FAC,STLAM,
c     * SMLAM,GTLAM,KUP,KSP,KST
      COMMON/CONPAR/REQ,CONCRIT,RPERT,IVLAM,FAC,STLAM,
     * SMLAM,GTLAM,NIT,OPM,RGM,KUP,KSP,KST
C*****setcon.txt
      LOGICAL CONV,FAIL
      INTEGER NP,ITCT,PPIND
      DOUBLE PRECISION CSO,CSZ,LAM,SSOLD,GMOLD,GMNEW,SIGSQ,OLDB(22)
c      COMMON/SETCON/CONV,FAIL,NP,ITCT,PPIND,CSO,CSZ,LAM,SSOLD,
c     *GMOLD,GMNEW,SIGSQ,OLDB
      COMMON/SETCON/CSO,CSZ,LAM,SSOLD,
     *GMOLD,GMNEW,SIGSQ,OLDB,CONV,FAIL,NP,ITCT,PPIND

C*****veccom.txt
      DOUBLE PRECISION B(22),NEWB(22),PERB(22),DELB(22),
     * ERR(5000),RES(5000),PRES(22,5000),sres(5000)
      COMMON/VECCOM/B,NEWB,PERB,DELB,ERR,RES,PRES,sres
C*****repcom.txt
      LOGICAL PRPI
      INTEGER REQSW,KFSW
      DOUBLE PRECISION VOB
c      COMMON/REPCOM/PRPI,REQSW,VOB,KFSW
      COMMON/REPCOM/VOB,REQSW,KFSW,PRPI 
C*****repar2.txt
      DOUBLE PRECISION BCT(21),CHT(21,21)
      COMMON/REPAR2/BCT,CHT
C*****repar3.txt
      DOUBLE PRECISION ALPHA(21),ROOTR(20),ROOTI(20),XVECR(20),
     *XVECI(20)
      COMMON/REPAR3/ALPHA,ROOTR,ROOTI,XVECR,XVECI


      M=ARP+1
      IF(PFI.EQ.1)THEN
C        PRINT *,'NOT YET IMPLEMENTED'
      ELSE IF(PFI.EQ.2)THEN
C        PRINT *,'NOT YET IMPLEMENTED'
      ELSE IF(PFI.GE.3)THEN
        CALL ROOTS
        J=ARP+1
        VOB=CSZ
        IF(VRI.EQ.1)VOB=B(J)*B(J)
        IF(CCV.EQ.2)THEN
          IF(VRI.EQ.0)THEN
            CONST=B(J)
          ELSE
            CONST=B(J+1)
          END IF
        END IF
        IF(PFI.GE.5)THEN
          DO 5000 K=1,ARP
          U=ROOTR(K)
          Z=ROOTI(K)
          W=CSO-U
          X=-Z
          Y=CSO+U
          CALL DIVC(W,X,Y,Z,U,V)
          CALL MULR(U,V,-SCALE,ROOTR(K),ROOTI(K))
 5000     CONTINUE
        END IF
        PRPI=.FALSE.
        IF (SCC.EQ.1) THEN
          DO 310 K=1,ARP
          IF(ROOTR(K).GE.CSZ)THEN
            PRPI=.TRUE.
            RETURN
          END IF
  310     CONTINUE
        ENDIF
        IF(KFSW.EQ.0) THEN
          REV=CSO
          IF(ARP.GT.1)THEN
            DO I=2,ARP
              K=I-1
              S=ROOTR(I)
              T=ROOTI(I)
              DO J=1,K
                Y=ROOTR(J)
                Z=ROOTI(J)
                CALL SUBC(S,T,Y,Z,U,V)
                CALL ADDC(S,T,Y,Z,W,X)
                U=(U*U+V*V)/(W*W+X*X)
                IF(REV.GT.U)REV=U
              END DO
            END DO
          END IF
          REV=REV*1.0D2
          REV=REV/(CSO+REV)
          IF(REV.GT.REQ)THEN
            REQSW=0
          ELSE
            REQSW=1
          ENDIF
        ELSE
          REQSW=1
        ENDIF
        IF(REQSW.EQ.0)THEN
          K1=ARP-1
          DO 321 K=1,ARP
          U=CSO
          V=CSZ
          IF((PFI.EQ.4.OR.PFI.EQ.6).AND.ARP.GT.1)THEN
            S=CSO+ROOTR(K)/SCALE
            T=ROOTI(K)/SCALE
            Y=CSO
            Z=CSZ
            DO 320 K2=1,K1
            CALL MULC(S,T,Y,Z,W,X)
            Y=W
            Z=X
  320       CONTINUE
            U=W
            V=X
          END IF
          XVECR(K)=U
          XVECI(K)=V
  321     CONTINUE
        ELSE
          IF(PFI.GE.3)THEN
            IF(PFI.EQ.3.OR.PFI.EQ.5)THEN
              DO 330 K=1,ARP
              XVECR(K)=CSZ
  330         CONTINUE
              XVECR(1)=CSO
            ELSE
              DO 331 K=1,ARP
              XVECR(K)=BCT(K)
  331         CONTINUE
            END IF
            IF(PFI.GE.5)THEN
              DO 334 K=1,M
              S=CSZ
              U=CSO 
              DO 333 J=1,M
              S=S+CHT(K,J)*U
              U=B(J)
  333         CONTINUE
              IF(K.EQ.1)THEN
                V=S
              ELSE 
                ALPHA(K-1)=S/V
              END IF
  334         CONTINUE
            END IF
          END IF            
        END IF
      END IF
      RETURN
      END

*UPTODATE LYBSCTEXT
      SUBROUTINE LYBSC(N,A,NA,C,NC,X,NX,U,WRK,WIK,MODE,IERR)
      IMPLICIT NONE
C
C        MARK 2.2 RELEASE.  NAG COPYRIGHT 1987.
C
C PURPOSE
C
C        Solves for X the continuous-time Lyapunov equation
C
C               TRANS(A)*X + X*A = C
C
C        where A is an n by n matrix and C is an n by n symmetric
C        matrix.
C
C METHOD
C
C        This routine is a modification of the subroutine  ATXXAC,
C        written and discussed by  R.H.Bartels & G.W.Stewart
C        (see reference)
C
C REFERENCE
C
C         BARTELS, R.H. & STEWART, G.W
C            "Solution of the matrix equation  A'X + XB = C  ",
C            Commun. A.C.M., Vol 15, 1972, pp. 820-826 .
C
C         HAMMARLING, S.J.
C         "Numerical solution of the stable, non-negative definite
C          Lyapunov equation."
C          IMA Journal of Numerical Analysis, Vol. 2, No. 3, July 1982
C          pp 303-325
C
C AUXILIARY ROUTINES
C
C       ORTHES,ORTRAN,HQRORT (EISPACK)
C       SGEFA,SGESL          (LINPACK)
C       LYCSR,SDATA          (SLICE)
C
C ARGUMENTS IN
C
C
C       N        INTEGER
C                -The order of matrix A.
C                 N .GE. 1
C
C       A        REAL(NA,N)
C                -The leading N by N part of this array must contain
C                 the matrix A of the equation.
C                 On exit the N by N part of this array is overwritten
C                 but see COMMENTS below.
C
C       NA       INTEGER
C                -The first dimension of arrays A and U as declared
C                 in the calling program.
C                 NA .GE. N
C
C       C        REAL(NC,N)
C                -The leading N by N part of this array must contain
C                 the matrix C of the equation.
C
C       NC       INTEGER
C                -The first dimension of array C as declared in the
C                 calling program
C                 NC .GE. N
C
C       NX       INTEGER
C                -The first dimension of array X as declared in the
C                 calling program
C                 NX .GE. N
C
C       MODE     INTEGER
C                -Indicates the form of matrix A on input.
C                 MODE = 0  if  A  has not already been reduced to
C                                upper triangular form
C                 MODE = 1  if  A  has been reduced to triangular form
C                                (e.g) by a previous call to LYBSC
C
C ARGUMENTS OUT
C
C       A        REAL(NA,N)
C                -The leading N by N part of this array contains the
C                 upper triangular form of A.
C
C       X        REAL(NX,N)
C                -The leading N by N part of this array contains the
C                 solution matrix X.
C
C       U        REAL(NA,N)
C                -The leading N by N part of this array contains the
C                 orthogonal matrix which was used to reduce A to upper
C                 triangular form.
C
C       IERR     INTEGER
C                -Error indicator
C
C                IERR = 0     Successful return
C
C                IERR = 1     A  has a degenerate pair of eigenvalues
C
C                IERR = 2     A  cannot be reduced to triangular form
C
C                IERR = 3     Either N .LT. 1  or N .GT. NA  or
C                                    N .GT. NC or N .GT. NX or
C                                    0 .GT. MODE .GT. 1
C
C WORKING SPACE
C
C        WRK     REAL(N)
C
C        WIK     REAL(N)
C
C ORIGINATOR
C
C                Control Systems Research Group, Dept EECS, Kingston
C                Polytechnic, Penrhyn Rd.,Kingston-upon-Thames, England.
C
C VERSION        LYBSC 2.2 ( 20TH JANUARY 1987 )
C
C LIBRARY INDEX
C                SLICE 2.2 ( 20TH JANUARY 1987 )
C                Section 10. Synthesis of Controllers for
C                                        State Space Models.
C                Subsection 10.3. Lyapunov Equation Solvers.
C
C COMMENTS   The input coefficient matrix  A is overwritten by the
C            triangularized form. If required the matrix A can be
C            reformed from the matrix product UAU' (where U and A
C            are the matrices output in arrays U and A respectively)
C
C            If there is a shortage of storage space, the matrices
C            C  and  X  can share the same locations, but this will
C            of course, result in the loss of the matrix  C.
C
C******************************************************************
C
C     .. Scalar Arguments ..
      INTEGER IERR,MODE,N,NA,NC,NX
C     .. Array Arguments ..
      DOUBLE PRECISION A(NA,N),C(NC,N),U(NA,N),WIK(N),WRK(N),X(NX,N)
C*PT*WARNING* Already double-precision
C     .. Local Scalars ..
      DOUBLE PRECISION DPREC
      INTEGER I,IGH,J,K,LOW
C     .. External Subroutines ..
      EXTERNAL HQRORT,LYCSR,ORTHES,ORTRAN
C     .. Executable Statements ..
C
      IERR = 0
      IF (N.LT.1) IERR = 3
      IF (0.GT.MODE .OR. MODE.GT.1) IERR = 3
      IF (NA.LT.N .OR. NC.LT.N .OR. NX.LT.N) IERR = 3
      IF (IERR.EQ.3) RETURN
      IF (MODE.EQ.1) GO TO 40
      LOW = 1
      IGH = N
      CALL ORTHES(NA,N,LOW,IGH,A,WRK)
      CALL ORTRAN(NA,N,LOW,IGH,A,WRK,U)
      CALL HQRORT(NA,N,LOW,IGH,A,WRK,WIK,U,IERR)
C
C       SET THE ELEMENTS ON THE FIRST SUB-DIAGONAL OF THE BLOCK
C       UPPER TRIANGULAR MATRIX TO ZERO
C
      K = 1
   20 IF (K.GE.N) GO TO 40
      IF (WIK(K).EQ.0.0D0) THEN
          A(K+1,K) = 0.0D0
          K = K + 1
      ELSE
          K = K + 2
          IF (K.LE.N) A(K,K-1) = 0.D0
      END IF
      GO TO 20
   40 CONTINUE
      IF (IERR.NE.0) GO TO 520
C
      DO I = 1,N
          DO J = 1,N
              X(I,J) = C(I,J)
          END DO
          X(I,I) = X(I,I)*0.5D0
      END DO
C
      DO I = 1,N
          DO J = 1,N
C*PT*WARNING* Constant already double-precision
              DPREC = 0.0D0
              DO K = I,N
                  DPREC = DPREC + X(I,K)*U(K,J)
              END DO
              WRK(J) = DPREC
          END DO
          DO J = 1,N
              X(I,J) = WRK(J)
          END DO
      END DO
      DO J = 1,N
          DO I = 1,N
C*PT*WARNING* Constant already double-precision
              DPREC = 0.0D0
              DO K = 1,N
                  DPREC = DPREC + U(K,I)*X(K,J)
              END DO
              WRK(I) = DPREC
          END DO
          DO I = 1,N
              X(I,J) = WRK(I)
          END DO
      END DO
C
      DO I = 1,N
          DO J = I,N
              X(I,J) = X(I,J) + X(J,I)
              X(J,I) = X(I,J)
          END DO
      END DO
C
      CALL LYCSR(N,A,NA,X,NX,IERR)
      IF (IERR.NE.0) RETURN
C
      DO 300 I = 1,N
          X(I,I) = X(I,I)*0.5D0
  300 CONTINUE
C
      DO 380 I = 1,N
          DO 340 J = 1,N
C*PT*WARNING* Constant already double-precision
              DPREC = 0.0D0
              DO 320 K = I,N
                  DPREC = DPREC + X(I,K)*U(J,K)
  320         CONTINUE
              WRK(J) = DPREC
  340     CONTINUE
          DO 360 J = 1,N
              X(I,J) = WRK(J)
  360     CONTINUE
  380 CONTINUE
C
      DO 460 J = 1,N
          DO 420 I = 1,N
C*PT*WARNING* Constant already double-precision
              DPREC = 0.0D0
              DO 400 K = 1,N
                  DPREC = DPREC + U(I,K)*X(K,J)
  400         CONTINUE
              WRK(I) = DPREC
  420     CONTINUE
          DO 440 I = 1,N
              X(I,J) = WRK(I)
  440     CONTINUE
  460 CONTINUE
C
      DO 5000 I = 1,N
          DO 480 J = I,N
              X(I,J) = X(I,J) + X(J,I)
              X(J,I) = X(I,J)
  480     CONTINUE
 5000 CONTINUE
C
      GO TO 540
  520 IERR = 2
  540 RETURN
      END

      SUBROUTINE SIMI
      IMPLICIT NONE
      INTEGER I,I1,J,K,L
      DOUBLE PRECISION V,W,Y,Z
c      INCLUDE 'setcon.txt'
c      INCLUDE 'veccom.txt'
c      INCLUDE 'seacom.txt'
C*****setcon.txt
      LOGICAL CONV,FAIL
      INTEGER NP,ITCT,PPIND
      DOUBLE PRECISION CSO,CSZ,LAM,SSOLD,GMOLD,GMNEW,SIGSQ,OLDB(22)
c      COMMON/SETCON/CONV,FAIL,NP,ITCT,PPIND,CSO,CSZ,LAM,SSOLD,
c     *GMOLD,GMNEW,SIGSQ,OLDB
      COMMON/SETCON/CSO,CSZ,LAM,SSOLD,
     *GMOLD,GMNEW,SIGSQ,OLDB,CONV,FAIL,NP,ITCT,PPIND
C*****veccom.txt
      DOUBLE PRECISION B(22),NEWB(22),PERB(22),DELB(22),
     * ERR(5000),RES(5000),PRES(22,5000),sres(5000)
      COMMON/VECCOM/B,NEWB,PERB,DELB,ERR,RES,PRES,sres
C*****seacom.txt
      LOGICAL SOLV
      DOUBLE PRECISION ESSP(22,22),ECOV(22,22)
c      COMMON/SEACOM/SOLV,ESSP,ECOV
      COMMON/SEACOM/ESSP,ECOV,SOLV


      SOLV=.TRUE.
      DO L=1,NP
      K=NP+1-L
      IF(ESSP(K,K).LT.1.0D-9)THEN
        SOLV=.FALSE.
        RETURN
      END IF
      IF(K.GT.1)THEN
        I1=K-1
        Y=ESSP(K,K)
        DO I=1,I1
        Z=ESSP(I,K)
        DO J=1,I1
        ESSP(I,J)=ESSP(I,J)-Z*ESSP(K,J)/Y
        END DO
        DELB(I)=DELB(I)-Z*DELB(K)/Y
        END DO
      END IF
      END DO
      DO K=1,NP
      Y=ESSP(K,K)
      V=1.0D0
      IF(K.GT.1)THEN
        I1=K-1
        DO I=1,I1
        Z=0.0D0
        DO J=1,I1
        Z=Z-ESSP(J,I)*ESSP(K,J)
        END DO
        W=ESSP(K,I)
        Z=Z/Y
        V=V-W*Z
        ESSP(I,K)=Z
        DELB(K)=DELB(K)-W*DELB(I)
        END DO
        I1=K-1
        DO I=1,I1
        ESSP(K,I)=ESSP(I,K)
        END DO
      END IF
      ESSP(K,K)=V/Y
      DELB(K)=DELB(K)/Y
      END DO
      RETURN
      END

c     SUBROUTINE RESG1
      SUBROUTINE RESG1pre1
      IMPLICIT NONE
      INTEGER E,I,J,K,L,M,T
      DOUBLE PRECISION DEL
      DOUBLE PRECISION U,PRED,PEV
c      double precision forser
c      INCLUDE 'model.txt'
c      INCLUDE 'series.txt'
c      INCLUDE 'setcon.txt'
c      INCLUDE 'veccom.txt'
c      INCLUDE 'repcom.txt'
c      INCLUDE 'repar3.txt'
c      INCLUDE 'resgn2.txt'
c      INCLUDE 'resgd3.txt'
c      INCLUDE 'kfasav.txt'
C*****model.txt
      INTEGER PFI,ARP,VRI,CCV,LYAP,SCC,fct
      DOUBLE PRECISION SCALE,VR,CONST,PHI(20),PRDG
      COMMON/MODEL/SCALE,VR,CONST,PHI,PRDG,PFI,ARP,VRI,CCV,LYAP,SCC,fct
      integer for, fty
      common/model/for, fty
      integer ARI, tra
      common/model/ari,tra

C*****series.txt
      CHARACTER(LEN=40) NAME
      INTEGER LEN
      DOUBLE PRECISION TIM(5000),SER(5000),TDIF(5000)
c      COMMON/SERIES/NAME,LEN,TIM,SER,TDIF
      COMMON/SERIES/TIM,SER,TDIF,NAME,LEN
C*****setcon.txt
      LOGICAL CONV,FAIL
      INTEGER NP,ITCT,PPIND
      DOUBLE PRECISION CSO,CSZ,LAM,SSOLD,GMOLD,GMNEW,SIGSQ,OLDB(22)
c      COMMON/SETCON/CONV,FAIL,NP,ITCT,PPIND,CSO,CSZ,LAM,SSOLD,
c     *GMOLD,GMNEW,SIGSQ,OLDB
      COMMON/SETCON/CSO,CSZ,LAM,SSOLD,
     *GMOLD,GMNEW,SIGSQ,OLDB,CONV,FAIL,NP,ITCT,PPIND
C*****veccom.txt
      DOUBLE PRECISION B(22),NEWB(22),PERB(22),DELB(22),
     * ERR(5000),RES(5000),PRES(22,5000),sres(5000)
      COMMON/VECCOM/B,NEWB,PERB,DELB,ERR,RES,PRES,sres
C*****repcom.txt
      LOGICAL PRPI
      INTEGER REQSW,KFSW
      DOUBLE PRECISION VOB
c      COMMON/REPCOM/PRPI,REQSW,VOB,KFSW
      COMMON/REPCOM/VOB,REQSW,KFSW,PRPI 
C*****repar3.txt
      DOUBLE PRECISION ALPHA(21),ROOTR(20),ROOTI(20),XVECR(20),
     *XVECI(20)
      COMMON/REPAR3/ALPHA,ROOTR,ROOTI,XVECR,XVECI

C*****resgn2.txt
      DOUBLE PRECISION WK(20),VT(5000),BI(2,20,20),R(2,20,20),
     *RI(2,20,20)
c       DOUBLE PRECISION WK(20),VT(5000),BI(2,20,20),R(20,20),RI(2,20,20)
      INTEGER ERRNO1
      COMMON/RESGN2/WK,VT,BI,R,RI,ERRNO1
C*****resgd3.txt
      INTEGER IW1(20)
      DOUBLE PRECISION W1(20),W2(20),AL(20,20),BL(20,20),
     * CL(20,20),DL(20,20)
      DOUBLE PRECISION MS(20),NS(20),WS(20),WPS(20),AS(20,20),
     * CC(20,20),CS(20,20),CPS(20,20),AW0(20,20),AW1(20,20)
c      COMMON/RESGN3/IW1,W1,W2,AL,BL,CL,DL,MS,NS,WS,WPS,AS,CC,CS,
c     * CPS,AW0,AW1
      COMMON/RESGN3/W1,W2,AL,BL,CL,DL,MS,NS,WS,WPS,AS,CC,CS,
     * CPS,AW0,AW1,IW1

C*****kfasav.txt
      DOUBLE PRECISION PRE(5000),PRV(5000),FSER(5000),FVAR(5000)
     1,SSER(5000)
     1,SVAR(5000)
      DOUBLE PRECISION FSES(5000,20),FSCV(5000,20,20),SSES(5000,20)
     1,SSCV(5000,20,20),PSES(5000,20),PSCV(5000,20,20),TRAN(5000,20,20)
c      COMMON/KFASAV/PRE,PRV,FSER,FVAR,SSER,SVAR,FSES,FSCV,SSES,SSCV
c     1,PSES,PSCV,TRAN
      COMMON/KFASAV/FSES,FSCV,SSES,SSCV
     1,PSES,PSCV,TRAN,PRE,PRV,FSER,FVAR,SSER,SVAR

c      DO 110 I=1,ARP
c      WS(I)=CSZ
c      DO 110 J=1,ARP
c      CC(I,J)=CSZ
c      AS(I,J)=CSZ
c  110 CONTINUE
c      DO 111 J=2,ARP
c      AS(J-1,J)=CSO
c      AS(ARP,J-1)=-ALPHA(ARP+2-J)
c  111 CONTINUE
c      AS(ARP,ARP)=-ALPHA(1)
c      CC(ARP,ARP)=-CSO
c      DO 112 I=1,ARP
c      DO 112 J=1,ARP
c      AL(I,J)=AS(J,I)
c      CL(I,J)=CC(I,J)
c  112 CONTINUE
c      J=20
c      T=0
c      CALL LYBSC(ARP,AL,J,CL,J,BL,J,DL,W1,W2,T,I)
c      IF(I.NE.0)THEN
c        PRINT *,'PROGRAM FAILS IN SLICE ROUTINE LYBSC: ',I
c        STOP
c      END IF
c      DO 113 I=1,ARP
c      DO 113 J=1,ARP
c      U=BL(I,J)
c      CC(I,J)=U
c      CS(I,J)=U
c      BL(I,J)=AS(I,J)
c  113 CONTINUE
      E=0
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     print *,'main loop for predicting begins'
c      DO 150 T=1,LEN
c      if(fty.EQ.1)then
c           print *,'Length of data used to fit is ',LEN
c           print *,'fty=1, forecast past the end'
c      else
c      changed the next line 6/6/06
c           if(fct.EQ.1.AND.fty.EQ.2)then
c           print *,'fty=2, forecast last L-steps'
c           LEN=LEN-FOR
c           endif
c      endif  
      do 150 T=LEN+1,LEN+FOR
c     changed by Z.W. for predicting 20 steps temporially
      DEL=TDIF(T)
      J=20
      K=9
C      IF (KFSW.EQ.1) THEN
C        PRINT *,'JUST BEFORE MEPAD CALLED'
C        PRINT *,(ALPHA(KK),KK=1,ARP)
C      ENDIF
C     **** USE OF MEPAD ****
C     DEL IS THE TIME INTERVAL MULTIPLIER OF MATRIX BL IN THE EXP
C     DL IS THE RESULT EXP(DEL*BL)
C     AL AND CL ARE WORKING ARRAYS
C     *****             ****
      CALL MEPAD(ARP,K,DEL,BL,J,DL,J,L,M,IW1,AL,CL,W1,W2,I)
C      IF (KFSW.EQ.1)PRINT *,'JUST AFTER MEPAD CALLED'
      IF(I.EQ.4)THEN
        call rexit('PROGRAM FAILS IN SLICE ROUTINE MEPAD')
C        PRINT *,'PROGRAM FAILS IN SLICE ROUTINE MEPAD: ',I
C        STOP
        ERRNO1=31
        RETURN

      ELSE IF(I.NE.0)THEN
        E=I
      END IF
C     **** BL IS RESTORED HAVING BEEN MODIFIED BY MEPAD
      DO I=1,ARP
      DO J=1,ARP
      BL(I,J)=AS(I,J)
      AW0(I,J)=CC(I,J)-CS(I,J)
      AW1(I,J)=DL(I,J)
      END DO
      END DO
      DO I=1,ARP
      DO J=1,ARP
      U=CSZ
      DO 115 K=1,ARP
      U=U+AW0(I,K)*AW1(J,K)
  115 CONTINUE
      CC(I,J)=U
      END DO
      END DO
      DO I=1,ARP
      DO J=1,ARP
      U=CSZ
      DO 117 K=1,ARP
      U=U+AW1(I,K)*CC(K,J)
  117 CONTINUE
      CPS(I,J)=U+CS(I,J)
      END DO
      END DO
C     **** AW1 IS TRANSITION MATRIX
C          CS IS STATIONARY STATE COVARIANCE
C          CC IS OLD STATE COVARIANCE
C     **** CPS IS PREDICTED STATE COVARIANCE
      DO 120 I=1,ARP
      U=CSZ
      DO 119 J=1,ARP
      U=U+AW1(I,J)*WS(J)
  119 CONTINUE
      WPS(I)=U
  120 CONTINUE
C     **** WS IS OLD STATE ESTIMATE
C     **** WPS IS PREDICTED STATE ESTIMATE
      DO 122 I=1,ARP
      U=CSZ
      DO 121 J=1,ARP
      U=U+CPS(I,J)*XVECR(J)
  121 CONTINUE
      MS(I)=U
  122 CONTINUE
      PEV=CSZ
      PRED=CSZ
      DO 123 I=1,ARP
      U=XVECR(I)
      PEV=PEV+U*MS(I)
c     PEV is Jones (1981), (40), predicted error variance?
      PRED=PRED+U*WPS(I)
c      print *,'prediction',PRED
c     PRED is Jones (1981), (38), predict the obs. at time t_k
  123 CONTINUE
c     VOB is R in Jones (40)
      PEV=PEV+VOB
      VT(T)=PEV
c     VT(T) is Jones (43)?
c      U=SER(T)-CONST-PRED
c     for predicting, set U=0
      U=0
c     U is the innovation, Jones (42)
      ERR(T)=U
      DO 124 I=1,ARP
      NS(I)=MS(I)/PEV
      WS(I)=WPS(I)+NS(I)*U
c     WS(I) is the gain K_t, for predicting, set to 0
  124 CONTINUE
      DO I=1,ARP
      U=MS(I)
      DO J=1,ARP
      CC(I,J)=CPS(I,J)-U*NS(J)
c     CC is Jones (41)
      END DO
      END DO
C     **** CC IS NOW PRESENT (UPDATED) STATE COVARIANCE 
C     **** WS IS NOW PRESENT (UPDATED) STATE ESTIMATE
      IF (KFSW.EQ.1) THEN
        PRE(T)=CONST+PRED
        PRV(T)=PEV
        U=CSZ
        DO I=1,ARP
          FSES(T,I)=WS(I)
c     FSES is the filtered state
          PSES(T,I)=WPS(I)
          U=U+XVECR(I)*WS(I)
        END DO
        FSER(T)=U+CONST
c     FSER is the filtered series
c        print *, 'U:',U
        U=CSZ
        DO I=1,ARP
          DO J=1,ARP
            FSCV(T,I,J)=CC(I,J)
            PSCV(T,I,J)=CPS(I,J)
            TRAN(T,I,J)=AW1(I,J)
            U=U+CC(I,J)*XVECR(I)*XVECR(J)
          END DO
        END DO
c     U is Jones (43) without R
        FVAR(T)=U
      ENDIF
  150 CONTINUE
C      PRINT *,'MAIN LOOP IN RESG1 COMPLETED'
c      print *,'main loop for predicting completed'
      IF(E.NE.0)THEN
C        PRINT *,'WARNING IN SLICE ROUTINE MEPAD: ',E
      END IF
      RETURN
      END

*UPTODATE STORETEXT
      DOUBLE PRECISION FUNCTION STORE(A,B)
C
C        MARK 2.2 RELEASE.  NAG COPYRIGHT 1987.
C
C     .. Scalar Arguments ..
      DOUBLE PRECISION A,B
C     .. Executable Statements ..
C
C     STORE is intended to force A and B to be stored prior to doing the
C     addition of A and B. For use in situations where optimizers might
C     hold one of these in a register.
C
C
C  Nag Fortran 77 O( 1 ) basic linear algebra routine.
C
C  -- Written on 28-November-1984.
C     Sven Hammarling, Nag Central Office.
C
      STORE = A + B
C
      RETURN
C
C     End of STORE .
C
      END

"factab" <- 
function (object) 
{
    if(!inherits(object, "car"))
    stop("object must be car\n")
    call <- match.call()
    par <- object$phi
    p <- object$order
    k <- object$scale
    w <- sort(round(polyroot(rev(c(1, par[1:p]))),4))
    r <- -k * (1 - w)/(1 + w)
    sortind <- sort.list(abs(Re(r)), decreasing = TRUE)
    r <- rev(r[sortind]) #changed  
    car.freq <- abs(Im(r))/(2 * pi)
    structure(list(call=call,root=r,freq=car.freq),
              class="factab")
}

print.factab <- 

function (x, digits = 3, ...) 
{
    cat("\nCall:\n", deparse(x$call), "\n", sep = "")
    cat("\nCharacteristic root of original parameterization in alpha", "\n\n")
    root <- drop(round(x$root, digits=digits))
    freq <- drop(round(x$freq, digits=digits))
    names(root) <- seq(length = length(x$freq))
    print.default(root, print.gap = 2)
    cat("\nFrequency", "\n\n")
    names(freq) <- seq(length = length(x$freq))
    print.default(freq, print.gap = 2)
    invisible(x)
  }

#.First.lib <- function(lib, pkg)
.onLoad <- function(lib, pkg)
        library.dynam("cts", pkg, lib)

car_control <- function(fty=1, n.ahead=10, trace=FALSE, ari=TRUE, vri=FALSE, vr=0, pfi="MAPS",ccv="CTES", lpv=TRUE, scc=TRUE,  nit=40, opm=1, rgm=1, req=0.5, con=1.0e-5, rpe=1.0, ivl=1.0e-2, fac=1.0e1, stl=1.0e-5, sml=1.0e2, gtl=1.0e5, kst=TRUE, fct=TRUE){
  RET <- list(fty=fty, n.ahead=n.ahead, trace=trace, ari=ari, vri=vri, vr=vr, pfi=pfi,ccv=ccv, lpv=lpv, scc=scc, nit=nit, opm=opm, rgm=rgm, req=req, con=con, rpe=rpe, ivl=ivl, fac=fac, stl=stl, sml=sml, gtl=gtl, kst=kst, fct=fct)
  class(RET) <- c("car_control")
  RET
}

car <-
function(x, y=NULL, scale=1.5, order=3, ctrl=car_control())
  {
    call <- match.call()
 fty=ctrl$fty; n.ahead=ctrl$n.ahead; trace=ctrl$trace
    ari=ctrl$ari; phi=rep(0, order); vri=ctrl$vri; vr=ctrl$vr; pfi=ctrl$pfi; ccv=ctrl$ccv; lpv=ctrl$lpv; scc=ctrl$scc; nit=ctrl$nit; opm=ctrl$opm; rgm=ctrl$rgm; req=ctrl$req; con=ctrl$con; rpe=ctrl$rpe; ivl=ctrl$ivl; fac=ctrl$fac; stl=ctrl$stl; sml=ctrl$sml; gtl=ctrl$gtl; kst=ctrl$kst; fct=ctrl$fct;
    if (NCOL(x)==2){
      tim <- x[,1]
      ser <- x[,2]
   series <- deparse(substitute(x))}
    else {
      tim <- x
      ser <- y
      series <- deparse(substitute(y))
    }
    if (length(tim)!=length(ser)) stop("Number of time and observations are not equal")
    len <- length(tim)
    if (len > 5000) stop("Number of time beyond 5000 is not implemented")
    ### corresponding to setup.f
    csz <- 0
    if (pfi=="QLFA") pfi <- 1
    if (pfi=="QLFS") pfi <- 2
    if (pfi=="DIRA") pfi <- 3
    if (pfi=="DIRS") pfi <- 4
    if (pfi=="MAPA") pfi <- 5
    if (pfi=="MAPS") pfi <- 6
    if (pfi==0) stop("Invalid PFI option:",pfi)
#    scale <- sca
    if (scale <= csz) stop("Invalid SCALE value:",scale)
    if (order <= 0 || order > 20) stop("Model order value must between 1 and 20")

    if (ari==FALSE) ari <- 0
    if (ari==TRUE){
      if (order==length(phi)) ari <- 1
      else stop("Initial phi value length not equal to the model order")}
    if (ari < 0) stop("Invalid ARI option:", ari)
    if (ari==0) phi <- rep(0,order)

    if (vri==TRUE) vri <- 1
    if (vri==FALSE) vri <- 0
    if (vri < 0) stop("Invalid VRI option:", vri)
    if (vri==0) vr <- csz
    if (vr < 0) stop("Invalid VR value:", vr)
    if (is.null(ccv)) ccv <- 0
#    if (ccv=="NULL") ccv <- 0
    if (ccv=="MNCT") ccv <- 1
    if (ccv=="CTES") ccv <- 2
    if (ccv < 0) stop("Invalid CCV option:", ccv)

    if (lpv==FALSE) lyap <- 0
    if (lpv==TRUE) lyap <- 1
    if (lyap < 0) stop("Invalid LPV option:", lpv)
    if (lyap==1) prdg <- csz
    if (prdg < 0) stop("Invalid PRDG value:", prdg)
    
    if (scc==FALSE) scc <- 0
    if (scc==TRUE) scc <- 1
    if (scc < 0) stop("Invalid SCC option:", scc)
    if (scc==0 && lyap==1)
      {
        lyap <- 0
        prdg <- 1.0e4
        if (trace) message("SCC=N causes reset of LPV=Y to LPV=N and PRDG=1.0D4")
      }
    if (n.ahead < 0 || n.ahead > 5000) stop("Invalid forecasting steps:", n.ahead)
    if (trace) message("Reading of model parameters successful")
    
    if (nit < 0 || nit > 100) nit <- 25
    if (opm < 0) stop("Invalid OPM value:", opm)
    if (rgm < 0) stop("Invalid RGM value:", rgm)
    if (req < 0) stop("Invalid REQ value:", req)
    concrit <- con
    if (con < 0) stop("Invalid CONCRIT value:", concrit)
    rpert <- rpe
    if (rpe < 0) stop("Invalid RPERT value:", rpert)
    ivlam <- ivl
    if (ivlam < 0) stop("Invalid IVLAM value:", ivlam)
    if (fac < 0) stop("Invalid FAC value:", fac)
    stlam <- stl
    if (stlam < 0) stop("Invalid STLAM value:", stlam)
    smlam <- sml
    if (smlam < stlam) stop("Invalid SMLAM value:", smlam)
    gtlam <- gtl
    if (gtlam < 0) stop("Invalid GTLAM value:", gtlam)
    
    if (kst==FALSE) kst <- 0
    if (kst==TRUE) kst <- 1
    if (kst < 0) stop("Invalid KST value:", kst)

    if (fct==FALSE) fct <- 0
    if (fct==TRUE) fct <- 1
    if (fct < 0) stop("Invalid FCT value:", fct)

    if(fty < 1 || fty > 3) stop("Invalid fty value:", fty)
    tra <- 0
    if (trace) {
      tra <- 1
      message("Reading of control parameters successful")
    }
    np1 <- 0
    z <-.Fortran("setup",
                 as.integer(pfi),
                 as.integer(order),
                 as.integer(vri),
                 as.integer(ccv),
                 as.double(scale),
                 as.integer(ari),
                 as.double(vr),
                 as.double(phi),
                 as.integer(lyap),
                 as.double(prdg),
                 as.integer(scc),
                 as.integer(fct),
                 as.integer(fty),
                 as.integer(len),
                 as.integer(n.ahead),
                 as.double(tim),
                 as.double(ser),
                 as.integer(nit),
                 as.integer(opm),
                 as.integer(rgm),
                 as.double(req),
                 as.double(concrit),
                 as.double(rpert),
                 as.double(ivlam),
                 as.double(fac),
                 as.double(stlam),
                 as.double(smlam),
                 as.double(gtlam),
                 as.integer(kst),
                 np1=as.integer(np1),
                 as.integer(tra),
                 PACKAGE="cts")
        zpar <- .Fortran("loop",
                ss=double(nit+1),
                bit=as.double(matrix(0, nit+1, 22)),
                errno=integer(1),  PACKAGE="cts")
        if(zpar$errno == 1) stop("ROOT WITH POSITIVE REAL PART, WHICH WAS CALLED IN loop.f")
        else if(zpar$errno == 2) stop("ERROR IN LAPACK SUBROUTINE zgesv, WHICH WAS CALLED IN cinvert.f")
        else if(zpar$errno == 3) stop("PROGRAM FAILS IN SLICE ROUTINE LYBSC, WHICH WAS CALLED IN resg1d.f")
        else if(zpar$errno == 30) stop("PROGRAM FAILS IN SLICE ROUTINE LYBSC, WHICH WAS CALLED IN resg1dpre.f")
        else if(zpar$errno == 31) stop("PROGRAM FAILS IN SLICE ROUTINE LYBSC, WHICH WAS CALLED IN resg1dpre1.f")
        else if(zpar$errno == 4) stop("PROGRAM FAILS IN SLICE ROUTINE MEPAD, WHICH WAS CALLED IN resg1d.f")
        else if(zpar$errno == 40) stop("PROGRAM FAILS IN SLICE ROUTINE MEPAD, WHICH WAS CALLED IN resg1dpre.f")
        else if(zpar$errno == 51) stop("PROGRAM FAILS IN SUBROUTINE DPOTRF, WHICH WAS CALLED IN revg1.f")
        else if(zpar$errno == 52) stop("PROGRAM FAILS IN SUBROUTINE DPOTRI, WHICH WAS CALLED IN revg1.f")
        else if(zpar$errno == 6) stop("PROGRAM FAILS IN SUBROUTINE DIVC , WHICH WAS CALLED IN dpca.f")
        neff <- zpar$ss > 0
        zpar$tnit <- seq(0, sum(neff))
        zpar$ss <- zpar$ss[neff]
        zpar$bit <- matrix(zpar$bit, nrow=nit+1)
        zpar$bit <- zpar$bit[neff, 1:(order+1)]
        zfin <- .Fortran("complete",
                ss=double(1),
                bit=double(22),
                PACKAGE="cts")
        zpar$ss <- c(zpar$ss, zfin$ss)
        zpar$bit <- rbind(zpar$bit, zfin$bit[1:(order+1)])
    Z <- .Fortran("setcom",
                  pfi1=integer(1),
                  arp1=integer(1),
                  np1=integer(1),
                  vri1=integer(1),
                  ccv1=integer(1),
                  len1=integer(1),
                  scale1=double(1),            
                  vr1=double(1),
                  sigsq1=double(1),
                  ##ESSP1 in Fortran has dimention (22,22)
                  essp1=as.double(matrix(0,22,22)),
                  ecov1=as.double(matrix(0,22,22)),
                  b1= double(z$np1),
                  delb1=double(z$np1),
                  rootr1=double(order),
                  rooti1=double(order),
                  PACKAGE="cts")
    essp <- matrix(Z$essp1,byrow=FALSE,ncol=22)
    essp <- essp[1:Z$np1,1:Z$np1]
    ecov <- matrix(Z$ecov1,byrow=FALSE,ncol=22)
    ecov <- ecov[1:order,1:order]
    n <- length(tim)
  if (kst==1)
    {     
    .Fortran("kfilsm",PACKAGE="cts")   
    Zkfil <- .Fortran("setkfilsm",
                      fser1=double(n),
                      fvar1=double(n),
                      sser1=double(n),
                      svar1=double(n),
                      sres1=double(n),
                      PACKAGE="cts")
    filser <- Zkfil$fser1
    filvar <- Zkfil$fvar1
    sser <- Zkfil$sser1
    svar <- Zkfil$svar1
    sres <- Zkfil$sres1
  }
    else
      {
        filser <-  NULL
        filvar <- NULL
        sser <- NULL
        svar <- NULL
        sres <- NULL
      }

        if (pfi==1)
          ST1 <- 'QLFA'
        else if (pfi==2)
          ST1 <- 'QLFS'
        else if (pfi==3)
          ST1 <- 'DIRA'
        else if (pfi==4)
          ST1 <- 'DIRS'
        else if (pfi==5)
          ST1 <- 'MAPA'
    else
    ST1 <- 'MAPS'

    if (ccv==0)
      ST2 <- NULL
    else if (ccv==1) ST2 <- 'MNCT'
    else ST2 <- 'CTES'
    
    if (vri==1) ST3 <- 'N'
    else ST3 <- 'Y'

    .Fortran("update",PACKAGE="cts")
    phi <- .Fortran("setupdate",
                    phi1=double(20),
    ###             phi1=double(order), ### changed to avoid array overrun, 4/4/2012
                    PACKAGE="cts")$phi 
    ### I am not sure this phi is updated
    if (n.ahead > 0)
      {
        .Fortran("forecast",PACKAGE="cts")
        if(fct==TRUE && fty==1)
          ntim <- len+n.ahead
        else ntim <- len
        Zfor <- .Fortran("setfor",
                         pre1=double(n.ahead),
                         prv1=double(n.ahead),
                         tim1=double(ntim),
                         pre2=double(5000),
                         prv2=double(5000),
                         PACKAGE="cts")
        pre <- Zfor$pre1
        prv <- Zfor$prv1
        tim <- Zfor$tim1
        pre2 <- Zfor$pre2[Zfor$prv2 > 0]
        prv2 <- Zfor$prv2[Zfor$prv2 > 0]
      }
    else
      {
        pre <- NULL
        prv <- NULL
      }
    ntim <- length(ser); ntim1 <- length(tim)-length(pre)
    ntim2 <- length(tim)
    aic <- ntim * log(zpar$ss[length(zpar$tnit)]) + 2*Z$np1
    bic <- ntim * log(zpar$ss[length(zpar$tnit)]) + Z$np1*log(ntim)
    structure(list(call=call,series = series,order=Z$arp1,
                   np=Z$np1,scale=Z$scale1,
                   vri=Z$vri1, x.mean=ifelse(vri==1, 
                   Z$b1[order+2], Z$b1[order+1]), 
                   vob=ifelse(vri==1,Z$b1[order+1]^2, NA), vr=Z$vr1,
                   sigma2=Z$sigsq1,phi=Z$b1[1:order],
                   #sigma2=Z$sigsq1,phi=phi,
                   b=Z$b1,delb=Z$delb1,essp=essp,
                   ecov=ecov,
                   rootr=Z$rootr1,rooti=Z$rooti1,
                   tim=tim[1:ntim],ser=ser,n.used=Z$len1,
                   filser=filser,filvar=filvar,
                   sser=sser,svar=svar,
                   stdres=sres, pretime = tim[(ntim1+1):ntim2],
                   predict=pre,predict.var=prv*Z$sigsq1, pre2=pre2, prv2=prv2*Z$sigsq1, fty=fty, tnit=zpar$tnit, ss=zpar$ss, bit=zpar$bit, aic=aic, bic=bic),
              class="car")
  }

print.car <- function(x, digits = 3, ...)
{
  cat("\nCall:\n", deparse(x$call), "\n\n", sep = "")
  coef <- drop(round(x$phi, digits = digits))
  cat("\nEstimated coefficients\n")
  names(coef) <- paste("phi_",seq(length=x$order), sep="")
  print(coef)
  invisible(x)
}

summary.car <- function(object, ...)
{
  x <- object
  digits <- 3
  cat("\nCall:\n", deparse(x$call), "\n", sep = "")
  cat(paste("\nOrder of model = ", x$order, ", sigma^2 = ",
      format(x$sigma2, digits = digits), sep=""),"\n")
  tmp <- coef <- drop(round(x$phi, digits = digits))
#  coef <- drop(round(c(x$phi, x$x.mean), digits = digits))
#  names(coef) <- c(seq(length=x$order), "mean")
  if(x$vri==1){
#  tmp <- rbind(coef, x$delb[-(x$order+1)])
  cat("\nObservation error variance:",format(x$vob*x$sigma2, digits=digits),"\n")
}
  tmp <- rbind(coef, x$delb[1:x$order])
  cat("\nEstimated coefficients (standard errors):\n")
  colnames(tmp) <- paste("phi_",seq(length=x$order), sep="")
  rownames(tmp) <- c("coef", "S.E.")
  print(round(tmp, digits))
  if(!is.na(x$x.mean)){
  cat("\nEstimated mean (standard error):\n")
  print(round(x$x.mean, digits=digits))
  print(round(x$delb[x$np], digits))
}
  corout <- FALSE
  if(corout){
  cat("\nCorrelations:\n")
  x$essp <- round(x$essp, digits)
  x$essp[upper.tri(x$essp)] <- ""
  tmp <- as.data.frame(x$essp)
  colnames(tmp) <- 1:dim(x$essp)[1]
  print(tmp)
  }
  invisible(x)
}

predict.car <- function(object, se.fit=TRUE, digits = 3, plot.it=TRUE, ...)
{
      ar <- object$phi
      p <- object$order
      pretime <- object$pretime
      names(pretime) <- seq(length=length(object$predict))
      pred <- drop(round(object$predict,digits = digits))
      names(pred) <- seq(length=length(object$predict))
      prv <- drop(round(object$predict.var,digits = digits))
      names(prv) <- seq(length=length(object$predict.var))
      cat("\nCall:\n", deparse(object$call), "\n\n", sep = "")
      tmp <- rbind(pretime, pred)
      rownames(tmp) <- c("Time", "Predict") 
      print(tmp)
      if(plot.it)
      plot.predict.car(object, ...) 
      RET <- list(pretime=pretime, pred=pred)
}

plot.predict.car <- function(x, xlab = "time",
    ylab = "", type = "l", main = NULL, sub = NULL, ...)
  {
    if(!inherits(x, "car"))
    stop("object must be car\n")
    if(x$fty==1){ ### forecast type is beyond of the observation
    plot(x$tim, x$ser,
         xlab=xlab, ylab=ylab, type=type, main=main,sub=sub, ...)
    points(x$pretime, x$predict, lty=2, col="red", ...)
    }
    else{
    npre <- length(x$pretime)
    ntim <- length(x$tim)
    plot(x$tim, x$ser,
         xlab=xlab,ylab=ylab,type=type,main=main,sub=sub, ...)
#    plot(object$tim[1:length(object$ser)],object$ser,ylim=c(min(object$ser)-2*max(sqrt(object$predict.var)),max(object$ser)+2*max(sqrt(object$predict.var))),xlab=xlab,ylab=ylab,type=type,main=main,sub=sub, ...)
    points(x$pretime, x$predict,
           lty=2,col="red", ...)
}
#    lines(object$tim[(length(object$tim) - length(object$predict) +
#                      1):length(object$tim)],object$predict+2*sqrt(object$predict.var),lty=2,col="blue")
#    lines(object$tim[(length(object$tim) - length(object$predict) +
#                      1):length(object$tim)],object$predict-2*sqrt(object$predict.var),lty=2,col="blue")
  }

tsdiag <- function(object, ...)
    UseMethod("tsdiag")

tsdiag.car <- function(object, gof.lag = 10, ...)
{
    ## plot standardized residuals, acf of residuals,
  ## cumulative periodogram and Ljung-Box p-values
    if(!inherits(object, "car"))
    stop("object must be 'car' class\n") 
    oldpar<- par(mfrow = c(2, 2))
    on.exit(par(oldpar))
    stdres <- object$stdres
    plot(stdres, type = "h", main = "Standardized Residuals", ylab = "")
    abline(h = 0)
    acf(object$stdres, plot = TRUE, main = "ACF of Standardized Residuals",
        na.action = na.pass)
    cpgram(stdres,main="Cumulative periodogram")
    nlag <- gof.lag
    pval <- numeric(nlag)
    for(i in 1:nlag) pval[i] <- Box.test(stdres, i, type="Ljung-Box")$p.value
    plot(1:nlag, pval, xlab = "lag", ylab = "p value", ylim = c(0,1),
         main = "p values for Ljung-Box statistic")
    abline(h = 0.05, lty = 2, col = "blue")
    invisible(object)
}

      SUBROUTINE RESGEN
      IMPLICIT NONE
      INTEGER T
      DOUBLE PRECISION U,V
c      INCLUDE 'series.txt'
c      INCLUDE 'setcon.txt'
c      INCLUDE 'veccom.txt'
c      INCLUDE 'repcom.txt'
c      INCLUDE 'resgn2.txt'
C*****series.txt
      CHARACTER(LEN=40) NAME
      INTEGER LEN
      DOUBLE PRECISION TIM(5000),SER(5000),TDIF(5000)
c      COMMON/SERIES/NAME,LEN,TIM,SER,TDIF
      COMMON/SERIES/TIM,SER,TDIF,NAME,LEN
C*****setcon.txt
      LOGICAL CONV,FAIL
      INTEGER NP,ITCT,PPIND
      DOUBLE PRECISION CSO,CSZ,LAM,SSOLD,GMOLD,GMNEW,SIGSQ,OLDB(22)
c      COMMON/SETCON/CONV,FAIL,NP,ITCT,PPIND,CSO,CSZ,LAM,SSOLD,
c     *GMOLD,GMNEW,SIGSQ,OLDB
      COMMON/SETCON/CSO,CSZ,LAM,SSOLD,
     *GMOLD,GMNEW,SIGSQ,OLDB,CONV,FAIL,NP,ITCT,PPIND
C*****veccom.txt
      DOUBLE PRECISION B(22),NEWB(22),PERB(22),DELB(22),
     * ERR(5000),RES(5000),PRES(22,5000),sres(5000)
      COMMON/VECCOM/B,NEWB,PERB,DELB,ERR,RES,PRES,sres
C*****repcom.txt
      LOGICAL PRPI
      INTEGER REQSW,KFSW
      DOUBLE PRECISION VOB
c      COMMON/REPCOM/PRPI,REQSW,VOB,KFSW
      COMMON/REPCOM/VOB,REQSW,KFSW,PRPI 
C*****resgn2.txt
      DOUBLE PRECISION WK(20),VT(5000),BI(2,20,20),R(2,20,20),
     *RI(2,20,20)
c       DOUBLE PRECISION WK(20),VT(5000),BI(2,20,20),R(20,20),RI(2,20,20)
      INTEGER ERRNO1
      COMMON/RESGN2/WK,VT,BI,R,RI,ERRNO1
      INTEGER PFI,ARP,VRI,CCV,LYAP,SCC,fct
      DOUBLE PRECISION SCALE,VR,CONST,PHI(20),PRDG
      COMMON/MODEL/SCALE,VR,CONST,PHI,PRDG,PFI,ARP,VRI,CCV,LYAP,SCC,fct
      integer for, fty
      common/model/for, fty
      integer ARI, tra
      common/model/ari,tra

c      print *,'PPIND: ',PPIND
      IF(tra.EQ.1)THEN
      IF(PPIND.EQ.0)THEN
        call intpr('ROOT EQUALITY SWITCH: ', 22, REQSW, 1)
      END IF
      END IF
      IF(REQSW.EQ.0)THEN
        CALL RESG0
      ELSE
        CALL RESG1
      END IF
      U=CSZ
      DO 100 T=1,LEN
      U=U+DLOG(VT(T))
  100 CONTINUE
c      PRINT *,'SUM LOG VARIANCES FORMED'
      U=DEXP((CSO/LEN)*U)
      GMNEW=U
      DO 101 T=1,LEN
      V=ERR(T)*DSQRT(U/VT(T))
      IF(PPIND.EQ.0)THEN
        RES(T)=V
      ELSE
        PRES(PPIND,T)=V
      END IF
  101 CONTINUE
      RETURN
      END

      SUBROUTINE UPDATE
      IMPLICIT NONE
      CHARACTER(LEN=1) ST9
      CHARACTER(LEN=4) ST0,ST1
      INTEGER I,J,M,N
c      INCLUDE 'model.txt'
c      INCLUDE 'setcon.txt'
C*****model.txt
      INTEGER PFI,ARP,VRI,CCV,LYAP,SCC,fct
      DOUBLE PRECISION SCALE,VR,CONST,PHI(20),PRDG
      COMMON/MODEL/SCALE,VR,CONST,PHI,PRDG,PFI,ARP,VRI,CCV,LYAP,SCC,fct
      integer for, fty
      common/model/for, fty
      integer ARI, tra
      common/model/ari,tra
             
C*****setcon.txt
      LOGICAL CONV,FAIL
      INTEGER NP,ITCT,PPIND
      DOUBLE PRECISION CSO,CSZ,LAM,SSOLD,GMOLD,GMNEW,SIGSQ,OLDB(22)
c      COMMON/SETCON/CONV,FAIL,NP,ITCT,PPIND,CSO,CSZ,LAM,SSOLD,
c     *GMOLD,GMNEW,SIGSQ,OLDB
      COMMON/SETCON/CSO,CSZ,LAM,SSOLD,
     *GMOLD,GMNEW,SIGSQ,OLDB,CONV,FAIL,NP,ITCT,PPIND

      if(tra.EQ.1)then
C      PRINT *,'MODEL UPDATE IN PROGRESS'
      end if
      I=ARP+1
C        IF(I.GT.20)THEN
C          PRINT *,'PROGRAM FAILS: MAXIMUM ORDER EXCEEDED IN UPDATE'
C          STOP
C        END IF
c        OPEN(UNIT=4,FILE='newmodel.dat',STATUS='unknown')
        ST0='PFI='
        IF(PFI.EQ.1)THEN
          ST1='QLFA'
        ELSE IF(PFI.EQ.2)THEN
          ST1='QLFS'
        ELSE IF(PFI.EQ.3)THEN
          ST1='DIRA'
        ELSE IF(PFI.EQ.4)THEN
          ST1='DIRS'
        ELSE IF(PFI.EQ.5)THEN
          ST1='MAPA'
        ELSE
          ST1='MAPS'
        END IF
c        WRITE(4,100)ST0,ST1
c  100   FORMAT(2A4)
        ST0='SCA='
c        WRITE(4,101)ST0,SCALE
c  101   FORMAT(A4,D14.6)
        ST0='ARP='
c        WRITE(4,102)ST0,I
c  102   FORMAT(A4,I3)
        ST0='ARI='
        ST9='Y'
c        WRITE(4,103)ST0,ST9
c  103   FORMAT(A4,A1)
        DO 200 J=1,ARP
        PHI(J)=OLDB(J)
  200   CONTINUE
        IF(PFI.EQ.1)THEN
C         TO BE DEFINED LATER
        ELSE IF(PFI.EQ.2)THEN
C         TO BE DEFINED LATER
        ELSE IF(PFI.EQ.3.OR.PFI.EQ.4)THEN
          PHI(I)=PHI(ARP)*SCALE
          DO 201 J=2,ARP
          M=ARP-J
          N=M+2
          PHI(N)=PHI(N)+PHI(M+1)*SCALE
  201     CONTINUE
          PHI(1)=PHI(1)+SCALE
        ELSE
          PHI(I)=CSZ
        END IF
        DO 202 J=1,I
c        WRITE(4,104)PHI(J)
c  104   FORMAT(D14.6)
  202   CONTINUE
        ST0='VRI='
        IF(VRI.EQ.0)THEN
          ST9='N'
        ELSE
          ST9='Y'
        END IF
c        WRITE(4,103)ST0,ST9
        IF(VRI.EQ.1)THEN
c          WRITE(4,104)VR
        END IF
        ST0='CCV='
        IF(CCV.EQ.0)THEN
          ST1='NULL'
        ELSE IF(CCV.EQ.1)THEN
          ST1='MNCT'
        ELSE
          ST1='CTES'
        END IF
c       WRITE(4,100)ST0,ST1
c        CLOSE(UNIT=4)
      RETURN
      END

      SUBROUTINE ADDC(A,B,C,D,E,F)
      IMPLICIT NONE
      DOUBLE PRECISION A,B,C,D,E,F
      DOUBLE PRECISION WK(20),VT(5000),BI(2,20,20),R(2,20,20),
     *RI(2,20,20)
      INTEGER ERRNO1
      COMMON/RESGN2/WK,VT,BI,R,RI,ERRNO1
                        
      E=A+C
      F=B+D
      RETURN
      END
      SUBROUTINE SUBC(A,B,C,D,E,F)
      DOUBLE PRECISION A,B,C,D,E,F
      E=A-C
      F=B-D
      RETURN
      END
      SUBROUTINE DIVC(A,B,C,D,E,F)
      DOUBLE PRECISION A,B,C,D,E,F,R
      R=C*C+D*D
      IF(R.EQ.0.0D0)THEN
C        PRINT *,'FAILURE IN COMPLEX DIVISION'
         call rexit('FAILURE IN COMPLEX DIVISION')
C        STOP
      ERRNO1 = 6
      RETURN

      END IF
      E=(C*A+D*B)/R
      F=(C*B-D*A)/R
      RETURN
      END
      SUBROUTINE MULC(A,B,C,D,E,F)
      DOUBLE PRECISION A,B,C,D,E,F
      E=A*C-B*D
      F=A*D+B*C
      RETURN
      END
      SUBROUTINE MULR(A,B,C,E,F)
      DOUBLE PRECISION A,B,C,E,F
      E=A*C
      F=B*C
      RETURN
      END

      SUBROUTINE D6RPLY(A, B1, C, SR, SI, LR, LI)
C CALCULATE THE ZEROS OF THE QUADRATIC A*Z**2+B1*Z+C.
C THE QUADRATIC FORMULA, MODIFIED TO AVOID
C OVERFLOW, IS USED TO FIND THE LARGER ZERO IF THE
C ZEROS ARE REAL AND BOTH ZEROS ARE COMPLEX.
C THE SMALLER REAL ZERO IS FOUND DIRECTLY FROM THE
C PRODUCT OF THE ZEROS C/A.
      DOUBLE PRECISION A, B1, C, SR, SI, LR, LI, B,
     1 D, E, DSQRT
      IF (A.NE.0.D0) GO TO 20
      SR = 0.D0
      IF (B1.NE.0.D0) SR = -C/B1
      LR = 0.D0
   10 SI = 0.D0
      LI = 0.D0
      RETURN
   20 IF (C.NE.0.D0) GO TO 30
      SR = 0.D0
      LR = -B1/A
      GO TO 10
C COMPUTE DISCRIMINANT AVOIDING OVERFLOW
   30 B = B1/2.D0
      IF (DABS(B).LT.DABS(C)) GO TO 40
      E = 1.D0 - (A/B)*(C/B)
      D = DSQRT(DABS(E))*DABS(B)
      GO TO 50
   40 E = A
      IF (C.LT.0.D0) E = -A
      E = B*(B/DABS(C)) - E
      D = DSQRT(DABS(E))*DSQRT(DABS(C))
   50 IF (E.LT.0.D0) GO TO 60
C REAL ZEROS
      IF (B.GE.0.D0) D = -D
      LR = (-B+D)/A
      SR = 0.D0
      IF (LR.NE.0.D0) SR = (C/LR)/A
      GO TO 10
C COMPLEX CONJUGATE ZEROS
   60 SR = -B/A
      LR = SR
      SI = DABS(D/A)
      LI = -SI
      RETURN
      END

      SUBROUTINE DRPOLY(DEGREE,COEFF,ZEROR,ZEROI)
C
C FINDS THE ZEROS OF A POLYNOMIAL WITH REAL COEFFICIENTS.
C
C  INPUTS -
C
C     COEFF         -  DOUBLE PRECISION VECTOR OF THE COEFF-
C                      ICIENTS IN ORDER OF DECREASING POWERS.
C
C     DEGREE        -  INTEGER DEGREE OF POLYNOMIAL.
C
C  OUTPUTS -
C
C     ZEROR, ZEROI  -  DOUBLE PRECISION VECTORS OF REAL
C                      AND IMAGINARY PARTS OF THE ZEROS.
C
C THE SUBROUTINE USES SINGLE PRECISION CALCULATIONS
C FOR SCALING, BOUNDS AND ERROR CALCULATIONS. ALL
C CALCULATIONS FOR THE ITERATIONS ARE DONE IN DOUBLE
C PRECISION.
C
C
C  ERROR STATES -
C
C    1    - DEGREE IS LESS THAN 1
C    2    - LEADING COEFFICIENT IS ZERO
C    3    - THE DYNAMIC STORAGE STACK IS NOT BIG ENOUGH
C    NNN  - ONLY NNN (.LT. DEGREE) ZEROS HAVE BEEN
C          FOUND (RECOVERABLE)
C
C
C PORT NOTE -
C
C THE ORIGINAL PROGRAM HAS BEEN ADAPTED TO PORT BY -
C
C   (1) PUTTING IN AUTOMATIC ERROR HANDLING.
C   (2) SUBSTITUTING DYNAMIC STACK ALLOCATION FOR THE DIMENSIONED
C       ARRAYS IN NAMED COMMON.
C   (3) CHANGING THE NAMES OF THE INTERNAL ROUTINES TO AVOID USER
C       NAME CONFLICT.
C
C  THE FOLLOWING NAME EQUIVALENCES (ORIGINAL - NEW) APPLY -
C
C            RPOLY    -   D1RPLY
C            CALCSC   -   D2RPLY
C            FXSHFR   -   D3RPLY
C            NEWEST   -   D4RPLY
C            NEXTK    -   D5RPLY
C            QUAD     -   D6RPLY
C            QUADIT   -   D7RPLY
C            QUADSD   -   D8RPLY
C            REALIT   -   D9RPLY
C
C
C DYNAMIC STORAGE SPACE USED -
C
C    THE DRPOLY PROGRAMS USE 6*(DEGREE+1)
C    DOUBLE-PRECISION LOCATIONS PLUS 1*(DEGREE+1) REAL
C    LOCATIONS IN THE DYNAMIC STORAGE STACK.
C
C COMMON AREA
      COMMON/CSTAK/ISTAK
C
      INTEGER DEGREE
      REAL RSTAK(10000)
      DOUBLE PRECISION DSTAK(5000)
      INTEGER ISTAK(10000)
      DOUBLE PRECISION COEFF(1),ZEROR(1),ZEROI(1)
      LOGICAL FAIL
C
      INTEGER IK, IP, IPT, IQK, IQP, ISVK, ITEMP, KP10, NLEFT, NN, NNN
      INTEGER ISTKGT, ISTKQU
      EXTERNAL ISTKGT, ISTKQU
C
      EQUIVALENCE (ISTAK(1),DSTAK(1))
      EQUIVALENCE (ISTAK(1),RSTAK(1))
C
C THE DEGREE IS LESS THAN 1
C
C/6S
C     IF (DEGREE .LT. 1) CALL SETERR(
C    1   34HDRPOLY - THE DEGREE IS LESS THAN 1,34,1,2)
C/7S
      IF (DEGREE .LT. 1) CALL SETERR(
     1   'DRPOLY - THE DEGREE IS LESS THAN 1',34,1,2)
C/
C
C ALGORITHM FAILS IF THE LEADING COEFFICIENT IS ZERO.
C
C/6S
C     IF (COEFF(1) .EQ. 0.0D0) CALL SETERR(
C    1   36HDRPOLY - LEADING COEFFICIENT IS ZERO,36,2,2)
C/7S
      IF (COEFF(1) .EQ. 0.0D0) CALL SETERR(
     1   'DRPOLY - LEADING COEFFICIENT IS ZERO',36,2,2)
C/
C
C SET UP THE STORAGE IN THE DYNAMIC STORAGE STACK -
C IF THERE IS ROOM
C
      NN = DEGREE + 1
C
      NLEFT = ISTKQU(4) - 6*NN
C/6S
C     IF (NLEFT .LE. 0) CALL SETERR(
C    1   47HDRPOLY - THE DYNAMIC STORAGE LEFT IS NOT ENOUGH,47,3,2)
C/7S
      IF (NLEFT .LE. 0) CALL SETERR(
     1   'DRPOLY - THE DYNAMIC STORAGE LEFT IS NOT ENOUGH',47,3,2)
C/
C
      NNN = 6*NN
      IP    = ISTKGT(NNN,4)
      IQP   = IP   + NN
      IK    = IQP  + NN
      IQK   = IK   + NN
      ISVK  = IQK  + NN
      ITEMP = ISVK + NN
C
      NLEFT = ISTKQU(3) - 1*NN
C/6S
C     IF (NLEFT .LE. 0) CALL SETERR(
C    1   47HDRPOLY - THE DYNAMIC STORAGE LEFT IS NOT ENOUGH,47,3,2)
C/7S
      IF (NLEFT .LE. 0) CALL SETERR(
     1   'DRPOLY - THE DYNAMIC STORAGE LEFT IS NOT ENOUGH',47,3,2)
C/
C
      IPT   = ISTKGT(NN,4)
C
C
C  CALL THE JENKINS-TRAUB ROUTINE (OLD RPOLY IS NOW D1RPLY) WITH
C  THE STACK LOCATIONS INCLUDED IN THE CALL.
C
      CALL D1RPLY(COEFF,DEGREE,ZEROR,ZEROI,FAIL,
     1  DSTAK(IP),DSTAK(IQP),DSTAK(IK),DSTAK(IQK),DSTAK(ISVK),
     2  DSTAK(ITEMP),DSTAK(IPT))
C
C  SEE IF THE THING FAILED
C
      IF (.NOT. FAIL) GO TO 10
C
C  OTHERWISE FIGURE HOW MANY ROOTS WERE FOUND
C
      KP10 = DEGREE+10
C
C/6S
C     CALL SETERR(
C    1   37HDRPOLY - ONLY K ZEROS HAVE BEEN FOUND,37,KP10,1)
C/7S
      CALL SETERR(
     1   'DRPOLY - ONLY K ZEROS HAVE BEEN FOUND',37,KP10,1)
C/
C
  10  CALL ISTKRL(2)
      RETURN
      END

      SUBROUTINE D1RPLY(OP, DEGREE, ZEROR, ZEROI,
     1              FAIL, P, QP, K, QK, SVK, TEMP, PT)
C FINDS THE ZEROS OF A REAL POLYNOMIAL
C OP  - DOUBLE PRECISION VECTOR OF COEFFICIENTS IN
C       ORDER OF DECREASING POWERS.
C DEGREE   - INTEGER DEGREE OF POLYNOMIAL.
C ZEROR, ZEROI - OUTPUT DOUBLE PRECISION VECTORS OF
C                ZEROS.
C FAIL  - TRUE IF D1RPLY HAS FOUND FEWER THAN
C         DEGREE NUMBER OF ZEROS.
C         IN THIS CASE, DEGREE IS RESET TO
C         THE NUMBER OF ZEROS FOUND.
C
C THE SUBROUTINE USES SINGLE PRECISION CALCULATIONS
C FOR SCALING, BOUNDS AND ERROR CALCULATIONS. ALL
C CALCULATIONS FOR THE ITERATIONS ARE DONE IN DOUBLE
C PRECISION.
C
      COMMON /P77PLY/ SR, SI, U,
     1 V, A, B, C, D, A1, A2, A3, A6, A7, E, F, G,
     2 H, SZR, SZI, LZR, LZI, ETA, ARE, MRE, N, NN
C
      INTEGER L, N, NN
      INTEGER DEGREE, CNT, NZ, I, J, JJ, NM1
      DOUBLE PRECISION ANG, ETA, ARE, MRE
      DOUBLE PRECISION PT(*), LO, MAX, MIN, XX, YY, COSR,
     1 SINR, XXX, X, SC, BND, XM, FF, DF, DX, INFIN,
     2 SMALNO, BASE
C
      DOUBLE PRECISION P(*), QP(*), K(*),
     1 QK(*), SVK(*), SR, SI, U, V, A, B, C, D,
     2 A1, A2, A3, A6, A7, E, F, G, H, SZR, SZI,
     3 LZR, LZI
C
      DOUBLE PRECISION OP(*), TEMP(*),
     1 ZEROR(*), ZEROI(*), T, AA, BB, CC, FACTOR
      LOGICAL FAIL, ZEROK
C
      DOUBLE PRECISION D1MACH
      INTEGER I1MACH
      EXTERNAL D1MACH, I1MACH
C
C THE FOLLOWING STATEMENTS SET MACHINE CONSTANTS USED
C IN VARIOUS PARTS OF THE PROGRAM. THE MEANING OF THE
C FOUR CONSTANTS ARE...
C ETA     THE MAXIMUM RELATIVE REPRESENTATION ERROR
C         WHICH CAN BE DESCRIBED AS THE SMALLEST
C         POSITIVE FLOATING POINT NUMBER SUCH THAT
C         1.D0+ETA IS GREATER THAN 1.
C INFINY  THE LARGEST FLOATING-POINT NUMBER.
C SMALNO  THE SMALLEST POSITIVE FLOATING-POINT NUMBER
C         IF THE EXPONENT RANGE DIFFERS IN SINGLE AND
C         DOUBLE PRECISION THEN SMALNO AND INFIN
C         SHOULD INDICATE THE SMALLER RANGE.
C
C PORT NOTE -
C     FOR THE ABOVE REASON, THE MACHINE-CONSTANT
C     ROUTINE R1MACH HAS BEEN USED FOR THESE TWO VALUES.
C
C BASE    THE BASE OF THE FLOATING-POINT NUMBER
C         SYSTEM USED.
C
      BASE = I1MACH(10)
      ETA = D1MACH(4)
      INFIN = D1MACH(2)
      SMALNO = D1MACH(1)
C ARE AND MRE REFER TO THE UNIT ERROR IN + AND *
C RESPECTIVELY. THEY ARE ASSUMED TO BE THE SAME AS
C ETA.
      ARE = ETA
      MRE = ETA
      LO = SMALNO/ETA
C INITIALIZATION OF CONSTANTS FOR SHIFT ROTATION
      XX = .5*SQRT(2.)
      YY = -XX
C
      ANG = 94./180.*(4.*ATAN(1.))
      COSR = COS(ANG)
      SINR = SIN(ANG)
      FAIL = .FALSE.
      N = DEGREE
      NN = N + 1
C
C REMOVE THE ZEROS AT THE ORIGIN IF ANY
   10 IF (OP(NN).NE.0.0D0) GO TO 20
      J = DEGREE - N + 1
      ZEROR(J) = 0.D0
      ZEROI(J) = 0.D0
      NN = NN - 1
      N = N - 1
      GO TO 10
C MAKE A COPY OF THE COEFFICIENTS
   20 DO 30 I=1,NN
        P(I) = OP(I)
   30 CONTINUE
C START THE ALGORITHM FOR ONE ZERO
   40 IF (N.GT.2) GO TO 60
      IF (N.LT.1) RETURN
C CALCULATE THE FINAL ZERO OR PAIR OF ZEROS
      IF (N.EQ.2) GO TO 50
      ZEROR(DEGREE) = -P(2)/P(1)
      ZEROI(DEGREE) = 0.0D0
      RETURN
   50 CALL D6RPLY(P(1), P(2), P(3), ZEROR(DEGREE-1),
     1 ZEROI(DEGREE-1), ZEROR(DEGREE), ZEROI(DEGREE))
      RETURN
C FIND LARGEST AND SMALLEST MODULI OF COEFFICIENTS.
   60 MAX = 0.
      MIN = INFIN
      DO 70 I=1,NN
        X = ABS(P(I))
        IF (X.GT.MAX) MAX = X
        IF (X.NE.0. .AND. X.LT.MIN) MIN = X
   70 CONTINUE
C SCALE IF THERE ARE LARGE OR VERY SMALL COEFFICIENTS
C COMPUTES A SCALE FACTOR TO MULTIPLY THE
C COEFFICIENTS OF THE POLYNOMIAL. THE SCALING IS DONE
C TO AVOID OVERFLOW AND TO AVOID UNDETECTED UNDERFLOW
C INTERFERING WITH THE CONVERGENCE CRITERION.
C THE FACTOR IS A POWER OF THE BASE
      SC = LO/MIN
      IF (SC.GT.1.0) GO TO 80
      IF (MAX.LT.10.) GO TO 110
      IF (SC.EQ.0.) SC = SMALNO
      GO TO 90
   80 IF (INFIN/SC.LT.MAX) GO TO 110
   90 L = NINT(DLOG(SC)/DLOG(BASE) + .5)
      FACTOR = (BASE*1.0D0)**L
      IF (FACTOR.EQ.1.D0) GO TO 110
      DO 100 I=1,NN
        P(I) = FACTOR*P(I)
  100 CONTINUE
C COMPUTE LOWER BOUND ON MODULI OF ZEROS.
  110 DO 120 I=1,NN
        PT(I) = ABS(P(I))
  120 CONTINUE
      PT(NN) = -PT(NN)
C COMPUTE UPPER ESTIMATE OF BOUND
      X = DEXP((DLOG(-PT(NN))-DLOG(PT(1)))/N)
      IF (PT(N).EQ.0.) GO TO 130
C IF NEWTON STEP AT THE ORIGIN IS BETTER, USE IT.
      XM = -PT(NN)/PT(N)
      IF (XM.LT.X) X = XM
C CHOP THE INTERVAL (0,X) UNTIL FF .LE. 0
  130 XM = X*.1
      FF = PT(1)
      DO 140 I=2,NN
        FF = FF*XM + PT(I)
  140 CONTINUE
      IF (FF.LE.0.) GO TO 150
      X = XM
      GO TO 130
  150 DX = X
C DO NEWTON ITERATION UNTIL X CONVERGES TO TWO
C DECIMAL PLACES
  160 IF (ABS(DX/X).LE..005) GO TO 180
      FF = PT(1)
      DF = FF
      DO 170 I=2,N
        FF = FF*X + PT(I)
        DF = DF*X + FF
  170 CONTINUE
      FF = FF*X + PT(NN)
      DX = FF/DF
      X = X - DX
      GO TO 160
  180 BND = X
C COMPUTE THE DERIVATIVE AS THE INTIAL K POLYNOMIAL
C AND DO 5 STEPS WITH NO SHIFT
      NM1 = N - 1
      DO 190 I=2,N
        K(I) = (NN-I)*P(I)/N
  190 CONTINUE
      K(1) = P(1)
      AA = P(NN)
      BB = P(N)
      ZEROK = K(N).EQ.0.D0
      DO 230 JJ=1,5
        CC = K(N)
        IF (ZEROK) GO TO 210
C USE SCALED FORM OF RECURRENCE IF VALUE OF K AT 0 IS
C NONZERO
        T = -AA/CC
        DO 200 I=1,NM1
          J = NN - I
          K(J) = T*K(J-1) + P(J)
  200   CONTINUE
        K(1) = P(1)
        ZEROK = DABS(K(N)).LE.DABS(BB)*ETA*10.
        GO TO 230
C USE UNSCALED FORM OF RECURRENCE
  210   DO 220 I=1,NM1
          J = NN - I
          K(J) = K(J-1)
  220   CONTINUE
        K(1) = 0.D0
        ZEROK = K(N).EQ.0.D0
  230 CONTINUE
C SAVE K FOR RESTARTS WITH NEW SHIFTS
      DO 240 I=1,N
        TEMP(I) = K(I)
  240 CONTINUE
C LOOP TO SELECT THE QUADRATIC  CORRESPONDING TO EACH
C NEW SHIFT
      DO 280 CNT=1,20
C QUADRATIC CORRESPONDS TO A DOUBLE SHIFT TO A
C NON-REAL POINT AND ITS COMPLEX CONJUGATE. THE POINT
C HAS MODULUS BND AND AMPLITUDE ROTATED BY 94 DEGREES
C FROM THE PREVIOUS SHIFT
        XXX = COSR*XX - SINR*YY
        YY = SINR*XX + COSR*YY
        XX = XXX
        SR = BND*XX
        SI = BND*YY
        U = -2.0D0*SR
        V = BND
C SECOND STAGE CALCULATION, FIXED QUADRATIC
        CALL D3RPLY(20*CNT, NZ, P, QP, K, QK, SVK)
        IF (NZ.EQ.0) GO TO 260
C THE SECOND STAGE JUMPS DIRECTLY TO ONE OF THE THIRD
C STAGE ITERATIONS AND RETURNS HERE IF SUCCESSFUL.
C DEFLATE THE POLYNOMIAL, STORE THE ZERO OR ZEROS AND
C RETURN TO THE MAIN ALGORITHM.
        J = DEGREE - N + 1
        ZEROR(J) = SZR
        ZEROI(J) = SZI
        NN = NN - NZ
        N = NN - 1
        DO 250 I=1,NN
          P(I) = QP(I)
  250   CONTINUE
        IF (NZ.EQ.1) GO TO 40
        ZEROR(J+1) = LZR
        ZEROI(J+1) = LZI
        GO TO 40
C IF THE ITERATION IS UNSUCCESSFUL ANOTHER QUADRATIC
C IS CHOSEN AFTER RESTORING K
  260   DO 270 I=1,N
          K(I) = TEMP(I)
  270   CONTINUE
  280 CONTINUE
C RETURN WITH FAILURE IF NO CONVERGENCE WITH 20
C SHIFTS
      FAIL = .TRUE.
      DEGREE = DEGREE - N
      RETURN
      END

      SUBROUTINE RESG0
      IMPLICIT NONE
      INTEGER I,J,T
      DOUBLE PRECISION U,V,W,X,Y,Z,PRED,PEV
c      INCLUDE 'model.txt'
c      INCLUDE 'series.txt'
c      INCLUDE 'setcon.txt'
c      INCLUDE 'veccom.txt'
c      INCLUDE 'repcom.txt'
c      INCLUDE 'repar3.txt'
c      INCLUDE 'resgn1.txt'
c      INCLUDE 'resgn2.txt'
C*****model.txt
      INTEGER PFI,ARP,VRI,CCV,LYAP,SCC,fct
      DOUBLE PRECISION SCALE,VR,CONST,PHI(20),PRDG
      COMMON/MODEL/SCALE,VR,CONST,PHI,PRDG,PFI,ARP,VRI,CCV,LYAP,SCC,fct
      integer for, fty
      common/model/for, fty
      integer ARI, tra
      common/model/ari,tra

C*****series.txt
      CHARACTER(LEN=40) NAME
      INTEGER LEN
      DOUBLE PRECISION TIM(5000),SER(5000),TDIF(5000)
c      COMMON/SERIES/NAME,LEN,TIM,SER,TDIF
      COMMON/SERIES/TIM,SER,TDIF,NAME,LEN
C*****setcon.txt
      LOGICAL CONV,FAIL
      INTEGER NP,ITCT,PPIND
      DOUBLE PRECISION CSO,CSZ,LAM,SSOLD,GMOLD,GMNEW,SIGSQ,OLDB(22)
c      COMMON/SETCON/CONV,FAIL,NP,ITCT,PPIND,CSO,CSZ,LAM,SSOLD,
c     *GMOLD,GMNEW,SIGSQ,OLDB
      COMMON/SETCON/CSO,CSZ,LAM,SSOLD,
     *GMOLD,GMNEW,SIGSQ,OLDB,CONV,FAIL,NP,ITCT,PPIND
C*****veccom.txt
      DOUBLE PRECISION B(22),NEWB(22),PERB(22),DELB(22),
     * ERR(5000),RES(5000),PRES(22,5000),sres(5000)
      COMMON/VECCOM/B,NEWB,PERB,DELB,ERR,RES,PRES,sres
C*****repcom.txt
      LOGICAL PRPI
      INTEGER REQSW,KFSW
      DOUBLE PRECISION VOB
c      COMMON/REPCOM/PRPI,REQSW,VOB,KFSW
      COMMON/REPCOM/VOB,REQSW,KFSW,PRPI 
C*****repar3.txt
      DOUBLE PRECISION ALPHA(21),ROOTR(20),ROOTI(20),XVECR(20),
     *XVECI(20)
      COMMON/REPAR3/ALPHA,ROOTR,ROOTI,XVECR,XVECI
C*****resgn1.txt
      DOUBLE PRECISION DR(20),DI(20),MR(20),MI(20),NR(20),NI(20),
     * UR(20),UI(20),WR(20),WI(20),WPR(20),WPI(20),CR(20,20),CI(20,20),
     * CPR(20,20),CPI(20,20),CSR(20,20),CSI(20,20)
      COMMON/RESGN1/DR,DI,MR,MI,NR,NI,UR,UI,WR,WI,WPR,WPI,CR,CI,CPR,CPI,
     * CSR,CSI
C*****resgn2.txt
      DOUBLE PRECISION WK(20),VT(5000),BI(2,20,20),R(2,20,20),
     *RI(2,20,20)
c       DOUBLE PRECISION WK(20),VT(5000),BI(2,20,20),R(20,20),RI(2,20,20)
      INTEGER ERRNO1
      COMMON/RESGN2/WK,VT,BI,R,RI,ERRNO1


      DO J=1,ARP
        DO I=1,ARP
          IF(I.EQ.1)THEN
            U=CSO
            V=CSZ
          ELSE IF(I.EQ.2)THEN
            U=ROOTR(J)
            V=ROOTI(J)
            IF(ARP.GT.2)THEN
              W=U
              X=V
            END IF
          ELSE IF(I.GT.2)THEN
            CALL MULC(U,V,W,X,Y,Z)
            U=Y
            V=Z
          END IF
          R(1,I,J)=U
          R(2,I,J)=V
        END DO
      END DO
      CALL CINVERT
      DO 102 I=1,ARP
      UR(I)=BI(1,I,ARP)
      UI(I)=BI(2,I,ARP)
c     in correspondence to the change of F04ADF to zgesv in LAPACK, the results are in BI, not RI
c      UR(I)=RI(1,I,ARP)
c      UI(I)=RI(2,I,ARP)
c     end of change by Z.W.
  102 CONTINUE
      DO 104 I=1,ARP
      WR(I)=CSZ
      WI(I)=CSZ
      DO 103 J=1,ARP
      CALL ADDC(ROOTR(I),ROOTI(I),ROOTR(J),-ROOTI(J),U,V)
      CALL MULC(-UR(I),-UI(I),UR(J),-UI(J),Y,Z)
      CALL DIVC(Y,Z,U,V,W,X)
      CSR(I,J)=W
      CSI(I,J)=X
      CR(I,J)=W
      CI(I,J)=X
  103 CONTINUE
  104 CONTINUE
      DO 150 T=1,LEN
      W=TDIF(T)
      DO 105 I=1,ARP
      CALL MULR(ROOTR(I),ROOTI(I),W,U,V)
      U=DEXP(U)
      DR(I)=U*DCOS(V)
      DI(I)=U*DSIN(V)
  105 CONTINUE
      DO 107 I=1,ARP
      CALL MULC(DR(I),DI(I),WR(I),WI(I),WPR(I),WPI(I))
      DO 106 J=1,ARP
      CALL SUBC(CR(I,J),CI(I,J),CSR(I,J),CSI(I,J),U,V)
      CALL MULC(U,V,DR(J),-DI(J),Y,Z)
      CALL MULC(DR(I),DI(I),Y,Z,U,V)
      CALL ADDC(U,V,CSR(I,J),CSI(I,J),CPR(I,J),CPI(I,J))
  106 CONTINUE
  107 CONTINUE
      DO 109 I=1,ARP
      Y=CSZ
      Z=CSZ
      DO 108 J=1,ARP
      CALL MULC(CPR(I,J),CPI(I,J),XVECR(J),-XVECI(J),W,X)
      Y=Y+W
      Z=Z+X
  108 CONTINUE
      MR(I)=Y
      MI(I)=Z
  109 CONTINUE
      PEV=CSZ
      PRED=CSZ
      DO 110 I=1,ARP
      U=XVECR(I)
      V=XVECI(I)
      CALL MULC(U,V,MR(I),MI(I),W,X)
      PEV=PEV+W
      CALL MULC(U,V,WPR(I),WPI(I),W,X)
      PRED=PRED+W
  110 CONTINUE
      PEV=PEV+VOB
      VT(T)=PEV
      U=SER(T)-CONST-PRED
      ERR(T)=U
      DO 111 I=1,ARP
      NR(I)=MR(I)/PEV
      NI(I)=MI(I)/PEV
      CALL MULR(NR(I),NI(I),U,W,X)
      CALL ADDC(WPR(I),WPI(I),W,X,WR(I),WI(I))
  111 CONTINUE
      DO I=1,ARP
        DO J=1,ARP
          CALL MULC(MR(I),MI(I),NR(J),-NI(J),W,X)
          CALL SUBC(CPR(I,J),CPI(I,J),W,X,CR(I,J),CI(I,J))
        END DO
      END DO
  150 CONTINUE
      RETURN
      END

      SUBROUTINE INTPR(MESAGE,LMESS,VALUE,N)
C
C  Print a message and integer values.
C
      INTEGER LMESS, NMESS
      INTEGER VALUE(*)
      CHARACTER(LEN=*) MESAGE
      INTEGER N
      INTRINSIC MIN, LEN
C
      NMESS = LEN(MESAGE)
      IF (LMESS .GE. 0) NMESS = MIN(LMESS, NMESS)
      IF (N .LT. 0) VALUE(1) = VALUE(1)
C     IF (NMESS .GT. 0) WRITE(*,'(A)') MESAGE(1:NMESS)
C     IF (N .GT. 0) WRITE(*,*) (VALUE(I), I=1,N)
      RETURN
      END

      subroutine setcom(PFI1,ARP1,NP1,VRI1,CCV1,LEN1,SCALE1,VR1,
     * SIGSQ1,ESSP1,ECOV1,B1,DELB1,ROOTR1,ROOTI1)
      IMPLICIT NONE 
      INTEGER I,J
c      INCLUDE 'model.txt'
c      INCLUDE 'conpar.txt'
c      INCLUDE 'setcon.txt'
c      INCLUDE 'series.txt'
c      INCLUDE 'veccom.txt'
c      INCLUDE 'redcom.txt'
c      INCLUDE 'seacom.txt'
c      INCLUDE 'repar3.txt'
c      INCLUDE 'seacom1.txt'
c      INCLUDE 'repar31.txt'
C*****model.txt
      INTEGER PFI,ARP,VRI,CCV,LYAP,SCC,fct
      DOUBLE PRECISION SCALE,VR,CONST,PHI(20),PRDG
      COMMON/MODEL/SCALE,VR,CONST,PHI,PRDG,PFI,ARP,VRI,CCV,LYAP,SCC,fct
      integer for, fty
      common/model/for, fty
      integer ARI, tra
      common/model/ari,tra      
C*****conpar.txt    
      INTEGER NIT,OPM,RGM,KUP,KSP,KST
      DOUBLE PRECISION REQ,CONCRIT,RPERT,IVLAM,FAC,STLAM,SMLAM,GTLAM
c      COMMON/CONPAR/NIT,OPM,RGM,REQ,CONCRIT,RPERT,IVLAM,FAC,STLAM,
c     * SMLAM,GTLAM,KUP,KSP,KST
      COMMON/CONPAR/REQ,CONCRIT,RPERT,IVLAM,FAC,STLAM,
     * SMLAM,GTLAM,NIT,OPM,RGM,KUP,KSP,KST
C*****setcon.txt
      LOGICAL CONV,FAIL
      INTEGER NP,ITCT,PPIND
      DOUBLE PRECISION CSO,CSZ,LAM,SSOLD,GMOLD,GMNEW,SIGSQ,OLDB(22)
c      COMMON/SETCON/CONV,FAIL,NP,ITCT,PPIND,CSO,CSZ,LAM,SSOLD,
c     *GMOLD,GMNEW,SIGSQ,OLDB
      COMMON/SETCON/CSO,CSZ,LAM,SSOLD,
     *GMOLD,GMNEW,SIGSQ,OLDB,CONV,FAIL,NP,ITCT,PPIND
C*****series.txt
      CHARACTER(LEN=40) NAME
      INTEGER LEN
      DOUBLE PRECISION TIM(5000),SER(5000),TDIF(5000)
c      COMMON/SERIES/NAME,LEN,TIM,SER,TDIF
      COMMON/SERIES/TIM,SER,TDIF,NAME,LEN
C*****veccom.txt
      DOUBLE PRECISION B(22),NEWB(22),PERB(22),DELB(22),
     * ERR(5000),RES(5000),PRES(22,5000),sres(5000)
      COMMON/VECCOM/B,NEWB,PERB,DELB,ERR,RES,PRES,sres
C*****redcom.txt
      DOUBLE PRECISION GRAD(22),SSP(22,22)
      COMMON/REDCOM/GRAD,SSP
C*****seacom.txt
      LOGICAL SOLV
      DOUBLE PRECISION ESSP(22,22),ECOV(22,22)
c      COMMON/SEACOM/SOLV,ESSP,ECOV
      COMMON/SEACOM/ESSP,ECOV,SOLV
C*****repar3.txt
      DOUBLE PRECISION ALPHA(21),ROOTR(20),ROOTI(20),XVECR(20),
     *XVECI(20)
      COMMON/REPAR3/ALPHA,ROOTR,ROOTI,XVECR,XVECI
C*****seacom1.txt
      DOUBLE PRECISION ESSP1(22,22),ECOV1(22,22)
C*****repar31.txt
      DOUBLE PRECISION ROOTR1(20),ROOTI1(20)
c      COMMON/REPAR3/ALPHA,ROOTR,ROOTI,XVECR,XVECI

C*****repcom.txt
      LOGICAL PRPI
      INTEGER REQSW,KFSW
      DOUBLE PRECISION VOB
c      COMMON/REPCOM/PRPI,REQSW,VOB,KFSW
      COMMON/REPCOM/VOB,REQSW,KFSW,PRPI 
C*****veccom1.txt
      DOUBLE PRECISION B1(22),DELB1(22)
C*****setcon1.txt
      INTEGER NP1
      DOUBLE PRECISION SIGSQ1
C*****model1.txt
      INTEGER LEN1, PFI1,ARP1,VRI1,CCV1
      DOUBLE PRECISION SCALE1,VR1

      PFI1=PFI
      ARP1=ARP
      NP1=NP
      VRI1=VRI
      CCV1=CCV
      LEN1=LEN
      SCALE1=SCALE
      VR1=VR
      SIGSQ1=SIGSQ
      DO I=1,NP
         DO J=1,NP
            ESSP1(I,J)=ESSP(I,J)
            ECOV1(I,J)=ECOV(I,J)
         END DO
      END DO
         DO 101 I=1,NP
            B1(I)=B(I)
 101     continue
         DO 102 I=1,NP
            DELB1(I)=DELB(I)
 102     continue
         DO 103 I=1,ARP
            ROOTR1(I)=ROOTR(I)
            ROOTI1(I)=ROOTI(I)
 103     continue
         RETURN
         END

      SUBROUTINE D8RPLY(NN, U, V, P, Q, A, B)
C DIVIDES P BY THE QUADRATIC  1,U,V  PLACING THE
C QUOTIENT IN Q AND THE REMAINDER IN A,B
      INTEGER NN
      DOUBLE PRECISION P(NN), Q(NN), U, V, A, B, C
      INTEGER I
      B = P(1)
      Q(1) = B
      A = P(2) - U*B
      Q(2) = A
      DO 10 I=3,NN
        C = P(I) - U*A - V*B
        Q(I) = C
        B = A
        A = C
   10 CONTINUE
      RETURN
      END

      SUBROUTINE REVG1
      IMPLICIT NONE
      INTEGER I,J,T,TR,IA,IFAIL
      DOUBLE PRECISION CSZ, CSO,U,V,W,X,TWM1(20,20),TWM2(20,20)
     *,AQ(20,20),AQI(20,20),APH(20,20),AP(20,20),AJ(20,20),SIG(20,20)
c      INCLUDE 'kfasav.txt'
c      INCLUDE 'series.txt'
c      INCLUDE 'model.txt'
c      INCLUDE 'repar3.txt'
C*****kfasav.txt
      DOUBLE PRECISION PRE(5000),PRV(5000),FSER(5000),FVAR(5000)
     1,SSER(5000)
     1,SVAR(5000)
      DOUBLE PRECISION FSES(5000,20),FSCV(5000,20,20),SSES(5000,20)
     1,SSCV(5000,20,20),PSES(5000,20),PSCV(5000,20,20),TRAN(5000,20,20)
c      COMMON/KFASAV/PRE,PRV,FSER,FVAR,SSER,SVAR,FSES,FSCV,SSES,SSCV
c     1,PSES,PSCV,TRAN
      COMMON/KFASAV/FSES,FSCV,SSES,SSCV
     1,PSES,PSCV,TRAN,PRE,PRV,FSER,FVAR,SSER,SVAR
C*****series.txt
      CHARACTER(LEN=40) NAME
      INTEGER LEN
      DOUBLE PRECISION TIM(5000),SER(5000),TDIF(5000)
c      COMMON/SERIES/NAME,LEN,TIM,SER,TDIF
      COMMON/SERIES/TIM,SER,TDIF,NAME,LEN
C*****model.txt
      INTEGER PFI,ARP,VRI,CCV,LYAP,SCC,fct
      DOUBLE PRECISION SCALE,VR,CONST,PHI(20),PRDG
      COMMON/MODEL/SCALE,VR,CONST,PHI,PRDG,PFI,ARP,VRI,CCV,LYAP,SCC,fct
      integer for, fty
      common/model/for, fty
      integer ARI, tra
      common/model/ari,tra      
C*****repar3.txt
      DOUBLE PRECISION ALPHA(21),ROOTR(20),ROOTI(20),XVECR(20),
     *XVECI(20)
      COMMON/REPAR3/ALPHA,ROOTR,ROOTI,XVECR,XVECI
      DOUBLE PRECISION WK(20),VT(5000),BI(2,20,20),R(2,20,20),
     *RI(2,20,20)
      INTEGER ERRNO1
      COMMON/RESGN2/WK,VT,BI,R,RI,ERRNO1

      CSO=1.D0
      CSZ=0.D0
      IA=20
      DO 100 T=1,LEN
c      do 100 t=1,3
        TR=1+LEN-T
        IF (T.GT.1) THEN
          DO I=1,ARP
            DO J=1,ARP
              V=PSCV(TR+1,I,J)
              AQ(I,J)=V
              AQI(I,J)=V
              APH(I,J)=TRAN(TR+1,I,J)
              SIG(I,J)=SSCV(TR+1,I,J)
              AP(I,J)=FSCV(TR,I,J)
            END DO
          END DO
          IFAIL=0
C          PRINT *,'BEFORE CALL OF F01ADF'
C          IF (TR.EQ.101) CALL MPRINT(ARP,AQI,IA,'AQIBEFORE')
c         added ZW
      CALL DPOTRF ('UPPER TRIANGLE OF A STORED', ARP, AQI, IA, IFAIL)

      IF (IFAIL .NE. 0) THEN
        call  intpr('Error factoring A, INFO = ', 27, IFAIL, 1)
C        PRINT 1030, IFAIL
        ERRNO1=51
        RETURN
C        STOP 1

      END IF

C

C     Use the factored form of A to compute the inverse of A and

C     print the inverse thus computed.

C

      CALL DPOTRI ('UPPER TRIANGLE OF A STORED', ARP, AQI, IA, IFAIL)

      IF (IFAIL .NE. 0) THEN
        call intpr('Error computing inverse of A, INFO = ', 27,IFAIL, 1)
C        PRINT 1040, ABS(IFAIL)
        ERRNO1=52
        RETURN

C        STOP 2

      END IF
c          print *,'AQI'
c          do 30 i=1,arp
c             do 30 j=1,arp
c                print *,'aqi',aqi(i,j)
c 30             continue
c      print *,'AQI',AQI
c     added ZW
c          CALL F01ADF(ARP,AQI,IA,IFAIL)
C          PRINT *,'AFTER CALL OF F01ADF'
C          IF (IFAIL.EQ.1) PRINT *,'SMOOTHING ERROR: NOT POS.DEF.'
c          DO 20 I=1,ARP
c          DO 20 J=I,ARP
c          AQI(I,J)=AQI(J+1,I)
c   20     AQI(J,I)=AQI(I,J)

          DO I=1,ARP-1
            DO J=I,ARP-1
              AQI(J+1,I)=AQI(I,J+1)
            END DO
          END DO
c             print *,'AQI'
c          do 30 i=1,arp
c             do 30 j=1,arp
c                print *,'aqi',aqi(i,j)
c 30             continue
C          IF (TR.EQ.101) CALL MPRINT(ARP,AQI,IA,'AQIAFTER')
          CALL MTPROD(ARP,TWM1,APH,AQI,IA)
          CALL MPROD(ARP,AJ,AP,TWM1,IA) 
          CALL MDIFF(ARP,TWM1,SIG,AQ,IA)
          CALL MSPROD(ARP,TWM2,AJ,TWM1,IA)
        ENDIF
        W=CSZ
        X=CSZ
        DO I=1,ARP
          U=FSES(TR,I)
          DO J=1,ARP
            V=FSCV(TR,I,J)
            IF (T.GT.1) THEN
              V=V+TWM2(I,J)
              U=U+AJ(I,J)*(SSES(TR+1,J)-PSES(TR+1,J))
            ENDIF
            X=X+XVECR(I)*XVECR(J)*V
            SSCV(TR,I,J)=V
          END DO
          W=W+XVECR(I)*U
          SSES(TR,I)=U
        END DO
        SSER(TR)=W+CONST
        SVAR(TR)=X
C      PRINT *,'END OF REVG1 LOOP WITH TR=',TR
  100 CONTINUE   
      END

      SUBROUTINE REJUCE
      IMPLICIT NONE
      INTEGER I,J,K 
      DOUBLE PRECISION U
c      INCLUDE 'series.txt'
c      INCLUDE 'setcon.txt'
c      INCLUDE 'veccom.txt'
c      INCLUDE 'redcom.txt'

C*****setcon.txt
      LOGICAL CONV,FAIL
      INTEGER NP,ITCT,PPIND
      DOUBLE PRECISION CSO,CSZ,LAM,SSOLD,GMOLD,GMNEW,SIGSQ,OLDB(22)
c      COMMON/SETCON/CONV,FAIL,NP,ITCT,PPIND,CSO,CSZ,LAM,SSOLD,
c     *GMOLD,GMNEW,SIGSQ,OLDB
      COMMON/SETCON/CSO,CSZ,LAM,SSOLD,
     *GMOLD,GMNEW,SIGSQ,OLDB,CONV,FAIL,NP,ITCT,PPIND
C*****veccom.txt
      DOUBLE PRECISION B(22),NEWB(22),PERB(22),DELB(22),
     * ERR(5000),RES(5000),PRES(22,5000),sres(5000)
      COMMON/VECCOM/B,NEWB,PERB,DELB,ERR,RES,PRES,sres
C*****series.txt
      CHARACTER(LEN=40) NAME
      INTEGER LEN
      DOUBLE PRECISION TIM(5000),SER(5000),TDIF(5000)
c      COMMON/SERIES/NAME,LEN,TIM,SER,TDIF
      COMMON/SERIES/TIM,SER,TDIF,NAME,LEN
C*****redcom.txt
      DOUBLE PRECISION GRAD(22),SSP(22,22)
      COMMON/REDCOM/GRAD,SSP

      DO 101 I=1,NP
      U=CSZ
      DO 100 J=1,LEN
      U=U+RES(J)*PRES(I,J)
  100 CONTINUE
      GRAD(I)=U
  101 CONTINUE
      DO I=1,NP
      DO J=1,NP
      U=CSZ
      DO 103 K=1,LEN
      U=U+PRES(I,K)*PRES(J,K)
  103 CONTINUE
      SSP(I,J)=U
      END DO
      END DO
      RETURN
      END

      subroutine setfor (PRE1,PRV1,TIM1, PRE2, PRV2)
      IMPLICIT NONE
      INTEGER I
c      INCLUDE 'model.txt'
c      INCLUDE 'series.txt'
c      INCLUDE 'kfasav.txt'
C*****model.txt
      INTEGER PFI,ARP,VRI,CCV,LYAP,SCC,fct
      DOUBLE PRECISION SCALE,VR,CONST,PHI(20),PRDG
      COMMON/MODEL/SCALE,VR,CONST,PHI,PRDG,PFI,ARP,VRI,CCV,LYAP,SCC,fct
      integer for, fty
      common/model/for, fty
      integer ARI, tra
      common/model/ari,tra      
C*****series.txt
      CHARACTER(LEN=40) NAME
      INTEGER LEN
      DOUBLE PRECISION TIM(5000),SER(5000),TDIF(5000)
c      COMMON/SERIES/NAME,LEN,TIM,SER,TDIF
      COMMON/SERIES/TIM,SER,TDIF,NAME,LEN
C*****kfasav.txt
      DOUBLE PRECISION PRE(5000),PRV(5000),FSER(5000),FVAR(5000)
     1,SSER(5000),SVAR(5000), PRE2(5000), PRV2(5000)
      DOUBLE PRECISION FSES(5000,20),FSCV(5000,20,20),SSES(5000,20)
     1,SSCV(5000,20,20),PSES(5000,20),PSCV(5000,20,20),TRAN(5000,20,20)
c      COMMON/KFASAV/PRE,PRV,FSER,FVAR,SSER,SVAR,FSES,FSCV,SSES,SSCV
c     1,PSES,PSCV,TRAN
      COMMON/KFASAV/FSES,FSCV,SSES,SSCV
     1,PSES,PSCV,TRAN,PRE,PRV,FSER,FVAR,SSER,SVAR

C*****kfasav1.txt
      DOUBLE PRECISION PRE1(5000),PRV1(5000),TIM1(5000)
C*****series1.txt

      DO 90 I=1,LEN+FOR
         TIM1(I)=TIM(I)
         PRE2(I)=PRE(I)
         PRV2(I)=PRV(I)
 90   CONTINUE
      
      IF(fct.EQ.0)then
      DO 102 I=1,for
         PRE1(I)=PRE(LEN+I)
c changed next line 6/7/06
         PRV1(I)=PRV(LEN+I)
         TIM1(I)=TIM(LEN+I)
 102     CONTINUE
      ELSE
        if(fty.EQ.1)then
c          LEN=LEN-FOR, changed 6/7/06
          do 103 I=1,for
          PRE1(I)=PRE(LEN+I)
c changed the next line 6/7/06
          PRV1(I)=PRV(LEN+I)
 103      Continue
        else 
          if(fty.EQ.2)then
           DO 104 I=1,for
C changed 6/6/06
C           PRE1(I)=PRE(LEN-for+I) 
C           PRV1(I)=PRV(LEN-for+I)
           PRE1(I)=PRE(LEN+I)
           PRV1(I)=PRV(LEN+I)
 104       CONTINUE
          else
           do 105 I=1,for
           pre1(I)=PRE(LEN+I)
           prv1(I)=PRV(LEN+I)
 105       continue
          endif
        endif
      ENDIF
      RETURN
      END

*UPTODATE MEPADTEXT
      SUBROUTINE MEPAD(N,NDIAG,DELTA,A,NA,E,NE,MDIG,IDIG,IWRK,WRKA1,
     +                 WRKA2,WRKV1,WRKV2,IERR)
C
C        MARK 2.2 RELEASE.  NAG COPYRIGHT 1987.
C
C PURPOSE
C
C        To compute Exp( A * delta ) , where A is a real N by N
C        matrix and delta is a scalar value. The routine also returns
C        the minimal number of accurate digits in the one norm of
C        Exp( A * delta ) at the 95 per cent confidence level.
C
C METHOD
C
C        The exponential of the matrix is evaluated from a
C        diagonal Pade approximant. This routine is a modification
C        of the subroutine PADE, written and discussed by R.C.Ward
C        (see reference).
C
C REFERENCE
C
C       WARD, R.C.
C       "Numerical computation of the matrix exponential
C       with accuracy estimate",
C       SIAM J. NUMER. ANAL. ,VOL. 14,1977, pp. 600-610
C
C AUXILIARY ROUTINES
C
C       BALANC (EISPACK)
C       SGEFA,SGESL (LINPACK)
C       SDATA,BALINV  (SLICE)
C
C ARGUMENTS IN
C
C        N       INTEGER
C                -The order of the matrix  A.
C                 N .GE. 1
C
C        NDIAG   INTEGER
C                -The specified order of the diagonal Pade approximant.
C                 In the absence of further information NDIAG should
C                 be set to 9
C
C        DELTA   REAL
C                -The scalar value delta of the problem.
C
C        A       REAL(NA,N)
C                -The leading N by N part of this array must contain
C                 the matrix A of the problem.
C
C        NA      INTEGER
C                -The first dimension of array A as declared in the
C                 calling program
C                 NA .GE. N
C
C        NE      INTEGER
C                -The first dimension of array E as declared in the
C                 calling program
C                 NE .GE. N
C
C ARGUMENTS OUT
C
C        E       REAL(NE,N)
C                -The leading N by N part of this array will contain
C                 the solution matrix  exp( A*delta )
C
C        MDIG    INTEGER
C                -The minimal number of accurate digits
C                in the norm of  E .
C
C        IDIG    INTEGER
C                -The number of accurate digits in the norm
C                of  E  at  95% confidence level.
C
C        IERR    INTEGER
C                -Error indicator
C                IERR = 0        Successful return
C
C                IERR = 1        The norm of A is too large to obtain
C                                an accurate result
C
C                IERR = 2        If MDIG = 0 and IDIG .GT.  0, warning
C                                for possible inaccuracy
C
C                IERR = 3        If MDIG = 0 and IDIG = 0, severe
C                                warning for inaccuracy
C
C                IERR = 4        Either NA .LT. N or NE .LT. N or
C                                       N .LT. 1
C
C WORKING SPACE
C
C        IWRK      INTEGER(N)
C
C        WRKA1     REAL(N,N)
C
C        WRKA2     REAL(N,NDIAG)
C
C        WRKV1     REAL(N)
C
C        WRKV2     REAL(NDIAG)
C
C ORIGINATOR
C
C                Control Systems Research Group, School of EECS,
C                Kingston Polytechnic, Kingston-upon-Thames, England.
C
C VERSION        MEPAD 2.2 ( 20TH JANUARY 1987 )
C
C LIBRARY INDEX
C                SLICE 2.2 ( 20TH JANUARY 1987 )
C                Section  2.  Mathematical Routines.
C                Subsection  2.1  Matrix Manipulation.
C
C COMMENTS
C
C                none
C
C*******************************************************************
C
C     .. Scalar Arguments ..
      DOUBLE PRECISION DELTA
      INTEGER IDIG,IERR,MDIG,N,NA,NDIAG,NE
C     .. Array Arguments ..
      DOUBLE PRECISION A(NA,N),E(NE,N),WRKA1(N,N),WRKA2(N,NDIAG),
     +                 WRKV1(N),WRKV2(NDIAG)
      INTEGER IWRK(N)
C     .. Scalars in Common ..
      DOUBLE PRECISION BASE,D1,D2,EPS
C     .. Local Scalars ..
      DOUBLE PRECISION ANORM,AVGEV,BD,BIG,EABS,EAVGEV,EMNORM,ENORM,
     +                 FACTOR,FN,GN,P,RERL,RERR,S,SD2,SIZE,SIZEJ,SMALL,
     +                 SS,SUM2D,TEMP,TR,U,VAR,VAREPS,XN
      INTEGER I,IGH,IM1,J,JOB,K,KEO,KP1,L,LOW,M,MPOWER,NDAGM1,NDAGM2,
     +        NDEC,NDECM1
C     .. External Subroutines ..
C      EXTERNAL BALANC,BALINV,SDATA,SGEFA,SGESL
      EXTERNAL BALANC,BALINV,SDATA,DGETRF,DGETRS
C     .. Intrinsic Functions ..
      INTRINSIC ABS,EXP,INT,LOG,LOG10,MOD,DBLE,SQRT
C     .. Common blocks ..
      COMMON /MDEPS/BASE,D1,D2,EPS
C     .. Executable Statements ..
C
C        COMMON block MDEPS  is shared with  SDATA , which provides
C        values for the machine dependent parameters  EPS, which
C        specifies the relative precision of floating point arithmetic,
C        and  BASE  which specifies the number base used for machine
C        representation.
C
      CALL SDATA
      IERR = 0
      IF (N.GT.NA .OR. N.GT.NE) IERR = 4
      IF (N.LT.1) IERR = 4
      IF (IERR.NE.0) RETURN
      JOB = 0
      NDEC = INT(LOG10(1.0D0/EPS)+1.0D0)
      NDECM1 = NDEC - 1
      IF (N.EQ.1) GO TO 800
      WRKV2(1) = 0.5D0
      DO 20 I = 2,NDIAG
          IM1 = I - 1
          WRKV2(I) = WRKV2(IM1)*DBLE(NDIAG-IM1)/DBLE(I* (2*NDIAG-IM1))
   20 CONTINUE
      VAREPS = ((BASE**2-1.0D0)/ (24.0D0*LOG(BASE)))*EPS**2
      XN = DBLE(N)
      TR = 0.0D0
      DO 60 I = 1,N
          DO 40 J = 1,N
              A(I,J) = A(I,J)*DELTA
   40     CONTINUE
          TR = TR + A(I,I)
   60 CONTINUE
      AVGEV = TR/XN
      DO 80 I = 1,N
          A(I,I) = A(I,I) - AVGEV
   80 CONTINUE
      CALL BALANC(NA,N,A,LOW,IGH,WRKV1)
      ANORM = 0.0D0
      DO 120 J = 1,N
          SS = 0.0D0
          DO 100 I = 1,N
              SS = SS + ABS(A(I,J))
  100     CONTINUE
          IF (SS.GT.ANORM) ANORM = SS
  120 CONTINUE
      M = 0
      IF (ANORM.LE.1.0D0) GO TO 220
      MPOWER = INT(LOG(1.0D0/EPS)/LOG(2.0D0))
      FACTOR = 2.0D0
      DO 140 I = 1,MPOWER
          M = I
          IF (ANORM.LE.FACTOR) GO TO 160
          FACTOR = FACTOR*2.0D0
  140 CONTINUE
      IERR = 1
      GO TO 820
  160 DO 200 I = 1,N
          DO 180 J = 1,N
              A(I,J) = A(I,J)/FACTOR
  180     CONTINUE
  200 CONTINUE
  220 NDAGM1 = NDIAG - 1
      NDAGM2 = NDAGM1 - 1
      DO 420 J = 1,N
          DO 260 I = 1,N
              S = 0.0D0
              DO 240 L = 1,N
                  S = S + A(I,L)*A(L,J)
  240         CONTINUE
              WRKA2(I,1) = S
  260     CONTINUE
          DO 320 K = 1,NDAGM2
              KP1 = K + 1
              DO 300 I = 1,N
                  S = 0.0D0
                  DO 280 L = 1,N
                      S = S + A(I,L)*WRKA2(L,K)
  280             CONTINUE
                  WRKA2(I,KP1) = S
  300         CONTINUE
  320     CONTINUE
          DO 400 I = 1,N
              S = 0.0D0
              U = 0.0D0
              DO 360 L = 1,NDAGM1
                  K = NDIAG - L
                  KP1 = K + 1
                  P = WRKV2(KP1)*WRKA2(I,K)
                  S = S + P
                  KEO = MOD(KP1,2)
                  IF (KEO.EQ.0) GO TO 340
                  U = U - P
                  GO TO 360
  340             U = U + P
  360         CONTINUE
              P = WRKV2(1)*A(I,J)
              S = S + P
              U = U - P
              IF (I.NE.J) GO TO 380
              S = S + 1.0D0
              U = U + 1.0D0
  380         E(I,J) = S
              WRKA1(I,J) = U
  400     CONTINUE
  420 CONTINUE
C      CALL SGEFA(WRKA1,N,N,IWRK,IERR)
      CALL DGETRF(N, N, WRKA1,N, IWRK,IERR)
      DO 440 J = 1,N
C          CALL SGESL(WRKA1,N,N,IWRK,E(1,J),JOB)
      CALL DGETRS('N',N, 1, WRKA1, N, IWRK, E(1,J), N, JOB)
C      CALL SGESL(T,LDIM,NSYS,IPVT,P,JOB)

  440 CONTINUE
      ENORM = 0.0D0
      DO 480 J = 1,N
          SS = 0.0D0
          DO 460 I = 1,N
              SS = SS + ABS(E(I,J))
  460     CONTINUE
          IF (SS.GT.ENORM) ENORM = SS
  480 CONTINUE
      EABS = (19.0D0*XN+47.0D0)*EPS*ENORM
      VAR = (12.0D0*XN)*VAREPS
      IF (M.EQ.0) GO TO 660
      VAR = XN*VAREPS
      FN = (4.0D0*XN)/ ((XN+2.0D0)* (XN+1.0D0))
      GN = (2.0D0* (XN**2)+10.0D0*XN-4.0D0)/
     +     (((XN+2.0D0)**2)* ((XN+1.0D0)**2))
      DO 600 K = 1,M
          DO 540 I = 1,N
              DO 520 J = 1,N
                  S = 0.0D0
                  DO 5000 L = 1,N
                      S = S + E(I,L)*E(L,J)
 5000             CONTINUE
                  WRKA1(I,J) = S
  520         CONTINUE
  540     CONTINUE
          TEMP = ENORM**2
          EABS = 2.0D0*EABS*ENORM + XN*EPS*TEMP + EABS**2
          VAR = (FN*VAR+GN*TEMP*VAREPS)*TEMP
          ENORM = 0.0D0
          DO 580 J = 1,N
              SS = 0.0D0
              DO 560 I = 1,N
                  E(I,J) = WRKA1(I,J)
                  SS = SS + ABS(E(I,J))
  560         CONTINUE
              IF (SS.GT.ENORM) ENORM = SS
  580     CONTINUE
  600 CONTINUE
      DO 640 I = 1,N
          DO 620 J = 1,N
              A(I,J) = A(I,J)*FACTOR
  620     CONTINUE
  640 CONTINUE
  660 CALL BALINV(NA,N,LOW,IGH,WRKV1,A)
      DO 680 I = 1,N
          A(I,I) = A(I,I) + AVGEV
  680 CONTINUE
      BIG = 1.0D0
      SMALL = 1.0D0
      SUM2D = 0.0D0
      IF (LOW.EQ.IGH) GO TO 740
      SUM2D = -1.0D0
      DO 720 I = LOW,IGH
          U = WRKV1(I)
          SUM2D = SUM2D + U**2
          IF (BIG.GE.U) GO TO 700
          BIG = U
  700     IF (SMALL.LE.U) GO TO 720
          SMALL = U
  720 CONTINUE
  740 SUM2D = SUM2D + XN + DBLE(LOW-IGH)
      SD2 = 2.0D0*SQRT(SUM2D*VAR+ (8.0D0*DBLE(IGH-
     +      LOW)* (ENORM**2)*SUM2D*VAREPS)/ (XN**2))
      CALL BALINV(NE,N,LOW,IGH,WRKV1,E)
      EAVGEV = EXP(AVGEV)
      EMNORM = 0.0D0
      SIZE = 0.0D0
      DO 780 J = 1,N
          SS = 0.0D0
          DO 760 I = 1,N
              SS = SS + ABS(E(I,J))
              E(I,J) = E(I,J)*EAVGEV
  760     CONTINUE
          IF (SS.GT.EMNORM) EMNORM = SS
          IF (LOW.EQ.IGH) GO TO 780
          BD = WRKV1(J)
          IF (J.LT.LOW .OR. IGH.LT.J) BD = 1.0D0
          SIZEJ = SS + SD2/BD
          IF (SIZEJ.GT.SIZE) SIZE = SIZEJ
  780 CONTINUE
      RERR = (BIG*EABS)/ (SMALL*EMNORM*EPS)
      IF (LOW.EQ.IGH) SIZE = EMNORM + SD2
      RERL = (SIZE-EMNORM)/ (EMNORM*EPS)
      MDIG = NDEC - INT(LOG10(RERR)+0.5D0)
      IDIG = NDEC - INT(LOG10(RERL)+0.5D0)
      IF (IDIG.GT.NDECM1) IDIG = NDECM1
      IF (MDIG.GT.0) GO TO 820
      MDIG = 0
      IERR = 2
      IF (IDIG.GT.0) GO TO 820
      IDIG = 0
      IERR = 3
      GO TO 820
  800 WRKA1(1,1) = A(1,1)*DELTA
      E(1,1) = EXP(WRKA1(1,1))
      MDIG = NDECM1
      IDIG = NDECM1
  820 CONTINUE
      RETURN
      END

      SUBROUTINE D4RPLY(TYPE, UU, VV, P, K)
C COMPUTE NEW ESTIMATES OF THE QUADRATIC COEFFICIENTS
C USING THE SCALARS COMPUTED IN D2RPLY.
      COMMON /P77PLY/ SR, SI, U,
     1 V, A, B, C, D, A1, A2, A3, A6, A7, E, F, G,
     2 H, SZR, SZI, LZR, LZI, ETA, ARE, MRE, N, NN
C
      INTEGER N, NN
      INTEGER TYPE
      DOUBLE PRECISION ETA, ARE, MRE
C
      DOUBLE PRECISION P(*), K(*),
     1  SR, SI, U, V, A, B, C, D,
     2 A1, A2, A3, A6, A7, E, F, G, H, SZR, SZI,
     3 LZR, LZI
      DOUBLE PRECISION A4, A5, B1, B2, C1, C2, C3,
     1 C4, TEMP, UU, VV
C USE FORMULAS APPROPRIATE TO SETTING OF TYPE.
      IF (TYPE.EQ.3) GO TO 30
      IF (TYPE.EQ.2) GO TO 10
      A4 = A + U*B + H*F
      A5 = C + (U+V*F)*D
      GO TO 20
   10 A4 = (A+G)*F + H
      A5 = (F+U)*C + V*D
C EVALUATE NEW QUADRATIC COEFFICIENTS.
   20 B1 = -K(N)/P(NN)
      B2 = -(K(N-1)+B1*P(N))/P(NN)
      C1 = V*B2*A1
      C2 = B1*A7
      C3 = B1*B1*A3
      C4 = C1 - C2 - C3
      TEMP = A5 + B1*A4 - C4
      IF (TEMP.EQ.0.D0) GO TO 30
      UU = U - (U*(C3+C2)+V*(B1*A1+B2*A7))/TEMP
      VV = V*(1.+C4/TEMP)
      RETURN
C IF TYPE=3 THE QUADRATIC IS ZEROED
   30 UU = 0.D0
      VV = 0.D0
      RETURN
      END

      SUBROUTINE D3RPLY(L2, NZ, P, QP, K, QK, SVK)
C COMPUTES UP TO  L2  FIXED SHIFT K-POLYNOMIALS,
C TESTING FOR CONVERGENCE IN THE LINEAR OR QUADRATIC
C CASE. INITIATES ONE OF THE VARIABLE SHIFT
C ITERATIONS AND RETURNS WITH THE NUMBER OF ZEROS
C FOUND.
C L2 - LIMIT OF FIXED SHIFT STEPS
C NZ - NUMBER OF ZEROS FOUND
      COMMON /P77PLY/ SR, SI, U,
     1 V, A, B, C, D, A1, A2, A3, A6, A7, E, F, G,
     2 H, SZR, SZI, LZR, LZI, ETA, ARE, MRE, N, NN
C
      INTEGER N, NN
      INTEGER L2, NZ, TYPE, I, J, IFLAG
C
      DOUBLE PRECISION ETA, ARE, MRE
      DOUBLE PRECISION BETAS, BETAV, OSS, OVV, SS, VV, TS, TV,
     1 OTS, OTV, TVV, TSS
      DOUBLE PRECISION P(*), QP(*), K(*),
     1 QK(*), SVK(*), SR, SI, U, V, A, B, C, D,
     2 A1, A2, A3, A6, A7, E, F, G, H, SZR, SZI,
     3 LZR, LZI
      DOUBLE PRECISION SVU, SVV, UI, VI, S
      LOGICAL VPASS, SPASS, VTRY, STRY
      NZ = 0
      BETAV = .25
      BETAS = .25
      OSS = SR
      OVV = V
C EVALUATE POLYNOMIAL BY SYNTHETIC DIVISION
      CALL D8RPLY(NN, U, V, P, QP, A, B)
      CALL D2RPLY(TYPE, K, QK)
      DO 80 J=1,L2
C CALCULATE NEXT K POLYNOMIAL AND ESTIMATE V
        CALL D5RPLY(TYPE, QP, K, QK)
        CALL D2RPLY(TYPE, K, QK)
        CALL D4RPLY(TYPE, UI, VI, P, K)
        VV = VI
C ESTIMATE S
        SS = 0.
        IF (K(N).NE.0.D0) SS = -P(NN)/K(N)
        TV = 1.
        TS = 1.
        IF (J.EQ.1 .OR. TYPE.EQ.3) GO TO 70
C COMPUTE RELATIVE MEASURES OF CONVERGENCE OF S AND V
C SEQUENCES
        IF (VV.NE.0.) TV = ABS((VV-OVV)/VV)
        IF (SS.NE.0.) TS = ABS((SS-OSS)/SS)
C IF DECREASING, MULTIPLY TWO MOST RECENT
C CONVERGENCE MEASURES
        TVV = 1.
        IF (TV.LT.OTV) TVV = TV*OTV
        TSS = 1.
        IF (TS.LT.OTS) TSS = TS*OTS
C COMPARE WITH CONVERGENCE CRITERIA
        VPASS = TVV.LT.BETAV
        SPASS = TSS.LT.BETAS
        IF (.NOT.(SPASS .OR. VPASS)) GO TO 70
C AT LEAST ONE SEQUENCE HAS PASSED THE CONVERGENCE
C TEST. STORE VARIABLES BEFORE ITERATING
        SVU = U
        SVV = V
        DO 10 I=1,N
          SVK(I) = K(I)
   10   CONTINUE
        S = SS
C CHOOSE ITERATION ACCORDING TO THE FASTEST
C CONVERGING SEQUENCE
        VTRY = .FALSE.
        STRY = .FALSE.
        IF (SPASS .AND. ((.NOT.VPASS) .OR.
     1   TSS.LT.TVV)) GO TO 40
   20   CALL D7RPLY(UI, VI, NZ, P, QP, K, QK, SVK)
        IF (NZ.GT.0) RETURN
C QUADRATIC ITERATION HAS FAILED. FLAG THAT IT HAS
C BEEN TRIED AND DECREASE THE CONVERGENCE CRITERION.
        VTRY = .TRUE.
        BETAV = BETAV*.25
C TRY LINEAR ITERATION IF IT HAS NOT BEEN TRIED AND
C THE S SEQUENCE IS CONVERGING
        IF (STRY .OR. (.NOT.SPASS)) GO TO 50
        DO 30 I=1,N
          K(I) = SVK(I)
   30   CONTINUE
   40   CALL D9RPLY(S, NZ, IFLAG, P, QP, K, QK)
        IF (NZ.GT.0) RETURN
C LINEAR ITERATION HAS FAILED. FLAG THAT IT HAS BEEN
C TRIED AND DECREASE THE CONVERGENCE CRITERION
        STRY = .TRUE.
        BETAS = BETAS*.25
        IF (IFLAG.EQ.0) GO TO 50
C IF LINEAR ITERATION SIGNALS AN ALMOST DOUBLE REAL
C ZERO ATTEMPT QUADRATIC INTERATION
        UI = -(S+S)
        VI = S*S
        GO TO 20
C RESTORE VARIABLES
   50   U = SVU
        V = SVV
        DO 60 I=1,N
          K(I) = SVK(I)
   60   CONTINUE
C TRY QUADRATIC ITERATION IF IT HAS NOT BEEN TRIED
C AND THE V SEQUENCE IS CONVERGING
        IF (VPASS .AND. (.NOT.VTRY)) GO TO 20
C RECOMPUTE QP AND SCALAR VALUES TO CONTINUE THE
C SECOND STAGE
        CALL D8RPLY(NN, U, V, P, QP, A, B)
        CALL D2RPLY(TYPE, K, QK)
   70   OVV = VV
        OSS = SS
        OTV = TV
        OTS = TS
   80 CONTINUE
      RETURN
      END

\name{car_control}
\alias{car_control}
\title{ Parameters for Predict and Numerical Optimization in Kalman Filter }
\description{
  Setup in the predict and numerical optimization in the Kalman filter algorithm.
}
\usage{
car_control(fty=1, n.ahead=10, trace=FALSE, ari=TRUE, vri=FALSE, vr=0, pfi="MAPS",
ccv="CTES", lpv=TRUE, scc=TRUE,  nit=40, opm=1, rgm=1, req=0.5, con=1.0e-5, rpe=1.0, 
ivl=1.0e-2, fac=1.0e1, stl=1.0e-5, sml=1.0e2, gtl=1.0e5, kst=TRUE, fct=TRUE)
}
\arguments{
  \item{fty}{\code{fty=1} forecast past the end. \code{fty=2} forecast
    last L-steps. \code{fty=3} forecast last L-steps updated (filtering)types. See also \code{fct}. If \code{fct=TRUE}, all time series is used to fit the model for \code{fty=1 or 3}. If \code{fty=2}, only the first (length of time - n.ahead) is used to fit the model. Thus, only the first (length of time - n.head) prediction values are the same for \code{fty=1 or 3} vs \code{fty=2}. See also \code{pre2, prv2}.}
  \item{n.ahead}{number of steps ahead at which to predict.}
  \item{trace}{ a logical value triggering printout of information during
                the fitting process, and major results for the fitted model.}
  \item{ari}{ \code{ari=TRUE}: parameter starting values. \code{ari=FALSE}: they are taken as zero. This is obsolete.}
  \item{vri}{ \code{vri=FALSE}, observation noise not included in the
    model. \code{vri=TRUE}, observation noise included}
  \item{vr}{ 0.5, initial value of observation noise ratio: only if
    \code{vri=TRUE}}
  \item{pfi}{ always use the option pfi="MAPS".}
  \item{ccv}{\code{ccv="CTES"} for constant term estimation. \code{ccv="MNCT"} if mean correction, \code{ccv=NULL} if omitted.}
  \item{lpv}{\code{lpv=TRUE} always use this option.}
  \item{scc}{ \code{scc=TRUE} always use this option. }
  \item{nit}{number of iteations. }
  \item{opm}{\code{opm=1} always use this.}
  \item{rgm}{\code{rgm=1} always use this.}
  \item{req}{root equality switch value.}
  \item{con}{convergence criterion.}
  \item{rpe}{relative size of parameter perturbations.}
  \item{ivl}{initial value of step size constraint parameter.}
  \item{fac}{step size constraint modification parameter. This value may
  be setup to \code{fac=5} for better convergency.}
  \item{stl}{typical smallest step size parameter.}
  \item{sml}{typical small step size parametrr.}
  \item{gtl}{typical greatest step size parameter. }
  \item{kst}{\code{kst=TRUE} to save estimated states. }
  \item{fct}{\code{fct=TRUE} to use all time series to fit the model. }
}
\details{

  Objects returned by this function specify predict and numerical optimization parameters of the Kalman filter algorithms implemented in \code{\link{car}},
  (via the \code{ctrl} argument).

}
\value{
  An object of class \code{car_control}, a list.
}
\seealso{\code{\link{car}} for the usage}
\keyword{misc}

\name{factab}
\alias{factab}
\title{Calculate Characteristic Roots and System Frequency}
\description{
Calculate characteristic roots and system frequency from the estimated
reparameterized coefficients of CAR fits.
}
\usage{
factab(object)
}

\arguments{
  \item{object}{a fitted time-series CAR model  }
}

\value{
A table with characteristic roots and frequencies for the corresponding model fit.
}
\references{
  Belcher, J. and Hampton, J. S. and Tunnicliffe Wilson,
  G. (1994). Parameterization of continuous time autoregressive models
  for irregularly sampled time series data. \emph{Journal of the Royal
    Statistical Society, Series B, Methodological},\bold{56},141--155

Jones, Richard H. (1981). Fitting a continuous time autoregression to
discrete data. \emph{Applied Time Series Analysis II}, 651--682

  Wang, Zhu(2004). \emph{The Application of the Kalman Filter to Nonstationary Time Series through Time Deformation}. PhD thesis, Southern Methodist University

 Wang, Zhu (2013). cts: An R Package for Continuous Time Autoregressive Models via Kalman Filter. \emph{Journal of Statistical Software}, \bold{Vol. 53(5)}, 1--19. \url{https://www.jstatsoft.org/v53/i05}
}

\author{
  G. Tunnicliffe Wilson and Zhu Wang
}
\seealso{ \code{\link{car}} and \code{\link{kalsmo}}}
\examples{
\donttest{
data(asth)
(fit <- car(asth,scale=0.25,order=4))
factab(fit)
}
}
\keyword{ ts }

\name{plotSpecLs}
\alias{plotSpecLs}
\title{Plotting Lomb-Scargle Periodogram}
\description{
  Plotting method for objects of class \code{"spec.ls"}.
}
\usage{
plotSpecLs(x, add = FALSE, ci = 0.95, log = c("yes", "dB", "no"), xlab
= "frequency", ylab = NULL, type = "l", main = NULL, sub = NULL, ...)
}
\arguments{
    \item{x}{an object of class \code{"spec.ls"}.}
  \item{add}{logical.  If \code{TRUE}, add to already existing plot.}
  \item{ci}{Coverage probability for confidence interval.  Plotting of
    the confidence bar is omitted unless \code{ci} is strictly
    positive.}
  \item{log}{If \code{"dB"}, plot on log10 (decibel) scale (as S-PLUS),
    otherwise use conventional log scale or linear scale.  Logical
    values are also accepted.  The default is \code{"yes"} unless
    \code{options(ts.S.compat = TRUE)} has been set, when it is
    \code{"dB"}.}
  \item{xlab}{the x label of the plot.}
  \item{ylab}{the y label of the plot.}
  \item{type}{the type of plot to be drawn, defaults to lines.}
  \item{main}{overall title for the plot.}
  \item{sub}{a sub title for the plot.}
  \item{\dots}{Further graphical parameters.}
}

\value{
Called for its side effect of producing a plot. Invisibly returns \code{x} (an object of class \code{"spec.ls"}).
}

\seealso{ \code{\link{spec.ls} }}
\keyword{ hplot }
\keyword{ ts }

\name{V22174}
\alias{V22174}
\docType{data}
\title{ Measurments of Relative Aboundance}
\description{
  Measurments of relative aboundance of an xxygen isotope in an ocean core.
}
\usage{data(V22174)}
\format{
  The format is:
 num [1:164, 1:2]  6.13  8.39 10.64 12.90 15.16 ...
}

\source{
Belcher, J. and Hampton, J. S. and Tunnicliffe Wilson,
  G. (1994). Parameterization of continuous time autoregressive models
  for irregularly sampled time series data. \emph{Journal of the Royal
    Statistical Society, Series B, Methodological},\bold{56},141--155
}
\references{
 Wang, Zhu (2013). cts: An R Package for Continuous Time Autoregressive Models via Kalman Filter. \emph{Journal of Statistical Software}, \bold{Vol. 53(5)}, 1--19. \url{https://www.jstatsoft.org/v53/i05}
}

\examples{
data(V22174)
str(V22174) ; plot(V22174)
}
\keyword{datasets}

\name{kalsmoComp}
\alias{kalsmoComp}
\title{Estimate Componenents with the Kalman Smoother }
\description{
Estimate unobserved components with the Kalman smoother to a fitted CAR model.
}
\usage{
kalsmoComp(x, comp = NULL, plot.it = TRUE, xlab= "time", ylab="",na.action = na.fail, ...)
}
\arguments{
  \item{x}{ the result of estimated components by \code{\link{kalsmo}}. }
  \item{comp}{ a numeric vector from which components are estimaed }
  \item{plot.it}{ plot the component? }
  \item{xlab}{ name of xlab}
  \item{ylab}{ name of ylab}
  \item{na.action}{ how to handle NAs? }
  \item{\dots}{further graphical parameters. }
}

\value{
A component is computed from the estimated components for each root of the
characteristic equation.
}
\references{
Belcher, J. and Hampton, J. S. and Tunnicliffe Wilson,
  G. (1994). Parameterization of continuous time autoregressive models
  for irregularly sampled time series data. \emph{Journal of the Royal
    Statistical Society, Series B, Methodological},\bold{56},141--155

  Wang, Zhu(2004). \emph{The Application of the Kalman Filter to Nonstationary Time Series through Time Deformation}. PhD thesis, Southern Methodist University

 Wang, Zhu (2013). cts: An R Package for Continuous Time Autoregressive Models via Kalman Filter. \emph{Journal of Statistical Software}, \bold{Vol. 53(5)}, 1--19. \url{https://www.jstatsoft.org/v53/i05}
}

\author{
  Zhu Wang
}

\seealso{ \code{\link{kalsmo}}}

\examples{
\donttest{
data(asth)
kalsmoComp(kalsmo(car(asth,scale=0.25,order=4)),c(2,3))
}
}
\keyword{ ts }

\name{car}
\alias{car}
\alias{print.car}
\alias{summary.car}
\alias{plot.car}
\alias{predict.car}
\alias{spectrum}
\alias{spectrum.car}
\alias{tsdiag}
\alias{tsdiag.car}
\alias{AIC.car}
\alias{kalsmo}
\alias{kalsmo.car}
\title{Fit Continuous Time AR Models to Irregularly Sampled Time Series}
\description{
  Fit a continuous AR model to an irregularly sampled univariate time series with the Kalman filter
}
\usage{
car(x, y=NULL, scale = 1.5, order = 3, ctrl=car_control())
\method{print}{car}(x, digits = 3, ...)
\method{summary}{car}(object, ...)
\method{plot}{car}(x, type=c("spec", "pred", "diag"),...) 
\method{predict}{car}(object, se.fit = TRUE, digits = 3, plot.it=TRUE,...)
\method{spectrum}{car}(object, frmult=1, n.freq, plot.it = TRUE, na.action = na.fail, ...)
\method{AIC}{car}(object, ..., verbose=TRUE, k=2)
\method{tsdiag}{car}(object, gof.lag = 10, ...)
\method{kalsmo}{car}(object)
}
\arguments{
  \item{x}{two column data frame or matrix with the first column
    being the sampled time and the second column being the observations at
    the first column; otherwise \code{x} is a numeric vector of sampled time. It can be a \code{car} object for S3 methods}
  \item{y}{not used if \code{x} has two columns; otherwise \code{y} is a numeric vector of observations at sampled time \code{x}.}
  \item{scale}{ The kappa value referred to in the paper by Belcher et a. (1994). We now recommend selection of kappa along with the model order by using AIC. Also, it is suggested to choose kappa close to 2pi times 1/mean.delta (reciprocal of the mean time between observations), though it is a good idea to explore somewhat lower and higher values to see whether the spectrum estimates were sensitive to this choice. Choosing kappa lower increases the risk of trying to estimate the spectrum beyond the effective Nyquist frequency of the data - though this does depend on the distribution of intersample times. }
  \item{order}{order of autoregression.}
  \item{ctrl}{ control parameters used in predict and numerical optimization. }
  \item{object}{ object of class \code{car} }
  \item{type}{ in \code{plot} a character indicating the type of plot. type="spec", call \code{spec};
                 type="pred", call \code{predict};
                 type="diag", call \code{diag}}
  \item{se.fit}{ Logical: should standard errors of prediction be returned?}
  \item{digits}{ return value digits }
  \item{plot.it}{ Logical: plot the forecast values? }
  \item{gof.lag}{the maximum number of lags for a Portmanteau
    goodness-of-fit test}
  \item{frmult}{ numerical value, can be used to multiply the frequency range}
  \item{n.freq}{number of frequency}
  \item{verbose}{logical; if \code{TRUE}, prints the model selection table using \code{\link{message}}.}
  \item{k}{numeric penalty per parameter (as in \code{\link[stats]{AIC}}); default is 2.}
  \item{na.action}{ \code{NA} action function.}
  \item{\dots}{further arguments to be passed to particular methods}
}
\details{
\code{spectrum} returns (and by default plots) the spectral density of the fitted model.

\code{tsdiag} is a generic diagnostic function for continuous AR model. It will generally plot the residuals,
  often standadized, the autocorrelation function of the residuals, and
  the p-values of a Portmanteau test for all lags up to \code{gof.lag}.
  The method for \code{\link{car}} object plots residuals scaled by the estimate of their (individual) variance, and use the Ljung--Box version of the portmanteau test.

\code{AIC} For continuous CAR model selection, t-statistic and AIC are calculated
based on reparameterized coefficients \code{phi} and covariance matrix
\code{ecov}. From the t-statistic, the final model is chosen such that
if the true model \code{order} is less than the large value used for
model estimation then for \code{i > order} the deviations of the
estimated parameters \code{phi} from their true value of 0 will be
small. From the AIC, the final model is chosen based on the smallest AIC
value. A table with t-statistic and AIC for the corresponding model order.

\code{factab} calculate characteristic roots and system frequency from the estimated
reparameterized coefficients of CAR fits.

\code{smooth} computes components corresponding to the diagonal transition matrix with the Kalman smoother. This may not be stable for some data due to numerical inversion of matrix.

}
\value{
  A list of class \code{"car"} with the following elements:
  \item{n.used}{The number of observations used in fitting}
  \item{order }{The order of the fitted model. This is chosen by the user.}
  \item{np }{The number of parameters estimated. This may include the mean and the observation noise ratio.}
  \item{scale}{The kappa value referred to in the paper of Belcher et al.}
  \item{vri}{If vri=1, estimate the observation noise ratio.}
  \item{vr}{The estimated observation noise ratio.}
  \item{sigma2}{The estimated innovation variance.}
  \item{phi}{The estimated reparameterized autoregressive parameters.}
  \item{x.mean}{The estimated mean of the series used in fitting and for use in prediction.}
  \item{b}{All estimated parameters, which include \code{phi}, and possibly \code{x.mean} and \code{vr}.}
  \item{delb}{The estimated standard error of \code{b}}
  \item{essp}{The estimated correlation matrix of \code{b}}
  \item{ecov}{The estimated covariance matrix of \code{phi}. See also
    \code{\link{AIC}}}
  \item{rootr}{The real part of roots of \code{phi}. See also \code{\link{AIC}}}
  \item{rooti}{The imaginary part of roots of \code{phi}. See also
    \code{\link{AIC}}}
  \item{tim}{The numeric vector of sampled time.}
  \item{ser}{The numeric vector of observations at sampled time
    \code{tim}.}
  \item{filser}{The filtered time series with the Kalman filter.}
  \item{filvar}{The estimated variance of Kalman filtered time series
    \code{filser}}
  \item{sser}{The smoothed time series with the Kalman smoother.}
  \item{svar}{The estimated variance of smoothed time series
    \code{sser}}
  \item{stdred}{The standardized residuals from the fitted model.}
  \item{pretime}{Time of predictions.}
  \item{pred}{Predictions for the \code{pretime}.}
  \item{prv}{Prediction variance of \code{pred}}
  \item{pre2}{Fitted values including \code{pred} for all the time series. See also \code{fty}.}
  \item{prv2}{Variance of fitted values including \code{prv} for all the time series. See also \code{fty}.}
  \item{fty}{Forecast type}
  \item{tnit}{Numeric vector: iteration}
  \item{ss}{Numeric vector: sum of squares for each \code{tnit}}
  \item{bit}{Matrix with rows for \code{tnit} and columns for parameter estimates}
  \item{aic}{AIC value for the fitted model}
  \item{bic}{BIC value for the fitted model}
}
\references{
  Belcher, J. and Hampton, J. S. and Tunnicliffe Wilson,
  G. (1994). Parameterization of continuous time autoregressive models
  for irregularly sampled time series data. \emph{Journal of the Royal
    Statistical Society, Series B, Methodological},\bold{56},141--155
  
  Jones, Richard H. (1981). Fitting a continuous time autoregression to
  discrete data. \emph{Applied Time Series Analysis II}, 651--682

  Wang, Zhu (2004). \emph{The Application of the Kalman Filter to Nonstationary Time Series through Time Deformation}. PhD thesis, Southern Methodist University

  Wang, Zhu and Woodward, W. A. and Gray, H. L. (2009). The Application of the Kalman Filter to Nonstationary Time Series through Time Deformation. \emph{Journal of Time Series Analysis}, \bold{30(5)}, 559-574.

  Wang, Zhu (2013). cts: An R Package for Continuous Time Autoregressive Models via Kalman Filter. \emph{Journal of Statistical Software}, \bold{Vol. 53(5)}, 1--19. \url{https://www.jstatsoft.org/v53/i05}
}

\author{
  G. Tunnicliffe Wilson and Zhu Wang
}
\seealso{\code{\link{car_control}} for predict and numerical optimization parameters, and \code{\link{AIC}} for model selection}
\examples{
\donttest{
data(V22174)
fit <- car(V22174,scale=0.2,order=7, ctrl=car_control(trace=TRUE))
summary(fit)
spectrum(fit)
tsdiag(fit)
AIC(fit)
factab(fit)
###fitted values vs observed values
ntim <- dim(V22174)[1]
plot(V22174[,1], V22174[,2], type="l")
points(V22174[,1], fit$pre2[1:ntim], col="red")
### alternatively
fit2 <- car(V22174,scale=0.2,order=7, ctrl=car_control(fty=3))
plot(V22174[,1], V22174[,2], type="l")
points(V22174[,1], fit2$pre2, col="red")

data(asth)
fit <- car(asth,scale=0.25,order=4, ctrl=car_control(n.ahead=10))
kalsmo(fit)
}
}
\keyword{ts}

\name{spec.ci}
\alias{spec.ci}
\title{Internal Function}
\description{Internal Function}
\usage{
spec.ci(spec.obj, coverage = 0.95)
}
\arguments{
  \item{spec.obj}{An object with component \code{df} (degrees of freedom), for example as returned by \code{\link{spec.ls}}.}
  \item{coverage}{Coverage probability for the pointwise confidence interval, in \code{[0,1)}.}
}
\value{A numeric vector of length 2 giving multiplicative factors \code{c(lower, upper)} for pointwise confidence limits for a spectrum estimate.}
\details{This is a small utility used by \code{\link{plotSpecLs}} to draw confidence bars.}
\keyword{internal}

\name{plotSpecCar}
\alias{plotSpecCar}

\title{Plotting Spectral Densities }
\description{
 Internal function used in spectrum.car only. Not called by user.
 Plotting method for objects of class \code{"spectrum.car"}.}
\usage{
plotSpecCar(x, add = FALSE, ci = 0.95, log = "dB", xlab = "frequency",
ylab = NULL, type = "l", main = NULL, sub = NULL,...)
}

\arguments{
    \item{x}{an object of class \code{"spectrum.car"}.}
  \item{add}{logical.  If \code{TRUE}, add to already existing plot.}
  \item{ci}{Coverage probability for confidence interval.  Plotting of
    the confidence bar is omitted unless \code{ci} is strictly
   positive.}
  \item{log}{If \code{"dB"}, plot on log10 (decibel) scale (as S-PLUS),
    otherwise use conventional log scale or linear scale.  Logical
    values are also accepted.  The default is \code{"yes"} unless
    \code{options(ts.S.compat = TRUE)} has been set, when it is
    \code{"dB"}.}
  \item{xlab}{the x label of the plot.}
  \item{ylab}{the y label of the plot.}
  \item{type}{the type of plot to be drawn, defaults to lines.}
  \item{main}{overall title for the plot.}
  \item{sub}{a sub title for the plot.}
  \item{\dots}{further graphical parameters.}
}
\value{plot of spectral density from continuous time autoregressive model}
\references{
  Belcher, J. and Hampton, J. S. and Tunnicliffe Wilson,
  G. (1994). Parameterization of continuous time autoregressive models
  for irregularly sampled time series data. \emph{Journal of the Royal
    Statistical Society, Series B, Methodological},\bold{56},141--155
  
  Jones, Richard H. (1981). Fitting a continuous time autoregression to
  discrete data. \emph{Applied Time Series Analysis II}, 651--682

  Wang, Zhu (2004). \emph{The Application of the Kalman Filter to Nonstationary Time Series through Time Deformation}. PhD thesis, Southern Methodist University

 Wang, Zhu (2013). cts: An R Package for Continuous Time Autoregressive Models via Kalman Filter. \emph{Journal of Statistical Software}, \bold{Vol. 53(5)}, 1--19. \url{https://www.jstatsoft.org/v53/i05}
}

\author{
  G. Tunnicliffe Wilson and Zhu Wang
}
\seealso{ \code{\link{spectrum.car}}}

\keyword{ hplot }
\keyword{ ts }

\name{spec.ls}
\alias{spec.ls}
\title{ Estimate Spectral Density of an Irregularly Sampled Time Series by a Smoothed
    Periodogram}
\description{
The most commonly used method of computing the spectrum on unevenly
spaced time series is periodogram analysis, see Lomb (1975) and Scargle
(1982). The Lomb-Scargle method for unevenly spaced data is known to be
a powerful tool to find, and test significance of, weak perriodic
signals. The Lomb-Scargle periodogram possesses the same statistical
properties of standard power spectra.
}
\usage{
spec.ls(x, y=NULL, spans = NULL, kernel = NULL, taper = 0.1, pad = 0,
fast = TRUE, type = "lomb",demean = FALSE, detrend = TRUE, plot.it = TRUE,
na.action = na.fail, ...)
}
\arguments{
  \item{x}{two column data frame or matrix with the first column
    being the sampled time and the second column being the observations at
    the first column; otherwise \code{x} is a numeric vector of sampled time.}
  \item{y}{not used if \code{x} has two columns; otherwise \code{y} is a numeric vector of observations at sampled time \code{x}.the time at which \code{x} is observed}
  \item{spans}{vector of odd integers giving the widths of modified
    Daniell smoothers to be used to smooth the periodogram.}
  \item{kernel}{alternatively, a kernel smoother of class
    \code{"tskernel"}.}
  \item{taper}{proportion of data to taper.  A split cosine bell taper
    is applied to this proportion of the data at the beginning and
    end of the series.}
  \item{pad}{proportion of data to pad. Zeros are added to the end of
    the series to increase its length by the proportion \code{pad}.}
  \item{fast}{logical; if \code{TRUE}, pad the series to a highly composite
    length.}
  \item{type}{Lomb-Scargle spectrum of Fourier transformation spectrum}
  \item{demean}{logical. If \code{TRUE}, subtract the mean of the
    series.}
  \item{detrend}{logical. If \code{TRUE}, remove a linear trend from
    the series. This will also remove the mean.}
  \item{plot.it}{plot the periodogram?}
  \item{na.action}{\code{NA} action function.}
  \item{\dots}{graphical arguments passed to \code{plotSpecLs}.}
}

\details{
  The raw Lomb-Scargle periodogram for irregularly sampled time series is not a consistent estimator of the spectral density,
  but adjacent values are asymptotically independent. Hence a consistent
  estimator can be derived by smoothing the raw periodogram, assuming that
  the spectral density is smooth.

  The series will be automatically padded with zeros until the series
  length is a highly composite number in order to help the Fast Fourier
  Transform. This is controlled by the \code{fast} and not the \code{pad}
  argument.

  The periodogram at zero is in theory zero as the mean of the series
  is removed (but this may be affected by tapering): it is replaced by
  an interpolation of adjacent values during smoothing, and no value
  is returned for that frequency.
}
\value{
  A list object of class \code{"spec.ls"} with the following additional components:
  \item{kernel}{The \code{kernel} argument, or the kernel constructed
    from \code{spans}.}
  \item{df}{The distribution of the spectral density estimate can be
    approximated by a chi square distribution with \code{df} degrees
    of freedom.}
  \item{bandwidth}{The equivalent bandwidth of the kernel smoother as
    defined by Bloomfield (1976, page 201).}
  \item{taper}{The value of the \code{taper} argument.}
  \item{pad}{The value of the \code{pad} argument.}
  \item{detrend}{The value of the \code{detrend} argument.}
  \item{demean}{The value of the \code{demean} argument.}

  The result is returned invisibly if \code{plot.it} is true.
}
\references{
Bloomfield, P. (1976) \emph{Fourier Analysis of Time Series: An
  Introduction.}Wiley.

Lomb, N. R. (1976) Least-squares frequency-analysis of unequally spaced data.
  \emph{Astrophysics and Space Science},
  \bold{39},447-462

W. H. Press and S. A. Teukolsky and W. T. Vetterling and B.P. Flannery (1992)
\emph{Numerical Recipes in C: The Art of Scientific Computing.}, Cambridge University Press,
second edition.

Scargle, J.D. (1982) Studies in astronomical time series analysis II. Statistical aspects of spectral analysis of unevenly spaced data,
\emph{The Astrophysical Journal},
\bold{263}, volume 2, 835-853.
}

\author{Lomb-Scargle periodogram by Zhu Wang
 }
\note{This is 'slow' program and a fast program may use FFT, see (Press
  et al, 1992) }

\seealso{ \code{\link{spec.taper}},
  \code{\link{plotSpecLs}}, \code{\link{fft}}}

\examples{
data(V22174)
spec.ls(V22174)

data(asth)
spec.ls(asth)
}
\keyword{ ts }

\name{asth}
\alias{asth}
\docType{data}
\title{Measurements of The Lung Function}
\description{
Measurements of the lung function of an asthma patient.
}
\usage{data(asth)}
\format{
  The format is:
 num [1:209, 1:2] 8 10 12 14 16 18 20 22 32 34 ...
}
\details{
These are made by the patients mostly at 2h time intervals but with
irregular gaps in this record.
}
\source{
Belcher, J. and Hampton, J. S. and Tunnicliffe Wilson,
  G. (1994). Parameterization of continuous time autoregressive models
  for irregularly sampled time series data. \emph{Journal of the Royal
    Statistical Society, Series B, Methodological},\bold{56},141--155
}

\examples{
data(asth)
str(asth) ; plot(asth)
}
\keyword{datasets}
